@@ -97,8 +97,8 @@ vertex_id v_id; /* vertex for DETECT mode, or passing on to constraint eval */
 
   if ( concount > SDIM ) 
   { sprintf(errmsg,
-     "Trying to project vertex %d on more constraints, %d, than space dimensions, %d.",
-     ordinal(v_id)+1,concount,SDIM);  
+     "Trying to project vertex %s on more constraints, %d, than space dimensions, %d.",
+     ELNAME(v_id),concount,SDIM);  
      if ( conlist )
      { sprintf(errmsg+strlen(errmsg),"    Projecting on constraints ");
         for ( i = 0 ; i < concount ; i++ )
@@ -153,7 +153,7 @@ do_over:
        }
        goto do_over;
      }
- 
+
             
   /* form combination */
   for ( i = 0 ; i < SDIM ; i++ )
@@ -163,7 +163,90 @@ do_over:
   }
   
   return concount;
-}
+} /* end constr_proj */
+
+
+/******************************************************************
+*
+*  Function: constr_proj_matrix()
+*
+*  Purpose:  Find matrix for projection onto constraint tangents.
+*
+*  Explanation:  Projecting a point or vector onto constraints
+*      requires finding an appropriate linear combination of
+*      constraint gradients c[i]*grad(f[i]) where c[i] = (inv A)*B,
+*      where B is an appropriate vector (input) and 
+*                A[i][j] = <grad(f[i]),grad(f[j])>.
+*/
+
+
+int constr_proj_matrix(v_id,mat)
+vertex_id v_id; /* vertex for DETECT mode, or passing on to constraint eval */
+REAL **mat;  /* projection matrix */
+{
+/* maximum number of hit constraints */
+#define MAXCONSTR MAXCOORD
+  int i,j;
+  MAT2D(grad,MAXCONSTR,MAXCOORD);     /* gradients of constraints */
+  MAT2D(c,MAXCONSTR,MAXCONSTR);                      /* combination coefficients */
+  REAL fval;                                 /* value of constraint */
+  MAT2D(a,MAXCONSTR,MAXCONSTR);        /* matrix for projection */
+  int concount=0;
+  conmap_t * conmap = get_v_constraint_map(v_id);
+  struct constraint *constr[MAXCONSTR];
+  for ( j = 1 ; j <= (int)conmap[0] ; j++ )
+   if ( conmap[j] & CON_HIT_BIT )
+     constr[concount++] = get_constraint(conmap[j]);
+
+  if ( concount > SDIM ) 
+  { sprintf(errmsg,
+     "Vertex %s has more constraints, %d, than space dimensions, %d.",
+        ELNAME(v_id),concount,SDIM);  
+ 
+     sprintf(errmsg+strlen(errmsg),"    Projecting on constraints ");
+     for ( i = 0 ; i < concount ; i++ )
+        sprintf(errmsg+strlen(errmsg)," %s",constr[i]->name);
+     strcat(errmsg,".  Maybe redundant one-sided constraints?");
+     
+     strcat(errmsg,"\n");
+     kb_error(3515,errmsg,RECOVERABLE);
+  }
+
+  if ( concount > 0 )
+  { REAL *coord = get_coord(v_id);
+  
+    /* calculate gradients */
+    for ( i = 0 ; i < concount ; i++ )     
+      eval_all(constr[i]->formula,coord,SDIM,&fval,grad[i],v_id);
+  
+    /* construct matrix A */  
+    for ( i = 0 ; i < concount ; i++ )     
+      for ( j = 0 ; j < concount ; j++ )
+        a[i][j] = SDIM_dot(grad[i],grad[j]);
+  
+    /* invert */
+    mat_inv(a,concount);
+  
+    /* combination coefficients */
+    mat_mult(a,grad,c,concount,concount,SDIM);
+    tr_mat_mul(grad,c,mat,concount,SDIM,SDIM);
+  
+    /* form combination */
+    for ( i = 0 ; i < SDIM ; i++ )
+    { for ( j = 0 ; j < SDIM ; j++ )
+        mat[i][j] *= -1;
+      mat[i][i] += 1.0;
+    }
+  }
+  /* else identity */
+  else
+  { for ( i = 0 ; i < SDIM ; i++ )
+      for ( j = 0 ; j < SDIM ; j++ )
+        mat[i][j] = (i==j) ? 1.0 : 0.0;
+  }
+
+  return concount;
+} /* end constr_proj_matrix */
 
 /******************************************************************
 *
@@ -235,8 +318,8 @@ int mode; /* TEST_MOVE or ACTUAL_MOVE */
   if ( (itercount >= MAXCONITER) && (mode == ACTUAL_MOVE) )
   { struct constraint *con = get_constraint(dcon);
     sprintf(msg,
-  "Vertex %d doesn't converge to constraint %s after %d iterations. \n maxdiff = %g, constraint_tolerance %g\n",
-         ordinal(v_id)+1,con->name,MAXCONITER,(DOUBLE)totaldiff,web.tolerance);
+  "Vertex %s doesn't converge to constraint %s after %d iterations. \n maxdiff = %g, constraint_tolerance %g\n",
+         ELNAME(v_id),con->name,MAXCONITER,(DOUBLE)totaldiff,web.tolerance);
      kb_error(1792,msg,WARNING);
   }
 
@@ -268,7 +351,7 @@ vertex_id v_id;
   int sign;
   REAL fval,deriv[MAXCOORD];
 
-  int_val = get_original(v_id);  /* for eval  of file parameters */
+  int_val = ordinal(get_original(v_id))+1;  /* for eval  of file parameters */
   f = get_force(v_id);
   coord = get_coord(v_id);
   if ( get_vattr(v_id) & NEGBOUNDARY ) sign = -1;
@@ -309,7 +392,7 @@ edge_id e_id;
   vertex_id headv,tailv;
   conmap_t *conmap;
 
-  int_val = get_original(e_id);  /* for eval  of file parameters */
+  int_val = ordinal(get_original(e_id))+1;  /* for eval  of file parameters */
   if ( web.modeltype == QUADRATIC )
   { constr_edge_force_q(e_id);
     return;
@@ -369,15 +452,15 @@ vertex_id v_id;
   conmap_t *conmap = get_v_constraint_map(v_id);
   struct constraint *constr;
 
-  int_val = get_original(v_id);  /* for eval  of file parameters */
+  int_val = ordinal(get_original(v_id))+1;  /* for eval  of file parameters */
   for ( j = 1 ;  j <= (int)conmap[0] ; j++ )
   { constr = get_constraint(conmap[j]);
     if ( !(constr->attr & CON_ENERGY) || (constr->compcount != 1) ) continue;
     e = eval(constr->envect[0],get_coord(v_id),v_id,NULL);  
     if ( get_vattr(v_id) & NEGBOUNDARY )
-      web.total_energy -= e;
+      binary_tree_add(web.total_energy_addends,-e);
     else
-      web.total_energy += e;
+      binary_tree_add(web.total_energy_addends,e);
   }
 }
 
@@ -403,7 +486,7 @@ edge_id e_id;
   vertex_id headv,tailv;
   conmap_t *conmap;
 
-  int_val = get_original(e_id);  /* for eval  of file parameters */
+  int_val = ordinal(get_original(e_id))+1;  /* for eval  of file parameters */
   if ( web.modeltype == QUADRATIC )
   { constr_edge_energy_q(e_id);
     return;
@@ -440,7 +523,7 @@ edge_id e_id;
 
   }
 
-  web.total_energy += energy;
+  binary_tree_add(web.total_energy_addends,energy);
 
 }
 
@@ -467,7 +550,7 @@ vertex_id v_id;
 
   if ( !valid_id(e_id) ) return;  /* not on a cell */
 
-  int_val = get_original(v_id);  /* for eval  of file parameters */
+  int_val = ordinal(get_original(v_id))+1;  /* for eval  of file parameters */
   conmap = get_v_constraint_map(v_id); /* only hit constraints */
   for ( j = 1 ; j <= (int)conmap[0] ; j++ )
   { if ( !(conmap[j] & CON_HIT_BIT) ) continue;
@@ -491,6 +574,7 @@ vertex_id v_id;
     f_id = get_fe_facet(fe_id);
     if ( valid_id(f_id) ) 
     { b_id = get_facet_body(f_id);
+      add_facet_area(f_id,-e);
       if ( valid_id(b_id) )
         add_body_volume(b_id,-e);
       b_id = get_facet_body(inverse_id(f_id));
@@ -505,6 +589,7 @@ vertex_id v_id;
     f_id = get_fe_facet(ffe);
     if ( valid_id(f_id) ) 
     { b_id = get_facet_body(f_id);
+      add_facet_area(f_id,-e);
       if ( valid_id(b_id) )
         add_body_volume(b_id,-e);
       b_id = get_facet_body(inverse_id(f_id));
@@ -549,7 +634,7 @@ edge_id e_id;
         RECOVERABLE);
 
 
-  int_val = get_original(e_id);  /* for eval  of file parameters */
+  int_val = ordinal(get_original(e_id))+1;  /* for eval  of file parameters */
   conmap = get_e_constraint_map(e_id); 
   if ( get_eattr(e_id) & NEGBOUNDARY ) sign = -1;
   else sign = 1;
@@ -654,8 +739,8 @@ edge_id e_id;
     ff = SDIM_dot(grad,grad);
     if ( ff <= 0.0 )
     { sprintf(errmsg,
-   "Gap energy error: Vertex %d is on a convex constraint at zero gradient.\n",
-             ordinal(tail)+1);
+   "Gap energy error: Vertex %s is on a convex constraint at zero gradient.\n",
+             ELNAME(tail));
        kb_error(2001,errmsg,WARNING);
        ff = 1.0;
     }
@@ -667,8 +752,8 @@ edge_id e_id;
     ff = SDIM_dot(grad,grad);
     if ( ff <= 0.0 )
     { sprintf(errmsg,
-  "Gap energy error: Vertex %d is on a convex constraint at zero gradient.\n",
-             ordinal(head)+1);
+  "Gap energy error: Vertex %s is on a convex constraint at zero gradient.\n",
+             ELNAME(head));
       kb_error(2002,errmsg,WARNING);
       ff = 1.0;
     }
@@ -691,7 +776,7 @@ edge_id e_id;
 #endif
 
   sprenergy *= web.spring_constant;
-  web.total_energy += sprenergy;
+  binary_tree_add(web.total_energy_addends,sprenergy);
   web.spring_energy += sprenergy;
 }
 
@@ -844,7 +929,7 @@ edge_id e_id;
   int sign;
   REAL gpt[MAXCOORD];
           
-  int_val = get_original(e_id);  /* for eval  of file parameters */
+  int_val = ordinal(get_original(e_id))+1;  /* for eval  of file parameters */
   conmap = get_e_constraint_map(e_id);
   get_edge_side(e_id,side);
   if ( get_eattr(e_id) & NEGBOUNDARY ) sign = -1;
@@ -881,7 +966,7 @@ edge_id e_id;
     }
   }
      
-  web.total_energy += energy;
+  binary_tree_add(web.total_energy_addends,energy);
 
 }
  
@@ -913,7 +998,7 @@ edge_id e_id;
   REAL gpt[MAXCOORD];
   REAL *force[EDGE_CTRL];
           
-  int_val = get_original(e_id);  /* for eval  of file parameters */
+  int_val = ordinal(get_original(e_id))+1;  /* for eval  of file parameters */
   conmap = get_e_constraint_map(e_id);
   get_edge_side(e_id,side);
   if ( get_eattr(e_id) & NEGBOUNDARY ) sign = -1;
@@ -991,7 +1076,7 @@ edge_id e_id;
   facetedge_id fe_id = get_edge_fe(e_id);
   facetedge_id first_fe;
           
-  int_val = get_original(e_id);  /* for eval  of file parameters */
+  int_val = ordinal(get_original(e_id))+1;  /* for eval  of file parameters */
   conmap = get_e_constraint_map(e_id);
   get_edge_side(e_id,side);
   if ( get_eattr(e_id) & NEGBOUNDARY ) sign = -1;
@@ -1083,7 +1168,7 @@ edge_id e_id;
   facet_id f_id;
   facetedge_id fe,start_fe;
 
-  int_val = get_original(e_id);  /* for eval  of file parameters */
+  int_val = ordinal(get_original(e_id))+1;  /* for eval  of file parameters */
   /* get basic edge data */          
   conmap = get_e_constraint_map(e_id);
   if ( get_eattr(e_id) & NEGBOUNDARY ) sign = -1;
