@@ -180,7 +180,7 @@ char *fname;
        graph_end = painter_end;
        need_bounding_box = 1;
        edgewidths = pswidths;
-       graph_capabilities = GC_ARCS;
+       graph_capabilities = web.torus_display_period ? 0 : GC_ARCS;
 
        /* do output */
        graphgen();
@@ -481,9 +481,6 @@ char *expr;
     MAT2D(temp_mat,MAXCOORD+1,MAXCOORD+1);
     int no_identity = 0;
 
-   
-     
-
     if ( expr[0] == 0 ) /* empty transform string */
     { stacktop = 1 ; stack[1].count = 0; 
       stack[stacktop].mats = dmatrix3(1,SDIM+1,SDIM+1);
