@@ -10,9 +10,22 @@
 *  Contents: defines for expression parsing and evaluation.
 */
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* node types, numbered above 1512 to avoid yacc token numbers */
 #define GEN_ERROR_TOKEN 10000
 
+#define V_AUTOCHOP_LENGTH 2286
+#define QUIETLOAD_     2285
+#define LITTLE_ENDIAN_ 2284
+#define BIG_ENDIAN_    2283
+#define BINARY_PRINTFHEAD_ 2282
+#define V_FACET_REVERSE_COUNT 2281
+#define V_EDGE_REVERSE_COUNT 2280
+#define V_THIS_MACHINE 2279
+#define POP_ENJOIN_ 2278
 #define V_WINDOW_ASPECT_RATIO 2277
 #define PRINT_SINGLE_LETTER_ 2276
 #define PDELTA_LVALUE_   2275
@@ -734,7 +747,7 @@ struct treenode
             int bdry_id;           /* number of boundary */
             int toggle_id;
             int toggle_state;      /* ON_ or OFF_        */ 
-            int bool;              /* 0 or 1             */
+            int bool_val;          /* 0 or 1             */
             int intpow;            /* integer power      */
             int wherecount;
             int userfunc;
@@ -793,6 +806,10 @@ struct treenode
 #define HAS_STRING    0x20
 #define EPHEMERAL     0x40  /* refers to a nonpermanent name */
 #define PERMNODE      0x80  /* part of permanent command */
+#define HAS_LOCALLIST    0x100
+#define HAS_STRING_5  0x200
+#define DEALLOCATE_POINTER 0x400
+#define IN_ELEMENT_LOOP  0x800
 
 /* for some bit packing */
 #define ESHIFT 12
@@ -806,4 +823,10 @@ struct eval_frame {  /* for access to parent eval's */
    struct expnode *base_ex;     /* for permload kludge */
    struct eval_frame *parent;   /* up the frame chain */
    REAL *stacktop;              /* top of caller's stack, for passing args */
+   int  depth;                  /* to guard against infinite recursion */
+   int  in_element_loop;        /* whether in element loop */
 };
+
+#ifdef __cplusplus
+}
+#endif
