@@ -266,7 +266,7 @@ char *p; /* destination */
 * purpose: Convert node of parse tree to ASCII, recursively doing sons.
 */
 
-void exprint_recur(node,prec_parent)
+void exprint_recur(node,prec_parent) 
 struct treenode *node;
 int prec_parent;  /* precedence level of parent, for parenthesizing */
 {
@@ -369,8 +369,21 @@ int prec_parent;  /* precedence level of parent, for parenthesizing */
          sprintf(pos,"])"); pos+=2;
          return;
    
+      case MATRIX_INVERSE_:
+         sprintf(pos,"matrix_inverse(%s,%s)",
+            globals(node->op1.name_id)->name,globals(node->op2.name_id)->name);
+         pos += strlen(pos);
+         return;
+
+      case MATRIX_MULTIPLY_:
+         sprintf(pos,"matrix_multiply(%s,%s,%s)",
+            globals(node->op1.name_id)->name,globals(node->op2.name_id)->name,
+            globals(node->op3.name_id)->name);
+         pos += strlen(pos);
+         return;
+
       case MERGE_VERTEX_:
-         sprintf(pos,"vertex_merg("); pos += strlen(pos);
+         sprintf(pos,"vertex_merge("); pos += strlen(pos);
          exprint_recur(node+node->left,prec_parent);
          sprintf(pos,","); pos++;
          exprint_recur(node+node->right,prec_parent);
@@ -485,7 +498,14 @@ int prec_parent;  /* precedence level of parent, for parenthesizing */
          sprintf(pos,"%s",globals(node->op2.name_id)->name);
          pos += strlen(pos);
          exprint_recur(node+node->left,prec_parent);
-         strcat(pos," := "); pos += 4;
+         switch ( node->op1.assigntype )
+         { case ASSIGN_: sprintf(pos," := "); break;
+           case PLUSASSIGN_: sprintf(pos," += "); break;
+           case SUBASSIGN_: sprintf(pos," -= "); break;
+           case MULTASSIGN_: sprintf(pos," *= "); break;
+           case DIVASSIGN_: sprintf(pos," /= "); break;
+         } 
+         pos += strlen(pos);
          exprint_recur(node+node->right,prec_parent);
          break;
 
@@ -588,7 +608,15 @@ int prec_parent;  /* precedence level of parent, for parenthesizing */
          else exprint_recur(node+node->left,prec_parent);
          return;
 
+      case BINARY_PRINTFHEAD_:
+         sprintf(pos,"binary_printf ");
+         pos += strlen(pos);
+         if ( node->op1.string ) print_quote(node->op1.string);
+         else exprint_recur(node+node->left,prec_parent);
+         return;
+
       case PRINTF_:
+      case BINARY_PRINTF_:
       case ERRPRINTF_:
       case SPRINTF_:
          exprint_recur(node+node->left,prec_parent);
@@ -652,6 +680,7 @@ int prec_parent;  /* precedence level of parent, for parenthesizing */
          exprint_recur(node+node->left,prec_parent); /* IF part */
          if ( node->right )
            { sprintf(pos," else "); pos += strlen(pos);
+             newline();
              exprint_recur(node+node->right,prec_parent); /* command */
            }
          bracket_depth--;
@@ -674,8 +703,11 @@ int prec_parent;  /* precedence level of parent, for parenthesizing */
          exprint_recur(node+node->left,prec_parent); /* test part */
          sprintf(pos," do "); pos += strlen(pos);
          bracket_depth++;
-         newline();
-         exprint_recur(node+node->right,prec_parent); /* command */
+         if ( node->right )
+         { newline();
+           exprint_recur(node+node->right,prec_parent); /* command */
+         } else
+         { strcat(pos," ;"); pos += strlen(pos); }
          bracket_depth--;
          return;
 
@@ -707,7 +739,9 @@ int prec_parent;  /* precedence level of parent, for parenthesizing */
          sprintf(pos," ) "); pos += strlen(pos);
          bracket_depth++;
          newline();
-         exprint_recur(node+node->right,prec_parent); /* command3 */
+         if ( node->right )
+           exprint_recur(node+node->right,prec_parent); /* command3 */
+         else { strcat(pos," ;") ; pos += strlen(pos); } /* empty command3 */
          bracket_depth--;
          newline();
          return;
@@ -773,36 +807,39 @@ int prec_parent;  /* precedence level of parent, for parenthesizing */
          pos += strlen(pos);
          return;
 
-      case SHADING_: case FACET_COLORS_: case BOUNDARY_CURVATURE_:
-      case KUSNER_: case ESTIMATE_: case DETURCK_: case HOMOTHETY_:
-      case SQGAUSS_: case AUTOPOP_: case AUTOCHOP_: case QUIET_:
-      case OLD_AREA_: case APPROX_CURV_: case RUNGE_KUTTA_:
-      case CHECK_INCREASE_:  case DEBUG_: case MEAN_CURV_: 
-      case ASSUME_ORIENTED_: case RIBIERE_CG_: case HESSIAN_QUIET_:
-      case DIFFUSION_: case GRAVITY_: case CONJ_GRAD_: case TRANSFORMS_:
-      case CONF_EDGE_SQCURV_: case EFFECTIVE_AREA_: case AREA_FIXED_:
-      case RAW_CELLS_: case CONNECTED_CELLS_: case CLIPPED_CELLS_:
-      case THICKEN_: case SHOW_INNER_: case SHOW_OUTER_: case COLORMAP_: 
-      case POST_PROJECT_: case HESSIAN_DIFF_: case OPTIMIZE_:
-      case NORMAL_CURVATURE_: case DIV_NORMAL_CURVATURE_: case QUIETGO_:
-      case NORMAL_MOTION_: case PINNING_:  case VIEW_4D_: case MEMDEBUG_:
-      case METRIC_CONVERSION_: case AUTORECALC_: case GV_BINARY_:
-      case SELF_SIMILAR_: case AUTODISPLAY_: case FORCE_POS_DEF_:
-      case JIGGLE_TOGGLE_: case H_INVERSE_METRIC_: case BEZIER_BASIS_:
-      case BUNCH_KAUFMAN_: case YSMP_: case QUANTITIES_ONLY_:
-      case METRIC_CONVERT_: case LINEAR_METRIC_: case SQUARED_GRADIENT_:
-      case HESSIAN_DOUBLE_NORMAL_: case HESSIAN_NORMAL_: 
-      case HESSIAN_NORMAL_ONE_: case PSCOLORFLAG_: case GRIDFLAG_:
-      case CROSSINGFLAG_: case LABELFLAG_: case SHOW_ALL_QUANTITIES_:
-      case HESSIAN_NORMAL_PERP_: case ITDEBUG_: case METIS_FACTOR_:
-      case TORUS_FILLED_: case DIRICHLET_MODE_: case SOBOLEV_MODE_:
-      case KRAYNIKPOPVERTEX_FLAG_: case KRAYNIKPOPEDGE_FLAG_:
-      case RGB_COLORS_FLAG_:  case CIRCULAR_ARC_DRAW_:
-      case VISIBILITY_TEST_: case SPARSE_CONSTRAINTS_: case SMOOTH_GRAPH_:
-      case BLAS_FLAG_: case AUGMENTED_HESSIAN_: case HESSIAN_SPECIAL_NORMAL_:
-      case BREAK_AFTER_WARNING_: case MPI_DEBUG_: case POP_DISJOIN_:
-      case POP_TO_EDGE_: case POP_TO_FACE_:
-      case FULL_BOUNDING_BOX_:
+    /* toggles */ case QUIETGO_: case QUIETLOAD_:
+    case KUSNER_: case ESTIMATE_: case DETURCK_: case HOMOTHETY_:
+    case SQGAUSS_: case AUTOPOP_: case AUTOCHOP_: case QUIET_:
+    case OLD_AREA_: case APPROX_CURV_: case RUNGE_KUTTA_: 
+    case CHECK_INCREASE_:  case DEBUG_: case MEAN_CURV_: case RIBIERE_CG_:
+    case DIFFUSION_: case GRAVITY_: case CONJ_GRAD_: case TRANSFORMS_:
+    case CONF_EDGE_SQCURV_: case EFFECTIVE_AREA_: case AREA_FIXED_:
+    case RAW_CELLS_: case CONNECTED_CELLS_: case CLIPPED_CELLS_:
+    case THICKEN_: case SHOW_INNER_: case SHOW_OUTER_: case COLORMAP_: 
+    case HESSIAN_DIFF_: case POST_PROJECT_: case MEAN_CURV_INT_:
+    case OPTIMIZE_: case NORMAL_CURVATURE_: case DIV_NORMAL_CURVATURE_:
+    case SHADING_:  case FACET_COLORS_: case BOUNDARY_CURVATURE_:
+    case NORMAL_MOTION_: case PINNING_: case VIEW_4D_: case MEMDEBUG_:
+    case METRIC_CONVERSION_: case AUTORECALC_: case GV_BINARY_:
+    case SELF_SIMILAR_: case AUTODISPLAY_: case FORCE_POS_DEF_:
+    case ASSUME_ORIENTED_: case HESSIAN_QUIET_: case JIGGLE_TOGGLE_:
+    case HESSIAN_NORMAL_: case YSMP_: case BUNCH_KAUFMAN_:
+    case QUANTITIES_ONLY_: case LINEAR_METRIC_:
+    case SQUARED_GRADIENT_: case H_INVERSE_METRIC_:
+    case HESSIAN_DOUBLE_NORMAL_: case INTERP_BDRY_PARAM_: 
+    case HESSIAN_NORMAL_ONE_: case PSCOLORFLAG_: case GRIDFLAG_:
+    case CROSSINGFLAG_: case LABELFLAG_: case SHOW_ALL_QUANTITIES_:
+    case HESSIAN_NORMAL_PERP_: case HESSIAN_SPECIAL_NORMAL_: case ITDEBUG_:
+    case METIS_FACTOR_: case VOLGRADS_EVERY_: case ZENER_DRAG_: 
+    case BACKCULL_: case INTERP_NORMALS_: case TORUS_FILLED_: case VERBOSE_:
+    case AMBIENT_PRESSURE_: case DIRICHLET_MODE_: case SOBOLEV_MODE_:
+    case KRAYNIKPOPVERTEX_FLAG_: 
+    case KRAYNIKPOPEDGE_FLAG_: case VISIBILITY_TEST_: case SPARSE_CONSTRAINTS_:
+    case BLAS_FLAG_: case AUGMENTED_HESSIAN_: case BREAK_AFTER_WARNING_:
+    case RGB_COLORS_FLAG_:  case CIRCULAR_ARC_DRAW_: case BEZIER_BASIS_:
+    case SMOOTH_GRAPH_: case MPI_DEBUG_: case POP_DISJOIN_:
+    case POP_TO_EDGE_: case POP_TO_FACE_: case POP_ENJOIN_:
+    case FULL_BOUNDING_BOX_: case BIG_ENDIAN_: case LITTLE_ENDIAN_:
          sprintf(pos,"%s %s ",keywordname(node->type),
           node->op1.toggle_state==ON_?"ON":"OFF"); pos += strlen(pos);
          break;
@@ -1079,7 +1116,7 @@ int prec_parent;  /* precedence level of parent, for parenthesizing */
       case HESSIAN_: case SHOWQ_: case CLOSE_SHOW_: case HESSIAN_MENU_:
       case DIRICHLET_: case SOBOLEV_: case REORDER_STORAGE_:
       case DIRICHLET_SEEK_: case SOBOLEV_SEEK_: case CONVERT_TO_QUANTS_:
-      case RENUMBER_ALL_:
+      case RENUMBER_ALL_: case DUMP_MEMLIST_: case FREE_DISCARDS_:
          sprintf(pos,"%s ",keywordname(node->type)); pos+=strlen(pos);
          break;
              
@@ -1137,12 +1174,14 @@ int prec_parent;  /* precedence level of parent, for parenthesizing */
        }
 
       case FUNCTION_CALL_:
-         sprintf(pos,"%s(",globals(node->op1.name_id)->name);
+       { struct global *g = globals(node->op1.name_id);
+         sprintf(pos,"%s(",g->name);
          pos += strlen(pos);  
-         exprint_recur(node+node->left,prec_parent); /* arglist */
+         if ( node->left )
+           exprint_recur(node+node->left,prec_parent); /* arglist */
          strcat(pos,")"); pos++;
          break;
-
+       }
       case FUNCTION_START_: break;
       case FUNCTION_DEF_START_: break;
       case FUNCTION_PROTO_START_: break;
@@ -1430,6 +1469,7 @@ int prec_parent;  /* precedence level of parent, for parenthesizing */
       case RAW_VERTEX_AVERAGE_: case RAWEST_VERTEX_AVERAGE_:
       case EQUIANGULATE_: case POP_EDGE_TO_TRI_: case POP_TRI_TO_EDGE_:
       case POP_QUAD_TO_QUAD_: case T1_EDGESWAP_:
+      case REVERSE_ORIENTATION_:
          sprintf(pos,"%s ",keywordname(node->type));
          pos += strlen(pos);
          exprint_recur(node+node->left,prec_parent);
