@@ -10,8 +10,16 @@
 *  Purpose:  declare global variables for Evolver program.
 */  
 
-/* element id hashing */
-extern element_id (*id_hash) ARGS((element_id));
+
+#ifdef  __cplusplus
+
+/* types for exception handler */
+struct loadexcep { int a; };
+struct cmd_excep { int a; };
+struct graph_excep { int a; };
+
+extern "C" {
+#endif
 
 #define DEFAULT_EDGE_COLOR BLACK
 #define DEFAULT_FACET_COLOR WHITE
@@ -35,6 +43,8 @@ extern int auto_convert_flag; /* whether to automatically convert_to_quantities
 extern int exit_flag;  /* whether to end this command interpreter */
 #define END_COMMANDS 1234551234
 extern int subshell_flag; /* whether in subshell, so can't reload */
+extern char *curdir;  /* current directory */
+
 
 #define DEFAULT_BRIGHTNESS 0.65
 extern REAL brightness; /* midlevel gray for screen display */
@@ -58,6 +68,10 @@ extern int file_no_max; /* number allocated */
 extern char **file_names; /* names of source files */
 extern int function_kludge_flag; /* handling errors in function def in datafile */
 
+/* for binary_printf byte order */
+extern int big_endian_flag;
+extern int little_endian_flag;
+
 /* my own ctypes */
 extern char kb_upper_array[256];
 extern char kb_lower_array[256];
@@ -81,6 +95,7 @@ struct remote_list_t {
 };
 extern struct remote_list_t remote_elements[NUMELEMENTS];
 
+extern int sparse_ibase_flag; /* for new scheme permitting sparse ibase */
 
 /* types of element storage expansion */
 #define EXTEND_BATCH 1
@@ -106,6 +121,18 @@ extern REAL **optparam_congrads;  /* constraint gradients */
 #define PERM_BLOCK  0xABAB
 #define TEMP_BLOCK  0xDCDC
 #define ETERNAL_BLOCK 0xEAEA
+#define GRAPH_BLOCK 0xCBCB
+
+/* for per-thread temp memory */
+#if defined(WINTHREADS)
+#define THREADBLOCK(blocktype) blocktype = \
+    (draw_thread_id==GetCurrentThreadId()) ? GRAPH_BLOCK : TEMP_BLOCK
+#elif defined(PTHREADS)
+#define THREADBLOCK(blocktype) blocktype = \
+    (draw_thread_id==pthread_self()) ? GRAPH_BLOCK : TEMP_BLOCK
+#else
+#define THREADBLOCK(blocktype)
+#endif
 
 /* for calc_volgrads() mode */
 #define NO_OPTS  0
@@ -167,6 +194,7 @@ extern int show_all_quantities;  /* to display default quantities also */
 extern int random_seed;  /* seed for random number generators */
 extern int option_q;  /* record command line option */
 extern int dont_resize_flag; /* set if datafile specified view matrix */
+extern int do_show_flag; /* for Mac kludge graphics command prompt */
 
 extern char *VERSION;
 extern char *evolver_version;  /* for version checking */
@@ -266,7 +294,7 @@ extern FILE *keylogfilefd;
 
 /* structure used in check.c */
 struct vvvv { element_id id;  /* which element    */
-              int v[3];    /* id's of vertices */
+              element_id v[3];    /* id's of vertices */
             };
 extern int check_count;      /* number of errors returned by run_checks() */
 extern int gocount;          /* number of iterations left */ 
@@ -274,6 +302,7 @@ extern int go_display_flag;     /* display each change */
 extern int box_flag;  /* whether or not to show outline box */
 extern int quiet_flag; /* whether to display normal output */
 extern int quiet_go_flag; /* whether to display normal g output */
+extern int quiet_load_flag; /* whether to display output while loading file */
 extern int shading_flag;  /* for facet shading by orientation */
 extern int smooth_graph_flag; /* whether to smoothly plot in lagrange mode */
 extern int visibility_test; /* whether to do after depth sort */
@@ -313,6 +342,7 @@ extern long reset_timestamp;  /* when surface loaded */
 #define NOSETSTAMP 0
 #define SETSTAMP   1
 
+extern int quarter_turn_var; /* for quarter_turn symmetry */
 extern int need_fe_reorder_flag; /* setting causes recalc to fe_reorder */
 extern int ackerman_flag;  /* whether doing phase space motion */
 extern char pix_file_name[150]; /* for P 2 */
@@ -483,7 +513,7 @@ extern struct locallist_t *localbase;
 /* locallist_t flag bits */
 #define LL_HAS_ARRAY 1
 #define LL_PERMANENT 2
-
+#define LL_IN_USE    4
 /* type of listing desired */
 #define LIST_PROTO  1
 #define LIST_FULL   2
@@ -564,6 +594,7 @@ extern int torus_display_mode;  /* default, raw, connected, clipped */
 #define TORUS_CLIPPED_MODE 3
 
 extern int view_matrix_global; /* global id number */
+extern int eigenvalues_list_global; /* for accessing ritz() output list */ 
 extern int torus_periods_global;
 extern int inverse_periods_global;
 extern REAL window_aspect_ratio;
@@ -610,6 +641,7 @@ extern REAL estimated_change; /* stored result */
 extern int autorecalc_flag; /* for toggling autorecalc after variable assign  */
 extern int autopop_flag;     /* whether to do autopopping */
 extern int pop_disjoin_flag;  /* whether cones to be disjoined rather than merged */
+extern int pop_enjoin_flag;  /* whether cones to be enjoined rather than disjoined */
 extern int pop_to_edge_flag;  /* control which way popping goes */
 extern int pop_to_face_flag;  /* control which way popping goes */
 /* some popping modes for various popping functions */
@@ -643,6 +675,8 @@ extern REAL area_fixed_target;  /* target value for fixed area */
 extern REAL area_fixed_pressure; /* Lagrange multiplier */
 #define AREA_Q_ID 0x1111          /* for constraint identification */
 extern int post_project_flag;     /* project to quant constr after each motion */
+extern int edge_diffusion_attr;
+extern int facet_diffusion_attr;
 extern struct expnode mobility_formula;
 extern int mobility_flag; /* whether mobility in effect */
 extern struct expnode mobility_tensor[MAXCOORD][MAXCOORD];
@@ -1711,6 +1745,7 @@ extern void * graphmutex;
 extern void * mem_mutex;
 extern unsigned int locking_thread; /* so we can tell who has it */
 /* Note ENTER_GRAPH_MUTEX and LEAVE_GRAPH_MUTEX form a {} block! */
+/* Also set up to be safe to nest within a thread. */
 #define ENTER_GRAPH_MUTEX { int did_graphlock_here;\
      if ( locking_thread != GetCurrentThreadId() ) \
    { MsgWaitForMultipleObjects(1,&graphmutex,0,100000,0); \
@@ -1761,8 +1796,12 @@ extern pthread_mutex_t mem_mutex;
 #endif
 
 /* Multithreading with worker threads */
-extern int threadflag;   /* 1 if -p option used and worker threads in effect */
+#ifdef WIN32
+extern volatile LONG busythreads;  /* number of threads that have started on a task */
+#else
 extern int busythreads;  /* number of threads that have started on a task */
+#endif
+extern int threadflag;   /* 1 if -p option used and worker threads in effect */
 extern int thread_task;  /* which task to do */
 extern element_id global_id;   /* global iteration variable */
 extern int web_locks;  /* informational counters */
@@ -1852,7 +1891,7 @@ extern struct thread_data **thread_data_ptrs;  /* one per thread to be allocated
 #include <pthread.h>
 extern void * element_mutex_ptr;
 extern void * web_mutex_ptr;
-#define GET_THREAD_DATA pthread_getspecific(thread_data_key)
+#define GET_THREAD_DATA (struct thread_data *)pthread_getspecific(thread_data_key)
 #define LOCK_ELEMENT(id)  (threadflag ? mylock_element(id) : 0)
 #define UNLOCK_ELEMENT(id)  (threadflag ?  (elptr(id)->lock = 0) : 0)
 #define LOCK_WEB  (threadflag ? ((pthread_mutex_trylock(web_mutex_ptr)==EBUSY)?\
@@ -1865,8 +1904,8 @@ extern pthread_key_t thread_data_key;
 #ifdef WINTHREADS
 #ifndef _WINDOWS_
 /* some stuff so windows.h doesn't have to be included everywhere */
-extern void * element_mutex_ptr;
-extern void * web_mutex_ptr;
+extern LPCRITICAL_SECTION element_mutex_ptr;
+extern LPCRITICAL_SECTION web_mutex_ptr;
 int __stdcall TryEnterCriticalSection(void *);
 void __stdcall EnterCriticalSection(void *);
 void __stdcall LeaveCriticalSection(void *);
@@ -1929,7 +1968,7 @@ __int32 f_elapsed_time[2];
   /* operands are in source-destination order in gcc asm */
 #define PROF_NOW(fullname) { \
   asm("rdtsc" : : ); \
-  asm("movl %%eax,%0" : "=m"(fullname) : ); \
+  asm("movl %%eax,%0" : "=m"(fullname[0]) : ); \
   asm("movl %%edx,%0" : "=m"(fullname[1]) : ); }
 #define PROF_CYCLES(fullname) ((double)*(unsigned long long*)fullname)
 #ifdef PROFILING
@@ -2044,8 +2083,41 @@ extern __int64 thread_launch_start;
 extern __int64 thread_launch_end;
 
 #else
-#define THREAD_FOR_ALL_NEW(elementtype,action) 
+#define THREAD_FOR_ALL_NEW(elementtype,action)                       \
+{ element_id *idptr;                                                 \
+  int proc,nextproc;                                                 \
+  struct thread_stages_data *th;                                     \
+  struct thread_data *data;                                          \
+  int maxstage;                                                      \
+  int elnum,maxelnum;                                                \
+  int now[2];                                                    \
+                                                                     \
+  data = GET_THREAD_DATA;                                            \
+  proc = data->worker_id;                                            \
+  nextproc = (proc == nprocs - 1) ? 0 : proc+1;                      \
+  maxstage = (proc == 0) ? max_thread_stages+1 : max_thread_stages;  \
+  th = thread_stages + proc;                                         \
+                                                                     \
+  for ( th->stage = 0 ; th->stage < maxstage ; th->stage++ )         \
+  { /* wait for nextproc to reach same stage */                      \
+    while ( thread_stages[nextproc].stage < th->stage ) ;            \
+                                                                     \
+    /* do all elements in this stage */                              \
+    idptr = th->blocks[elementtype][th->stage];                      \
+    maxelnum = th->counts[elementtype][th->stage];                   \
+    for ( elnum = 0 ; elnum < maxelnum ; elnum++,idptr++ )           \
+     action;                                                         \
+                                                                     \
+  }                                                                  \
+}           
+/* end THREAD_FOR_ALL_NEW */
+
 #endif
+
 /* profiling storage */
 extern int q_facet_setup_elapsed_time[2];
 
+#ifdef  __cplusplus
+}
+#endif
+
