@@ -426,7 +426,7 @@ int find_fixed()
   fixcount = 0;
   if ( !everything_quantities_flag )
     MFOR_ALL_BODIES(b_id)
-      { if ( get_battr(b_id) & (FIXEDVOL|PRESSURE) ) 
+      { if ( get_battr(b_id) & (FIXEDVOL/*|PRESSURE*/) ) 
         { if ( web.full_flag && (redundant_bi < 0) )
           { set_body_fixnum(b_id,-1);
             redundant_bi = 1;
@@ -679,7 +679,7 @@ void calc_lagrange()
   { ysmp_solve(&LS,rightside,vpressures);
     free_system(&LS);
   }
-  else
+  else 
 /*
     matvec_mul(rleftside,rightside,vpressures,fixcount,fixcount);
 */
