@@ -656,67 +656,101 @@ struct method_instance *mi;
        RECOVERABLE);
 }
 
-REAL star_gauss_method_value(v_info)
+REAL star_gauss_method_all(v_info,mode)
 struct qinfo *v_info;
+int mode; /* METHOD_VALUE, METHOD_GRADIENT, or METHOD_HESSIAN */
 { REAL deficit = 2*M_PI;
-  int k;
+  int k,i,j;
   int fudge = (v_info->flags & INCOMPLETE_STAR) ? 1 : 0;
 
   if ( v_info->vcount <= 1 ) return 0.0;
 
   for ( k = 1 ; k < v_info->vcount-fudge ; k++ )
   { REAL ss1,ss2,s1s2;
-    int kk = (!fudge && k==(v_info->vcount-1)) ? 0 : k;
-    ss1 = SDIM_dot(v_info->sides[0][k-1],v_info->sides[0][k-1]);
-    ss2 = SDIM_dot(v_info->sides[0][kk],v_info->sides[0][kk]);
-    s1s2 = SDIM_dot(v_info->sides[0][k-1],v_info->sides[0][kk]);
+    int kk = (!fudge && k==(v_info->vcount-1)) ? 1 : k+1;
+    REAL *s1 = v_info->sides[0][k-1],*s2 = v_info->sides[0][kk-1];
+    REAL denom;
+
+    ss1 = SDIM_dot(s1,s1);
+    ss2 = SDIM_dot(s2,s2);
+    s1s2 = SDIM_dot(s1,s2);
     deficit -= acos(s1s2/sqrt(ss1*ss2));
+
+    if ( mode == METHOD_VALUE )
+      continue;
+
+    denom = sqrt(ss1*ss2 - s1s2*s1s2);
+    for ( i = 0 ; i < SDIM ; i++ )
+    { REAL ddefds1,ddefds2;
+      ddefds1 = (s2[i] - s1s2/ss1*s1[i])/denom;
+      ddefds2 = (s1[i] - s1s2/ss2*s2[i])/denom;
+
+      v_info->grad[k][i] += ddefds1;
+      v_info->grad[0][i] -= ddefds1;
+      v_info->grad[kk][i] += ddefds2;
+      v_info->grad[0][i] -= ddefds2;
+
+      if ( mode == METHOD_GRADIENT )
+        continue;
+  
+      for ( j = 0 ; j < SDIM ; j++ )
+      { REAL dddefds1ds1,dddefds1ds2,dddefds2ds1,dddefds2ds2;
+  
+        dddefds1ds1 = (-s2[j]/ss1*s1[i] + s1s2/ss1/ss1*2*s1[j]*s1[i]
+                 - (i==j ? s1s2/ss1 : 0))/denom
+          - ddefds1/denom*0.5/denom*(2*ss2*s1[j] - 2*s1s2*s2[j]); 
+
+        dddefds1ds2 = ((i==j ? 1.0 : 0.0) - s1[j]/ss1*s1[i])/denom
+          - ddefds1/denom*0.5/denom*(2*ss1*s2[j] - 2*s1s2*s1[j]);
+
+        dddefds2ds1 = ((i==j ? 1.0 : 0.0) - s2[j]/ss2*s2[i])/denom
+          - ddefds2/denom*0.5/denom*(2*ss2*s1[j] - 2*s1s2*s2[j]);
+
+        dddefds2ds2 = (-s1[j]/ss2*s2[i] + s1s2/ss2/ss2*2*s2[j]*s2[i]
+                 - (i==j ? s1s2/ss2 : 0))/denom
+          - ddefds2/denom*0.5/denom*(2*ss1*s2[j] - 2*s1s2*s1[j]);
+
+        v_info->hess[k][k][i][j] += dddefds1ds1;
+        v_info->hess[0][k][i][j] -= dddefds1ds1;
+        v_info->hess[k][0][i][j] -= dddefds1ds1;
+        v_info->hess[0][0][i][j] += dddefds1ds1;
+
+        v_info->hess[k][kk][i][j] += dddefds1ds2;
+        v_info->hess[0][kk][i][j] -= dddefds1ds2;
+        v_info->hess[k][0][i][j] -= dddefds1ds2;
+        v_info->hess[0][0][i][j] += dddefds1ds2;
+
+        v_info->hess[kk][k][i][j] += dddefds2ds1;
+        v_info->hess[0][k][i][j] -= dddefds2ds1;
+        v_info->hess[kk][0][i][j] -= dddefds2ds1;
+        v_info->hess[0][0][i][j] += dddefds2ds1;
+
+        v_info->hess[kk][kk][i][j] += dddefds2ds2;
+        v_info->hess[0][kk][i][j] -= dddefds2ds2;
+        v_info->hess[kk][0][i][j] -= dddefds2ds2;
+        v_info->hess[0][0][i][j] += dddefds2ds2;
+
+      }
+        
+    }
+
   }
+
+
   return deficit;
 }
 
+REAL star_gauss_method_value(v_info)
+struct qinfo *v_info;
+{ return star_gauss_method_all(v_info,METHOD_VALUE);
+}
 REAL star_gauss_method_grad(v_info)
 struct qinfo *v_info;
-{ REAL deficit = 2*M_PI;
-  int k,i;
-  int fudge = (v_info->flags & INCOMPLETE_STAR) ? 1 : 0;
-
-  if ( v_info->vcount <= 1 ) return 0.0;
-
-  for ( k = 1 ; k < v_info->vcount-fudge ; k++ )
-  { REAL ss1,ss2,s1s2;
-    int kk = (!fudge && k==(v_info->vcount-1)) ? 0 : k;
-    ss1 = SDIM_dot(v_info->sides[0][k-1],v_info->sides[0][k-1]);
-    ss2 = SDIM_dot(v_info->sides[0][kk],v_info->sides[0][kk]);
-    s1s2 = SDIM_dot(v_info->sides[0][k-1],v_info->sides[0][kk]);
-    deficit -= acos(s1s2/sqrt(ss1*ss2));
-  }
-
-  /* now individual vertex gradients */
-  for ( k = 1 ; k < v_info->vcount-fudge ; k++ )
-  { REAL ss1,ss2,s1s2;
-    int kk = (!fudge && k==(v_info->vcount-1)) ? 0 : k;
-    REAL *s1 = v_info->sides[0][k-1],*s2 = v_info->sides[0][kk];
-    REAL denom;
-
-     ss1 = SDIM_dot(v_info->sides[0][k-1],v_info->sides[0][k-1]);
-     ss2 = SDIM_dot(v_info->sides[0][kk],v_info->sides[0][kk]);
-     s1s2 = SDIM_dot(v_info->sides[0][k-1],v_info->sides[0][kk]);
-     
-     denom = sqrt(ss1*ss2 - s1s2*s1s2);
-     for ( i = 0 ; i < SDIM ; i++ )
-     { REAL ddefds1,ddefds2;
-       ddefds1 = (s2[i] - s1s2/ss1*s1[i])/denom;
-       ddefds2 = (s1[i] - s1s2/ss2*s2[i])/denom;
-
-       v_info->grad[k][i] += ddefds1;
-       v_info->grad[0][i] -= ddefds1;
-       v_info->grad[kk?k+1:1][i] += ddefds2;
-       v_info->grad[0][i] -= ddefds2;
-     }
-  }
-
-  return deficit;
+{ return star_gauss_method_all(v_info,METHOD_GRADIENT);
+}
+REAL star_gauss_method_hess(v_info)
+struct qinfo *v_info;
+{ return star_gauss_method_all(v_info,METHOD_HESSIAN);
 }
 
 /*****************************************************************************
@@ -735,13 +769,29 @@ struct method_instance *mi;
        RECOVERABLE);
 }
 
-REAL star_sqgauss_method_value(v_info)
+REAL star_sqgauss_method_all(v_info,mode)
 struct qinfo *v_info;
+int mode; /* METHOD_VALUE, etc. */
 { REAL deficit = 2*M_PI;
   REAL area = 0.0;
-  int k;
+  int k,kk,i,j;
+  REAL **dadv,**ddefdv; /* gradient storage */
+  int pairs,final_edge;
 
   if ( v_info->vcount <= 1 ) return 0.0;
+  pairs = (v_info->vcount - 1);
+  if ( pairs <= 0 ) return 0.0;
+  if ( v_info->flags & INCOMPLETE_STAR )
+  { pairs--;
+    final_edge = pairs;
+  }
+  else 
+    final_edge = 0;
+
+  if ( mode == METHOD_HESSIAN )
+  { dadv = temp_dmatrix(0,v_info->vcount,0,SDIM-1);
+    ddefdv = temp_dmatrix(0,v_info->vcount,0,SDIM-1);
+  } 
 
   for ( k = 1 ; k < v_info->vcount ; k++ )
   { REAL ss1,ss2,s1s2;
@@ -752,41 +802,25 @@ struct qinfo *v_info;
     deficit -= acos(s1s2/sqrt(ss1*ss2));
     area += sqrt(ss1*ss2 - s1s2*s1s2)/6;
   }
-  return deficit*deficit/area;
-}
 
-REAL star_sqgauss_method_grad(v_info)
-struct qinfo *v_info;
-{ REAL deficit = 2*M_PI;
-  REAL area = 0.0;
-  int k,i;
-
-  if ( v_info->vcount <= 1 ) return 0.0;
-
-  for ( k = 1 ; k < v_info->vcount ; k++ )
-  { REAL ss1,ss2,s1s2;
-    int kk = (k==(v_info->vcount-1)) ? 0 : k;
-    ss1 = SDIM_dot(v_info->sides[0][k-1],v_info->sides[0][k-1]);
-    ss2 = SDIM_dot(v_info->sides[0][kk],v_info->sides[0][kk]);
-    s1s2 = SDIM_dot(v_info->sides[0][k-1],v_info->sides[0][kk]);
-    deficit -= acos(s1s2/sqrt(ss1*ss2));
-    area += sqrt(ss1*ss2 - s1s2*s1s2)/6;
-  }
+  if ( mode == METHOD_VALUE )
+    return deficit*deficit/area;
 
   /* now individual vertex gradients */
   for ( k = 1 ; k < v_info->vcount ; k++ )
   {  REAL ss1,ss2,s1s2;
-     int kk = (k==(v_info->vcount-1)) ? 0 : k;
-     REAL *s1 = v_info->sides[0][k-1],*s2 = v_info->sides[0][kk];
+     int kk = (k==(v_info->vcount-1)) ? 1 : k+1;
+     REAL *s1 = v_info->sides[0][k-1],*s2 = v_info->sides[0][kk-1];
      REAL denom;
 
-     ss1 = SDIM_dot(v_info->sides[0][k-1],v_info->sides[0][k-1]);
-     ss2 = SDIM_dot(v_info->sides[0][kk],v_info->sides[0][kk]);
-     s1s2 = SDIM_dot(v_info->sides[0][k-1],v_info->sides[0][kk]);
+     ss1 = SDIM_dot(s1,s1);
+     ss2 = SDIM_dot(s2,s2);
+     s1s2 = SDIM_dot(s1,s2);
      
      denom = sqrt(ss1*ss2 - s1s2*s1s2);
      for ( i = 0 ; i < SDIM ; i++ )
      { REAL ddefds1,ddefds2,dads1,dads2,dsqds1,dsqds2;
+       int j;
         ddefds1 = (s2[i] - s1s2/ss1*s1[i])/denom;
         ddefds2 = (s1[i] - s1s2/ss2*s2[i])/denom;
         dads1 = (s1[i]*ss2 - s1s2*s2[i])/denom/6;
@@ -795,15 +829,130 @@ struct qinfo *v_info;
         dsqds2 = 2*deficit*ddefds2/area - deficit*deficit/area/area*dads2;
         v_info->grad[k][i] += dsqds1;
         v_info->grad[0][i] -= dsqds1;
-        v_info->grad[kk?k+1:1][i] += dsqds2;
+        v_info->grad[kk][i] += dsqds2;
         v_info->grad[0][i] -= dsqds2;
+
+        if ( mode == METHOD_GRADIENT )
+          continue;
+
+        /* record gradients for later */
+        ddefdv[k][i] += ddefds1;
+        ddefdv[kk][i] += ddefds2;
+        dadv[k][i] += dads1;
+        dadv[kk][i] += dads2;
+
+        /* add in pure second partial terms */
+        for ( j = 0 ; j < SDIM ; j++ )
+        { REAL dddefds1ds1,dddefds1ds2,dddefds2ds1,dddefds2ds2;
+          REAL ddads1ds1,ddads1ds2,ddads2ds1,ddads2ds2;
+          REAL ddenomds1j,ddenomds2j;
+          REAL ddsqds1ds1,ddsqds1ds2,ddsqds2ds1,ddsqds2ds2;     
+  
+          /*     denom = sqrt(ss1*ss2 - s1s2*s1s2);   */
+          ddenomds1j = 1.0/denom*(s1[j]*ss2 - s1s2*s2[j]);
+          ddenomds2j = 1.0/denom*(s2[j]*ss1 - s1s2*s1[j]);
+  
+          ddads1ds1 = ((i==j ? 1:0)*ss2 - s2[j]*s2[i])/denom/6
+                   - (s1[i]*ss2 - s1s2*s2[i])/denom/6/denom*ddenomds1j;
+
+          ddads1ds2 = (s1[i]*2*s2[j] - s1[j]*s2[i] - s1s2*(i==j?1.0:0.))/denom/6
+                   - (s1[i]*ss2 - s1s2*s2[i])/denom/6/denom*ddenomds2j;
+  
+          ddads2ds1 = (s2[i]*2*s1[j] - s2[j]*s1[i] - s1s2*(i==j?1:0))/denom/6
+                   - (s2[i]*ss1 - s1s2*s1[i])/denom/6/denom*ddenomds1j;
+  
+          ddads2ds2 = ((i==j?1.0:0.0)*ss1 - s1[j]*s1[i])/denom/6
+                   - (s2[i]*ss1 - s1s2*s1[i])/denom/6/denom*ddenomds2j;
+  
+           // ddefds1 = (s2[i] - s1s2/ss1*s1[i])/denom;
+          dddefds1ds1 = (-s2[j]/ss1*s1[i] + s1s2/ss1/ss1*2*s1[j]*s1[i]
+                   - (i==j ? s1s2/ss1 : 0))/denom - ddefds1/denom*ddenomds1j; 
+  
+          dddefds1ds2 = ((i==j ? 1.0 : 0.0) - s1[j]/ss1*s1[i])/denom
+                   - ddefds1/denom*ddenomds2j;
+  
+          dddefds2ds1 = ((i==j ? 1.0 : 0.0) - s2[j]/ss2*s2[i])/denom
+                   - ddefds2/denom*ddenomds1j;
+  
+          dddefds2ds2 = (-s1[j]/ss2*s2[i] + s1s2/ss2/ss2*2*s2[j]*s2[i]
+                   - (i==j ? s1s2/ss2 : 0))/denom - ddefds2/denom*ddenomds2j;
+  
+            //   dsqds1 = 2*deficit*ddefds1/area - deficit*deficit/area/area*dads1;
+          ddsqds1ds1 = 2*deficit*dddefds1ds1/area                                        
+                     - deficit*deficit/area/area*ddads1ds1;
+  
+          ddsqds1ds2 = 2*deficit*dddefds1ds2/area                    
+                     - deficit*deficit/area/area*ddads1ds2;
+  
+          ddsqds2ds1 = 2*deficit*dddefds2ds1/area 
+                     - deficit*deficit/area/area*ddads2ds1;
+  
+          ddsqds2ds2 = 2*deficit*dddefds2ds2/area 
+                     - deficit*deficit/area/area*ddads2ds2;
+   
+          v_info->hess[k][k][i][j] += ddsqds1ds1;
+          v_info->hess[0][k][i][j] -= ddsqds1ds1;
+          v_info->hess[k][0][i][j] -= ddsqds1ds1;
+          v_info->hess[0][0][i][j] += ddsqds1ds1;
+  
+          v_info->hess[k][kk][i][j] += ddsqds1ds2;
+          v_info->hess[0][kk][i][j] -= ddsqds1ds2;
+          v_info->hess[k][0][i][j] -= ddsqds1ds2;
+          v_info->hess[0][0][i][j] += ddsqds1ds2;
+  
+          v_info->hess[kk][k][i][j] += ddsqds2ds1;
+          v_info->hess[0][k][i][j] -= ddsqds2ds1;
+          v_info->hess[kk][0][i][j] -= ddsqds2ds1;
+          v_info->hess[0][0][i][j] += ddsqds2ds1;
+  
+          v_info->hess[kk][kk][i][j] += ddsqds2ds2;
+          v_info->hess[0][kk][i][j] -= ddsqds2ds2;
+          v_info->hess[kk][0][i][j] -= ddsqds2ds2;
+          v_info->hess[0][0][i][j] += ddsqds2ds2;
+  
+        }
      }
   }
 
+  if ( mode == METHOD_HESSIAN )
+  { /* add in product of gradient terms */
+    for ( k = 1 ; k < v_info->vcount ; k++ )
+      for ( kk = 1 ; kk < v_info->vcount ; kk++ )
+        for ( i = 0 ; i < SDIM ; i++ )
+          for ( j = 0 ; j < SDIM ; j++ )
+          { REAL term;
+            term = 2/area*ddefdv[k][i]*ddefdv[kk][j]
+                 - 2*deficit/area/area*dadv[k][i]*ddefdv[kk][j]
+                 - 2*deficit/area/area*ddefdv[k][i]*dadv[kk][j]
+                 + 2*deficit*deficit/area/area/area*dadv[k][i]*dadv[kk][j];
+
+            v_info->hess[k][kk][i][j] += term;
+            v_info->hess[0][kk][i][j] -= term;
+            v_info->hess[k][0][i][j] -= term;
+            v_info->hess[0][0][i][j] += term;
+         }
+    free_temp_matrix(dadv);
+    free_temp_matrix(ddefdv);
+  }
+   
   return deficit*deficit/area;
 }
 
 
+REAL star_sqgauss_method_value(v_info)
+struct qinfo *v_info;
+{ return star_sqgauss_method_all(v_info,METHOD_VALUE);
+}
+REAL star_sqgauss_method_grad(v_info)
+struct qinfo *v_info;
+{ return star_sqgauss_method_all(v_info,METHOD_GRADIENT);
+}
+REAL star_sqgauss_method_hess(v_info)
+struct qinfo *v_info;
+{ return star_sqgauss_method_all(v_info,METHOD_HESSIAN);
+}
+
+
 /**************************************************************************
 
                 levine_energy
