@@ -218,14 +218,15 @@ static int q_flag;  /* whether to print drawing stats */
 *                  then machine id bits, for MPI
 *                  then element number.
 */
-#define NAMEOFFSETBITS (8*sizeof(GLuint) - MACHINE_ID_BITS - 2)
+#define NAMEMACHBITS 8
+#define NAMEOFFSETBITS (8*sizeof(GLuint) - NAMEMACHBITS - 2)
 #define NAMEOFFSETMASK  (((GLuint)1 << NAMEOFFSETBITS) - 1 )
 #define NAMETYPESHIFT  (8*sizeof(GLuint)-2)
 #define NAMETYPE_MASK  (3 << NAMETYPESHIFT)
-#define NAMEMACHMASK (((1 << MACHINE_ID_BITS) - 1) << NAMEOFFSETBITS)
+#define NAMEMACHMASK (((1 << NAMEMACHBITS) - 1) << NAMEOFFSETBITS)
 
-void my_glLoadName(id)
-element_id id;
+void my_glLoadName ARGS1((id),
+element_id id)
 { GLuint name;
   name = id_type(id) << NAMETYPESHIFT;
 #ifdef MPI_EVOLVER
@@ -244,14 +245,14 @@ element_id id;
 *
 * purpose: unravel picked name to element id.
 */
-element_id name_to_id(name)
-GLuint name;
+element_id name_to_id ARGS1((name),
+GLuint name)
 { element_id id;
-  id = ((name & NAMETYPE_MASK) >> NAMETYPESHIFT) << TYPESHIFT;
+  id = (element_id)((name & NAMETYPE_MASK) >> NAMETYPESHIFT) << TYPESHIFT;
   if ( (name & NAMEOFFSETMASK) != NAMEOFFSETMASK )
      id |=   VALIDMASK | (name & NAMEOFFSETMASK);
 #ifdef MPI_EVOLVER
-  id |= ((name & NAMEMACHMASK) >> NAMEOFFSETBITS) << MACH_ID_SHIFT;
+  id |= (element_id)((name & NAMEMACHMASK) >> NAMEOFFSETBITS) << MACH_ID_SHIFT;
 #endif
   return id;
 }
@@ -317,8 +318,8 @@ void kb_glAntiNormal3dv(REAL *v) /* save for edges and facets */
 *          or pass it on to gl.
 */
 static float er,eg,eb,ea; /* current edge color */
-void e_glColor(c) 
-int c;
+void e_glColor ARGS1((c),
+int c)
 { 
   if ( edge_rgb_color_attr > 0 )
   {
@@ -341,8 +342,8 @@ int c;
 *
 * purpose: Either save current edge data in edge list, or pass on to gl.
 */
-void e_glVertex3dv(x)
-REAL *x;
+void e_glVertex3dv ARGS1((x),
+REAL *x)
 { if ( !arraysflag ) 
   { glVertex3d((float)x[0],(float)x[1],(float)x[2]); return; }
   if ( !edgearray ) edgemax = 0;
@@ -367,8 +368,8 @@ REAL *x;
 *          or pass it on to gl.
 */
 static float fr,fg,fb,fa; /* current facet color */
-void f_glColor(c) 
-int c;
+void f_glColor ARGS1((c),
+int c)
 { 
   if ( facet_rgb_color_attr > 0 )
   {
@@ -391,8 +392,8 @@ int c;
 *
 * purpose: Either save current facet data in facet list, or pass on to gl.
 */
-void f_glVertex3dv(x)
-REAL *x;
+void f_glVertex3dv ARGS1((x),
+REAL *x)
 { if ( !arraysflag )
   { glVertex3f((float)x[0],(float)x[1],(float)x[2]); return; }
   if ( !facetarray ) facetmax = 0;
@@ -521,8 +522,8 @@ void init_Oglz ARGS((void));
 GLuint pickbuf[PICKBUFLENGTH];
 int pick_flag;
 
-void pick_func(x,y)
-int x,y;
+void pick_func ARGS2((x,y),
+int x, int y)
 { struct graph_thread_data *td = GET_DATA;
   GLint hits, viewport[4];
   int i,n;
@@ -586,7 +587,7 @@ int x,y;
        
     }
   }
-  outstring("\n");  /* to get to next line after prompt */
+  erroutstring("\n");  /* to get to next line after prompt */
   if ( valid_id(v_id) )
     { pickvnum = loc_ordinal(v_id) + 1;
       #ifdef MPI_EVOLVER
@@ -594,7 +595,7 @@ int x,y;
       #else
       sprintf(msg,"Picked vertex %d\n",pickvnum);
       #endif
-      outstring(msg); 
+      erroutstring(msg); 
     }
 
   if ( valid_id(e_id) ) 
@@ -635,7 +636,7 @@ int x,y;
       #else
       sprintf(msg,"Picked vertex %d\n",pickvnum);
       #endif
-      outstring(msg); 
+      erroutstring(msg); 
     }
 #endif
     pickenum = loc_ordinal(e_id) + 1;
@@ -644,11 +645,11 @@ int x,y;
     #else
     sprintf(msg,"Picked edge %d\n",pickenum);
     #endif
-    outstring(msg);
+    erroutstring(msg);
   }
   else 
   if ( e_id == NULLEDGE )
-     outstring("Picked facet subdivision edge.\n");
+     erroutstring("Picked facet subdivision edge.\n");
 
   if ( valid_id(f_id) ) 
   { pickfnum = loc_ordinal(f_id) + 1;
@@ -657,9 +658,9 @@ int x,y;
     #else
     sprintf(msg,"Picked facet %d\n",pickfnum);
     #endif
-    outstring(msg);
+    erroutstring(msg);
   }
-  outstring(current_prompt);
+  erroutstring(current_prompt);
 
   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
@@ -674,8 +675,8 @@ int x,y;
 
 void mouse_func ARGS((int,int,int,int));
 
-void mouse_func(button,state,x,y)
-int button, state, x, y;
+void mouse_func ARGS4((button,state,x,y),
+int button, int state, int x, int y)
 { struct graph_thread_data *td = GET_DATA;
   switch ( button )
   { case GLUT_LEFT_BUTTON:
@@ -716,8 +717,8 @@ int button, state, x, y;
 */
 void mouse_loc_func ARGS((int,int));
 
-void mouse_loc_func(x,y)
-int x,y;
+void mouse_loc_func ARGS2((x,y),
+int x, int y)
 { struct graph_thread_data *td = GET_DATA;
   int i,j;
 
@@ -795,8 +796,8 @@ int x,y;
 */
 void reshape_func ARGS(( int, int ));
 
-void reshape_func ( x, y )
-int x,y;
+void reshape_func ARGS2(( x, y ),
+int x, int y)
 { struct graph_thread_data *td = GET_DATA;
 
   if ( window_aspect_ratio != 0.0 )
@@ -807,17 +808,12 @@ int x,y;
     if ( abs(td->xsize - x) < abs(td->ysize - y) )
     { /* moving y */
       /* if ( td->ysize == y ) return; */ /* not changing */
-      x = y/fabs(window_aspect_ratio);
+      x = (int)(y/fabs(window_aspect_ratio));
     }
-    else //if ( td->ysize == y )
+    else 
     { /* moving x */
-      y = x*fabs(window_aspect_ratio);
+      y = (int)(x*fabs(window_aspect_ratio));
     }
-   // else
-   // { /* must be moving both */
-    //  x = (int)(sqrt((area/window_aspect_ratio)));
-    //  y = (int)(sqrt((area*window_aspect_ratio)));
-   // }
     td->window_aspect = window_aspect_ratio;
     td->aspect_flag = 0;
     glutReshapeWindow(x,y);
@@ -897,8 +893,8 @@ int x,y;
 
 void specialkey_func ARGS((int,int,int));
 
-void specialkey_func(key,x,y)
-int key,x,y;
+void specialkey_func ARGS3((key,x,y),
+int key, int x, int y)
 { struct graph_thread_data *td = GET_DATA;
   switch ( key )
   { 
@@ -923,9 +919,9 @@ int key,x,y;
 
 void key_func ARGS((unsigned char,int,int));
 
-void key_func(key,x,y)
-unsigned char key;
-int x,y;
+void key_func ARGS3((key,x,y),
+unsigned char key,
+int x, int y)
 { struct graph_thread_data *td = GET_DATA;
   int i,j;
 
@@ -947,14 +943,14 @@ int x,y;
 
     case 'b':
         td->edge_bias -= 0.001; 
-        sprintf(msg,"\nEdge front bias now %f\n",td->edge_bias); outstring(msg);
-        outstring(current_prompt);
+        sprintf(msg,"\nEdge front bias now %f\n",td->edge_bias); erroutstring(msg);
+        erroutstring(current_prompt);
         break;
 
     case 'B':
         td->edge_bias += 0.001; 
-        sprintf(msg,"\nEdge front bias now %f\n",td->edge_bias); outstring(msg);
-        outstring(current_prompt);
+        sprintf(msg,"\nEdge front bias now %f\n",td->edge_bias); erroutstring(msg);
+        erroutstring(current_prompt);
         break;
 
     case 'R':
@@ -1011,22 +1007,22 @@ int x,y;
 
     case 'i': /* toggle interleaved elements in opengl arrays */ 
         interleaved_flag = !interleaved_flag;
-        outstring(interleaved_flag?"Interleaving ON.\n":"Interleaving OFF.\n"); 
-        outstring(current_prompt);
+        erroutstring(interleaved_flag?"Interleaving ON.\n":"Interleaving OFF.\n"); 
+        erroutstring(current_prompt);
         newarraysflag = 1;
         break;
 
     case 'I': /* indexed arrays */
         indexing_flag = !indexing_flag;
-        outstring(indexing_flag?"Array indexing ON.\n":"Array indexing OFF.\n"); 
-        outstring(current_prompt);
+        erroutstring(indexing_flag?"Array indexing ON.\n":"Array indexing OFF.\n"); 
+        erroutstring(current_prompt);
         newarraysflag = 1;
         break;
    
     case 'S': /* toggle sending strips in arrays */
         strips_flag = !strips_flag;
-        outstring(strips_flag?"Element strips ON.\n":"Element strips OFF.\n"); 
-        outstring(current_prompt);
+        erroutstring(strips_flag?"Element strips ON.\n":"Element strips OFF.\n"); 
+        erroutstring(current_prompt);
         normflag = 1; /* gourard shading */
         indexing_flag = 1;
         newarraysflag = 1;
@@ -1080,17 +1076,17 @@ int x,y;
     case 'D': /* toggle display list */
         dlistflag = dlistflag ? NOLIST:RESETLIST; 
         if ( dlistflag )
-        { outstring("\nOpenGL display list now ON.\n");
+        { erroutstring("\nOpenGL display list now ON.\n");
           if ( arraysflag )
           { glDisableClientState(GL_COLOR_ARRAY);
             glDisableClientState(GL_NORMAL_ARRAY);
             glDisableClientState(GL_VERTEX_ARRAY);
-            outstring("\nOpenGL arrays now OFF.\n");
+            erroutstring("\nOpenGL arrays now OFF.\n");
           }
           arraysflag = 0;
         }
-        else outstring("\nOpenGL display list now OFF.\n");
-        outstring(current_prompt);
+        else erroutstring("\nOpenGL display list now OFF.\n");
+        erroutstring(current_prompt);
         graph_timestamp = ++global_timestamp; /* force recalculate arrays */
         break;
   
@@ -1106,7 +1102,7 @@ int x,y;
         if ( arraysflag )
         { /* Workaround really bizarre line-drawing bug */
           if ( td->linewidth == 1.0 ) { td->linewidth = 0.5; glLineWidth(0.5);}
-          outstring("\nOpenGL vertex arrays now ON.\n");
+          erroutstring("\nOpenGL vertex arrays now ON.\n");
           glInterleavedArrays(GL_C4F_N3F_V3F,0,(void*)fullarray);
           newarraysflag = 1;
           dlistflag = 0;
@@ -1116,9 +1112,9 @@ int x,y;
           glDisableClientState(GL_COLOR_ARRAY);
           glDisableClientState(GL_NORMAL_ARRAY);
           glDisableClientState(GL_VERTEX_ARRAY);
-          outstring("\nOpenGL vertex_arrays now OFF.\n");
+          erroutstring("\nOpenGL vertex_arrays now OFF.\n");
         }
-        outstring(current_prompt);
+        erroutstring(current_prompt);
         break;
 
    case 's': /* toggle stereo */
@@ -1128,16 +1124,16 @@ int x,y;
  
    case 'p': /* perspective mode */
       if ( td->stereomode ) 
-         outstring("\nOpenGL projection now CROSS-EYED STEREO.\n"); 
+         erroutstring("\nOpenGL projection now CROSS-EYED STEREO.\n"); 
       else if ( td->projmode==P_ORTHO ) 
       { td->projmode=P_PERSP; 
-        outstring("\nOpenGL projection now PERSPECTIVE.\n");
+        erroutstring("\nOpenGL projection now PERSPECTIVE.\n");
       }
       else 
       { td->projmode=P_ORTHO; 
-        outstring("\nOpenGL projection now ORTHOGONAL.\n");
+        erroutstring("\nOpenGL projection now ORTHOGONAL.\n");
       }
-      outstring(current_prompt);
+      erroutstring(current_prompt);
       reshape_func(td->xsize,td->ysize);
       break;
  
@@ -1154,18 +1150,18 @@ int x,y;
 
    case 'Q': /* toggle drawing stats printing */
       q_flag = !q_flag;
-      outstring(q_flag ?
+      erroutstring(q_flag ?
        "\nPrinting drawing stats ON\n":"\nPrinting drawing stats OFF\n");
-      outstring(current_prompt);
+      erroutstring(current_prompt);
       break;
 
    #ifdef MPI_EVOLVER
    case 'y': /* MPI version only */
       mpi_show_corona_flag = ! mpi_show_corona_flag;
       newarraysflag = 1;
-      outstring(mpi_show_corona_flag ?
+      erroutstring(mpi_show_corona_flag ?
        "\nShowing MPI corona ON\n":"\nShowing MPI corona OFF\n");
-      outstring(current_prompt);
+      erroutstring(current_prompt);
       break;
    #endif
 
@@ -1174,70 +1170,70 @@ int x,y;
       return;
 
    case 'h': case '?':
-      outstring("\nGraphics window help:\n");
-      outstring("Left mouse: move   Right mouse: pick\n");
-      outstring("Graphics window keys:\n");
-      outstring(td->mouse_mode==MM_ROTATE?
+      erroutstring("\nGraphics window help:\n");
+      erroutstring("Left mouse: move   Right mouse: pick\n");
+      erroutstring("Graphics window keys:\n");
+      erroutstring(td->mouse_mode==MM_ROTATE?
         "r  Rotate mode for left mouse button, now ACTIVE\n":
         "r  Rotate mode for left mouse button\n");
-      outstring(td->mouse_mode==MM_TRANSLATE?
+      erroutstring(td->mouse_mode==MM_TRANSLATE?
         "t  Translate mode for left mouse button, now ACTIVE\n":
         "t  Translate mode for left mouse button\n");
-      outstring(td->mouse_mode==MM_SCALE?
+      erroutstring(td->mouse_mode==MM_SCALE?
         "z  Zoom mode for left mouse button, now ACTIVE\n":
         "z  Zoom mode for left mouse button\n");
-      outstring(td->mouse_mode==MM_SPIN?
+      erroutstring(td->mouse_mode==MM_SPIN?
         "c  Clockwise/counterclockwise mode, now ACTIVE\n":
         "c  Clockwise/counterclockwise mode\n");
-      outstring("W  Widen edges\n");
-      outstring("w  Narrow edges\n");
-      outstring("B  Increase edge front bias by 0.001\n");
-      outstring("b  Decrease edge front bias by 0.001\n");
-      outstring(normflag? 
+      erroutstring("W  Widen edges\n");
+      erroutstring("w  Narrow edges\n");
+      erroutstring("B  Increase edge front bias by 0.001\n");
+      erroutstring("b  Decrease edge front bias by 0.001\n");
+      erroutstring(normflag? 
         "g  Gourard shading (smooth shading) toggle, now ON\n":
         "g  Gourard shading (smooth shading) toggle, now OFF\n" );
-      outstring("R  Reset view\n");
-      outstring("m  Center image\n");
-      outstring(edgeshow_flag?"e  Toggle showing all edges, now ON\n":
+      erroutstring("R  Reset view\n");
+      erroutstring("m  Center image\n");
+      erroutstring(edgeshow_flag?"e  Toggle showing all edges, now ON\n":
         "e  Toggle showing all edges, now OFF\n");
-      outstring(td->facetshow_flag?"f  Toggle showing facets, now ON\n":
+      erroutstring(td->facetshow_flag?"f  Toggle showing facets, now ON\n":
         "f  Toggle showing facets, now OFF\n");
-      outstring(td->projmode==P_PERSP?
+      erroutstring(td->projmode==P_PERSP?
         "p  Toggle orthogonal/perspective projection, now perspective\n":
         "p  Toggle orthogonal/perspective projection, now orthogonal\n");
-      outstring("s  Toggle cross-eyed stereo\n");
-      outstring(normflag? "g  Gourard shading (smooth shading) toggle, now ON\n":
+      erroutstring("s  Toggle cross-eyed stereo\n");
+      erroutstring(normflag? "g  Gourard shading (smooth shading) toggle, now ON\n":
              "g  Gourard shading (smooth shading) toggle, now OFF\n" );
-      outstring("F  Set rotate/zoom focus to last picked vertex\n");
-      outstring("arrow keys  translate image\n");
-      outstring("G  Another graphics window\n");
-      outstring("H  Guru-level help items\n");
-      outstring("x  Close graphics\n");
-      outstring(current_prompt);
+      erroutstring("F  Set rotate/zoom focus to last picked vertex\n");
+      erroutstring("arrow keys  translate image\n");
+      erroutstring("G  Another graphics window\n");
+      erroutstring("H  Guru-level help items\n");
+      erroutstring("x  Close graphics\n");
+      erroutstring(current_prompt);
       break;
 
    case 'H': /* guru level help */
-     outstring("\nFollowing for fiddling with OpenGL drawing modes:\n");
-     outstring(dlistflag?"D  Toggle using display list, now ON\n":
+     erroutstring("\nFollowing for fiddling with OpenGL drawing modes:\n");
+     erroutstring(dlistflag?"D  Toggle using display list, now ON\n":
        "D  Toggle using display list, now OFF\n");
-     outstring(arraysflag?"a  Toggle using vertex and color arrays, now ON\n"
+     erroutstring(arraysflag?"a  Toggle using vertex and color arrays, now ON\n"
         :"a  Toggle using vertex and color arrays, now OFF\n");
-     outstring(indexing_flag ? "I  Indexed vertex arrays, now ON\n"
+     erroutstring(indexing_flag ? "I  Indexed vertex arrays, now ON\n"
                            : "I  Indexed vertex arrays, now OFF\n");
-     outstring(interleaved_flag ? "i  Interleaved vertex arrays, now ON\n"
+     erroutstring(interleaved_flag ? "i  Interleaved vertex arrays, now ON\n"
                            : "i  Interleaved vertex arrays, now OFF\n");
-     outstring(strips_flag?
+     erroutstring(strips_flag?
     "S  Use element strips, now ON (indexed elements automatically turned on)\n"
     :"S  Use element strips, now OFF\n");
-     outstring("Y  One-time coloring of strips generated in S mode.\n");
-     outstring("       Caution: assumes display facets same as real, with no gaps.\n");
-     outstring("u  Toggle window update every graphics command, now ");
-     outstring("Q  Toggle printing some statistics during drawing\n");
+     erroutstring("Y  One-time coloring of strips generated in S mode.\n");
+     erroutstring("       Caution: assumes display facets same as real, with no gaps.\n");
+     erroutstring("u  Toggle window update every graphics command, now ");
+     erroutstring("Q  Toggle printing some statistics during drawing\n");
      #ifdef MPI_EVOLVER
-     outstring("y  Toggle corona display\n");
+     erroutstring("y  Toggle corona display\n");
      #endif
  
-     outstring(current_prompt);
+     erroutstring(current_prompt);
      break;
   }
   glutPostRedisplay();  /* generate redraw message */
@@ -1247,14 +1243,14 @@ int x,y;
  void mainmenu_func ARGS((int));
  void submenu_func ARGS((int));
 
- void mainmenu_func(choice)
- int choice;
+ void mainmenu_func ARGS1((choice),
+ int choice)
  { 
    key_func(choice,0,0); 
  }
  
- void submenu_func(choice)
- int choice;
+ void submenu_func ARGS1((choice),
+ int choice)
  { 
    key_func(choice,0,0);
  }
@@ -1352,8 +1348,8 @@ BOOL __stdcall handle_func(HWND hwnd, LPARAM lParam)
  * purpose: catch signals meant to wake up thread.
  */
 
- void glut_catcher(x)
- int x;
+ void glut_catcher ARGS1((x),
+ int x)
  {
    signal(SIGKICK,glut_catcher);
  }
@@ -1367,11 +1363,11 @@ BOOL __stdcall handle_func(HWND hwnd, LPARAM lParam)
  * purpose: Create OpenGL display thread and window.
  */
 #ifdef PTHREADS
-void * draw_thread(arglist)
-void *arglist;
+void * draw_thread ARGS1((arglist),
+void *arglist)
 #else
-void __cdecl draw_thread(arglist)
-void *arglist;
+void __cdecl draw_thread ARGS1((arglist),
+void *arglist)
 #endif
 { int i,j;
   int argc = 1; /* to keep glutInit happy */
@@ -1383,7 +1379,7 @@ void *arglist;
   int xpixels,ypixels;
 
 #ifdef WIN32
-  HANDLE foregroundhwnd = GetForegroundWindow();
+  HWND foregroundhwnd = GetForegroundWindow();
   draw_thread_id = GetCurrentThreadId();
 #else
   draw_thread_id = pthread_self();  /* get thread id */
@@ -1475,7 +1471,7 @@ void *arglist;
         td->from_focus[i][j] = tdd->from_focus[i][j];
       }
     }
-    dup_window = 0;
+    dup_window = 0; 
   }
   else
   { for ( i = 0 ; i < HOMDIM ; i++ )
@@ -1562,12 +1558,6 @@ void init_Oglz()
 
   if ( !initz_flag )
   { initz_flag = 1;
-    if ( !to_focus )
-    { to_focus = dmatrix(0,MAXCOORD,0,MAXCOORD);
-      from_focus = dmatrix(0,MAXCOORD,0,MAXCOORD);
-      for ( n = 0 ; n <= MAXCOORD ; n++ )
-      to_focus[n][n] = from_focus[n][n] = 1.0;
-    }
 
 #ifdef WIN32
      _beginthread(draw_thread,0,0);
@@ -1583,6 +1573,7 @@ void init_Oglz()
     main_pid = getpid();
 #ifdef MAC_OS_X
     /* GLUT doesn't work as secondary thread on 10.0.2, so main draws */
+    curdir = getcwd(NULL,0);  /* so command thread gets proper directory */
     pthread_create(&th,NULL,(void*(*)(void*))mac_exec_commands,NULL);
     draw_thread(NULL);
 #else
@@ -1606,13 +1597,6 @@ void Oglz_start ARGS((void));
 void Oglz_start()
 {
   if ( initz_flag == 0 ) init_Oglz();
-  if ( !to_focus )
-  { int n;
-    to_focus = dmatrix(0,MAXCOORD,0,MAXCOORD);
-    from_focus = dmatrix(0,MAXCOORD,0,MAXCOORD);
-    for ( n = 0 ; n <= MAXCOORD ; n++ )
-      to_focus[n][n] = from_focus[n][n] = 1.0;
-  }
 
 }    
 
@@ -1625,9 +1609,9 @@ void Oglz_start()
 */
 void Oglz_edge ARGS((struct graphdata *,edge_id));
 
-void Oglz_edge(g,e_id)
-struct graphdata *g;
-edge_id e_id;
+void Oglz_edge ARGS2((g,e_id),
+struct graphdata *g,
+edge_id e_id)
 {  
   int k;
   int e_color;
@@ -1669,9 +1653,9 @@ edge_id e_id;
 
 void Oglz_facet ARGS((struct graphdata *,facet_id));
 
-void Oglz_facet(g,f_id)
-struct graphdata *g;
-facet_id f_id;
+void Oglz_facet ARGS2((g,f_id),
+struct graphdata *g,
+facet_id f_id)
 {  
   int i,k;
   REAL len;
@@ -1788,6 +1772,9 @@ void Ogl_close ARGS((void))
   glutDestroyWindow(td->win_id);
   memset((char*)td,0,sizeof(struct graph_thread_data));
 
+#ifndef MAC_OS_X
+ /* Mac objects to closing graphics thread */
+
   for ( i = 0 ; i < MAXGRAPHWINDOWS ; i++ )
   { if ( gthread_data[i].in_use != 0 ) break;
   }
@@ -1805,8 +1792,9 @@ void Ogl_close ARGS((void))
 #elif defined(PTHREADS)
   pthread_exit(NULL);
 #endif
-
   }
+#endif
+
 }
 
 /***********************************************************************
@@ -1828,8 +1816,8 @@ void Ogl_close_show()
 *
 */
 
-int vercolcomp(a,b)
-struct vercol *a,*b;
+int vercolcomp ARGS2((a,b),
+struct vercol *a, struct vercol *b)
 { 
   int i;
   for ( i = 0 ; i < 10 ; i++ )
@@ -1847,8 +1835,8 @@ struct vercol *a,*b;
 * purpose: comparison function for sorting indexed edges 
 */
 
-int eecomp(a,b)
-int *a,*b;
+int eecomp ARGS2((a,b),
+int *a, int *b)
 { if ( *a < *b ) return -1;
   if ( *a > *b ) return 1;
   if ( a[1] > b[1] ) return 1;
@@ -2024,9 +2012,14 @@ void draw_screen()
     glutSetWindowTitle(td->wintitle);
   td->new_title_flag = 0;
 
+#ifdef __cplusplus
+  try
+  {
+#else
   /* Set up longjmp to return here in case of error  */
   if ( setjmp(graphjumpbuf) )
   { return; }
+#endif
 
   GL_ERROR_CHECK
 
@@ -2226,7 +2219,6 @@ void draw_screen()
   /* make sure global view matrix is same as first window */
   if ( td->win_id == 1 )
   { 
-    if ( view == NULL ) { return; } /* rapid permload?? */
     for ( i = 0 ; i < HOMDIM ; i++ )
       for ( j = 0 ; j < HOMDIM ; j++ )
         view[i][j] = td->view[i][j];
@@ -2274,6 +2266,15 @@ void draw_screen()
   glFlush();
   glutSwapBuffers();  
 
+  temp_free_all();  /* should free just graphics thread temps */
+
+#ifdef __cplusplus
+  }
+  catch(...)
+  {
+  }
+#endif
+
 bailout:
   glutIdleFunc(idle_func);
 
@@ -2304,11 +2305,15 @@ void idle_func()
   else
   {
 #if (defined(SUN) || defined(LINUX)) && !defined(__STRICT_ANSI__)
-    /* let the thread could sleep .1 sec; need link with -lrt */
+    /* let the thread could sleep .02 sec; need link with -lrt */
     struct timespec delay;
-    delay.tv_sec = 0; delay.tv_nsec = 100000000;
+    delay.tv_sec = 0; delay.tv_nsec = 20000000;
+#ifdef SUN
+    __nanosleep(&delay,NULL); 
+#else
     nanosleep(&delay,NULL); 
 #endif
+#endif
 
 #ifdef WIN32
     /* sleep until some event */
@@ -2346,10 +2351,10 @@ void display()
  
 #ifdef MAC_OS_X
       /* glutPostRedisplay() generates ugly NSEvent leak messages */
-      glutSetWindow(td->win_id);
-      draw_screen(); 
+  /*      glutSetWindow(td->win_id); */
+     /*   draw_screen();  Conflict with graphics thread! */
          
-      /* glutPostWindowRedisplay(td->win_id); */  /* generate redraw message */    
+      glutPostWindowRedisplay(td->win_id);  /* generate redraw message */    
 #else
 /*   WARNING: glutSetWindow() in non-drawing thread causes problems. */
 /*      glutSetWindow(td->win_id); */
@@ -2381,8 +2386,8 @@ void display()
 * Output:  striparray
 */
 
-int ecomp(a,b)
-char *a,*b;
+int ecomp ARGS2((a,b),
+char *a, int *b)
 { int ai = indexarray[edgestart+*(int*)a]; 
   int bi = indexarray[edgestart+*(int*)b]; 
   if ( ai < bi ) return -1;
@@ -2394,9 +2399,12 @@ struct festruct { int v[3];  /* head and tail and opposite vertices */
                   int f;     /* facet on left */
                 };
 
-int fecomp ( a,b)
-struct festruct *a,*b;
-{ if ( a->v[0] < b->v[0] ) return -1;
+int fecomp  ARGS2(( aa,bb),
+void *aa, void*bb)
+{
+  struct festruct *a = (struct festruct *)aa;
+  struct festruct *b = (struct festruct *)bb;
+  if ( a->v[0] < b->v[0] ) return -1;
   if ( a->v[0] > b->v[0] ) return  1;
   if ( a->v[1] < b->v[1] ) return -1;
   if ( a->v[1] > b->v[1] ) return  1;
@@ -2574,8 +2582,8 @@ void make_strips()
         /*find in felist */
         if ( whichway ) { key.v[0] = va; key.v[1] = vb; }
         else { key.v[1] = va; key.v[0] = vb; }
-        fe = bsearch(&key,(void*)felist,facetcount,sizeof(struct festruct),
-             FCAST fecomp);
+        fe = (struct festruct*)bsearch(&key,(void*)felist,facetcount,
+          sizeof(struct festruct), FCAST fecomp);
         if ( fe==NULL ) break;  /* done; maybe hit edge of surface */
 
         /*see if facet done yet */
@@ -2656,8 +2664,8 @@ void make_strips()
 * purpose: Compute hash value for vertex.
 */
 int hashsize; /* size of hashtable */
-int hashfunc(a)
-struct vercol *a;
+int hashfunc ARGS1((a),
+struct vercol *a)
 {
   int h;
   int scale = 100000;
