@@ -88,6 +88,10 @@ void do_show()
   char line[100]; /* for reading user commands */
   int old_flag = iterate_flag;
 
+#ifdef MAC_OS_X
+  do_show_flag++;  /* kludge so mac_exec_commands() does do_show() again */
+#endif
+
 #ifndef OPENGL
   /* to prevent unnecessary recalculation of display */
   graph_timestamp = ++global_timestamp;  /* new surface */
@@ -428,70 +432,73 @@ char *string;
 */
 void init_view()
 {
-  int i,n;
+  int i;
 
-  view = dmatrix(0,SDIM,0,SDIM);
+  if ( identmat == NULL )
+  { /* first time set-up */
+
+    /* set up identity matrix */
+    identmat = perm_matrix2(MAXCOORD+1,MAXCOORD+1); 
+    for ( i = 0 ; i <= MAXCOORD ; i++ )
+      identmat[i][i] = 1.0;
+
+
+    view = perm_matrix2(MAXCOORD+1,MAXCOORD+1); 
+    to_focus = perm_matrix2(MAXCOORD+1,MAXCOORD+1); 
+    from_focus = perm_matrix2(MAXCOORD+1,MAXCOORD+1); 
+  
+    /* set rotation matrices */
+    spinr = perm_matrix2(MAXCOORD+1,MAXCOORD+1);
+    matcopy(spinr,identmat,SDIM+1,SDIM+1);
+    spinr[0][0] = spinr[1][1] = cos(dang);
+    spinr[0][1] = -(spinr[1][0] = sin(dang));
+  
+    spinl = perm_matrix2(MAXCOORD+1,MAXCOORD+1);
+    matcopy(spinl,identmat,SDIM+1,SDIM+1);
+    spinl[0][0] = spinl[1][1] = cos(dang);
+    spinl[0][1] = -(spinl[1][0] = -sin(dang));
+  
+    tipup = perm_matrix2(MAXCOORD+1,MAXCOORD+1);
+    matcopy(tipup,identmat,SDIM+1,SDIM+1);
+    tipup[0][0] = tipup[2][2] = cos(dang);
+    tipup[0][2] = -(tipup[2][0] = sin(dang));
+  
+    tipdown = perm_matrix2(MAXCOORD+1,MAXCOORD+1);
+    matcopy(tipdown,identmat,SDIM+1,SDIM+1);
+    tipdown[0][0] = tipdown[2][2] = cos(dang);
+    tipdown[0][2] = -(tipdown[2][0] = -sin(dang));
+  
+    clockwise = perm_matrix2(MAXCOORD+1,MAXCOORD+1);
+    matcopy(clockwise,identmat,SDIM+1,SDIM+1);
+    clockwise[1][1] = clockwise[2][2] = cos(dang);
+    clockwise[1][2] = -(clockwise[2][1] = -sin(dang));
+  
+    counterclock = perm_matrix2(MAXCOORD+1,MAXCOORD+1);
+    matcopy(counterclock,identmat,SDIM+1,SDIM+1);
+    counterclock[1][1] = counterclock[2][2] = cos(dang);
+    counterclock[1][2] = -(counterclock[2][1] = sin(dang));
+  
+    /* set magnifying matrix */
+    zoom = perm_matrix2(MAXCOORD+1,MAXCOORD+1);
+    matcopy(zoom,identmat,SDIM+1,SDIM+1);
+    zoom[0][0] = zoom[1][1] = zoom[2][2] = zoomfactor;
+  
+    /* set shrink matrix */
+    shrink = perm_matrix2(MAXCOORD+1,MAXCOORD+1);
+    matcopy(shrink,identmat,SDIM+1,SDIM+1);
+    shrink[0][0] = shrink[1][1] = shrink[2][2] = 1/zoomfactor;
+  
+    /* set translation matrices */
+    transleft = perm_matrix2(MAXCOORD+1,MAXCOORD+1);
+    transup = perm_matrix2(MAXCOORD+1,MAXCOORD+1);
+    transright = perm_matrix2(MAXCOORD+1,MAXCOORD+1);  
+    transdown = perm_matrix2(MAXCOORD+1,MAXCOORD+1);
+  } 
+ 
   set_view_matrix_global();
-  to_focus = dmatrix(0,SDIM,0,SDIM);
-  from_focus = dmatrix(0,SDIM,0,SDIM);
-  for ( n = 0 ; n <= SDIM ; n++ )
-    to_focus[n][n] = from_focus[n][n] = 1.0;
+  matcopy(to_focus,identmat,SDIM+1,SDIM+1);
+  matcopy(from_focus,identmat,SDIM+1,SDIM+1);
 
-
-  /* set up identity matrix */
-  identmat = dmatrix(0,MAXCOORD,0,MAXCOORD);   /* SDIM may be too small */
-  for ( i = 0 ; i <= MAXCOORD ; i++ )
-  {
-    identmat[i][i] = 1.0;
-  }
-
-  /* set rotation matrices */
-  spinr = dmatrix(0,SDIM,0,SDIM);
-  matcopy(spinr,identmat,SDIM+1,SDIM+1);
-  spinr[0][0] = spinr[1][1] = cos(dang);
-  spinr[0][1] = -(spinr[1][0] = sin(dang));
-
-  spinl = dmatrix(0,SDIM,0,SDIM);
-  matcopy(spinl,identmat,SDIM+1,SDIM+1);
-  spinl[0][0] = spinl[1][1] = cos(dang);
-  spinl[0][1] = -(spinl[1][0] = -sin(dang));
-
-  tipup = dmatrix(0,SDIM,0,SDIM);
-  matcopy(tipup,identmat,SDIM+1,SDIM+1);
-  tipup[0][0] = tipup[2][2] = cos(dang);
-  tipup[0][2] = -(tipup[2][0] = sin(dang));
-
-  tipdown = dmatrix(0,SDIM,0,SDIM);
-  matcopy(tipdown,identmat,SDIM+1,SDIM+1);
-  tipdown[0][0] = tipdown[2][2] = cos(dang);
-  tipdown[0][2] = -(tipdown[2][0] = -sin(dang));
-
-  clockwise = dmatrix(0,SDIM,0,SDIM);
-  matcopy(clockwise,identmat,SDIM+1,SDIM+1);
-  clockwise[1][1] = clockwise[2][2] = cos(dang);
-  clockwise[1][2] = -(clockwise[2][1] = -sin(dang));
-
-  counterclock = dmatrix(0,SDIM,0,SDIM);
-  matcopy(counterclock,identmat,SDIM+1,SDIM+1);
-  counterclock[1][1] = counterclock[2][2] = cos(dang);
-  counterclock[1][2] = -(counterclock[2][1] = sin(dang));
-
-  /* set magnifying matrix */
-  zoom = dmatrix(0,SDIM,0,SDIM);
-  matcopy(zoom,identmat,SDIM+1,SDIM+1);
-  zoom[0][0] = zoom[1][1] = zoom[2][2] = zoomfactor;
-
-  /* set shrink matrix */
-  shrink = dmatrix(0,SDIM,0,SDIM);
-  matcopy(shrink,identmat,SDIM+1,SDIM+1);
-  shrink[0][0] = shrink[1][1] = shrink[2][2] = 1/zoomfactor;
-
-  /* set translation matrices */
-  transleft = dmatrix(0,SDIM,0,SDIM);
-  transup = dmatrix(0,SDIM,0,SDIM);
-  transright = dmatrix(0,SDIM,0,SDIM);  
-  transdown = dmatrix(0,SDIM,0,SDIM);
-   
 }
 
 /********************************************************************
@@ -545,7 +552,9 @@ REAL val;
 
 void set_zoom(val)
 REAL val;
-{ zoom[0][0] = zoom[1][1] = zoom[2][2] = val;
+{ zoom[0][0] = zoom[1][1] = val;
+  if ( web.sdim >= 3 ) 
+     zoom[2][2] = val;
 }
 
 /********************************************************************
@@ -576,6 +585,8 @@ void reset_view()
      transleft[0][HOMDIM-1] = -.25;
      transup[1][HOMDIM-1] = .25;
      transdown[1][HOMDIM-1] = -.25;
+     shrink[2][2] = 1.0;
+     zoom[2][2] = 1.0;
   }
 }
 
