@@ -317,7 +317,7 @@ void plain_facets()
   verts = (REAL **)temp_calloc(ctrlpts,sizeof(REAL *));
   if ( web.modeltype == LAGRANGE  )
   { gorder = smooth_graph_flag ? segs : web.lagrange_order;
-    points = dmatrix3(gorder+1,gorder+1,SDIM);
+    points = temp_dmatrix3(gorder+1,gorder+1,SDIM);
     for ( i = 0 ; i < ctrlpts ; i++ ) 
       verts[i] = gdata[i].x;
   }
@@ -659,7 +659,7 @@ void plain_facets()
 
   temp_free((char*)gdata);
   temp_free((char*)verts);
-  if ( points ) free_matrix3(points);
+  if ( points ) free_temp_matrix3(points);
 
 } /* end plain_facets() */
 
@@ -836,7 +836,7 @@ void plain_edges()
   int to_alloc =  ctrlpts > (segs+1) ? ctrlpts : (segs+1);
   gdata = (struct graphdata *)mycalloc(to_alloc,sizeof(struct graphdata));
 
-  verts = dmatrix(0,to_alloc,0,SDIM);
+  verts = temp_dmatrix(0,to_alloc,0,SDIM);
   gdata[0].etype = REGULAR_EDGE;
   MFOR_ALL_EDGES(e_id)
   { if ( breakflag ) break;
@@ -1044,7 +1044,7 @@ void plain_edges()
        }
      }
    }
-  free_matrix(verts);
+  free_temp_matrix(verts);
   myfree((char*)gdata);
 
 }  /* end plain_edges() */
@@ -1223,6 +1223,8 @@ int m;                    /* coordinate number */
     a = SDIM_dot(invper[m],gdata[0].x);
     b = SDIM_dot(invper[m],gdata1[1].x);
     t = (cut - a)/(b - a);
+    if ( t < 0.0 ) t = 0.0;   /* clamp */
+    else if ( t > 1.0 ) t = 1.0;
     for ( k = 0 ; k < SDIM ; k++ )
       gdata1[0].x[k] = gdata[1].x[k] =
           (1 - t)*gdata[0].x[k] + t*gdata1[1].x[k];
@@ -1668,7 +1670,7 @@ facet_id f_id;            /* parent facet */
  for ( i = 0 ; i < FACET_VERTS ; i++ )
   { 
     w = SDIM_dot(invper[m],gdata0[i].x);
-    wrap[i] = (w < .5) ? ((int)floor(w+.000000001)):((int)floor(w-.00000001));
+    wrap[i] = (w < .5) ? ((int)floor(w+.00000001)):((int)floor(w-.00000001));
     /* slight shift to prevent spurious wraps due to rounding */
   }
 
@@ -1732,14 +1734,18 @@ facet_id f_id;            /* parent facet */
         /* calculate new vertices */
         a = SDIM_dot(invper[m],gdata0[oddman].x);
         b = SDIM_dot(invper[m],gdata1[pair1].x);
-        if ( fabs(a-b) < 0.00000000001 ) t = 0.5;
+        if ( fabs(a-b) < 0.00000001 ) t = 0.5;
         else t = (cut - a)/(b - a);
+        if ( t < 0.0 ) t = 0.0;   /* clamp; necessary due to wiggle room above */
+        else if ( t > 1.0 ) t = 1.0;
         for ( k = 0 ; k < SDIM ; k++ )
           gdata1[oddman].x[k] = gdata0[pair1].x[k] =
               (1 - t)*gdata0[oddman].x[k] + t*gdata1[pair1].x[k];
         b = SDIM_dot(invper[m],gdata2[pair2].x);
-        if ( fabs(a-b) < 0.00000000001 ) t = 0.5;
+        if ( fabs(a-b) < 0.00000001 ) t = 0.5;
         else t = (cut - a)/(b - a);
+        if ( t < 0.0 ) t = 0.0;   /* clamp; necessary due to wiggle room above */
+        else if ( t > 1.0 ) t = 1.0;
         for ( k = 0 ; k < SDIM ; k++ )
           gdata2[oddman].x[k] = gdata0[pair2].x[k] = gdata1[pair2].x[k] =
               (1 - t)*gdata0[oddman].x[k] + t*gdata2[pair2].x[k];
@@ -1852,7 +1858,7 @@ void torus_bodies()
   int *bodystarts; /* spots in bodyface list */
 
   gdata = (struct graphdata *)temp_calloc(ctrlpts+1,sizeof(struct graphdata));
-  xxx = dmatrix(0,ctrlpts,0,SDIM);
+  xxx = temp_dmatrix(0,ctrlpts,0,SDIM);
 
   facemax = 2*web.skel[FACET].count;
   allfaces = (struct bodyface *) temp_calloc(facemax,sizeof(struct bodyface));
@@ -2340,7 +2346,7 @@ void torus_bodies()
     temp_free((char *)allfaces);
     temp_free((char *)gdata);
     temp_free((char *)bodystarts);
-    free_matrix(xxx);
+    free_temp_matrix(xxx);
 } /* end torus_bodies() */
 
 
@@ -2362,19 +2368,18 @@ void torus_cells()
   int i; 
   WRAPTYPE wrap = 0;
   edge_id e_id;
-  REAL **per,**invper;
+  REAL **invper;
 
   memset(gdata,0,3*sizeof(struct graphdata));
   
 /*
   if ( web.torus_display_period )
   { 
-    per = web.torus_period;
     invper = web.inverse_display_periods;
   }
   else
 */
-  { per = web.torus_period;
+  { 
     invper = web.inverse_periods;
   }
   MFOR_ALL_FACETS(f_id)
