@@ -259,15 +259,23 @@ char *line;
      strcat(buff,spot);
      if ( b == NULL ) break;
      spot = b + 1;
-     b = strchr(spot,'>'); if ( b ) *b = 0;
+     b = strchr(spot,'>'); 
+     if ( b ) 
+       *b = 0;
      c = strchr(spot,' ');
      if ( c ) *c = 0;
      if ( kb_stricmp(spot,"br") == 0 ) printbuff();
+     else if (kb_stricmp(spot,"sub") == 0 ) {strcat(buff,"_"); }
+     else if (kb_stricmp(spot,"sup") == 0 ) {strcat(buff,"^"); }
      else if (kb_stricmp(spot,"p") == 0 ) {strcat(buff,"\n"); printbuff();}
      else if (kb_strnicmp(spot,"/h",2) == 0 ) {strcat(buff,"\n"); printbuff();}
      else if (kb_stricmp(spot,"li") == 0 ) 
-      { int n; printbuff(); strcat(buff,"  "); 
-         for ( n = 0 ; n < ul_level ; n++ ) strcat(buff,"> ");}
+     { int n; 
+       printbuff(); 
+       strcat(buff,"  "); 
+       for ( n = 0 ; n < ul_level ; n++ ) 
+         strcat(buff,"> ");
+     }
      else if (kb_stricmp(spot,"dt") == 0 ) {printbuff();}
      else if (kb_stricmp(spot,"dd") == 0 ) {strcat(buff,": ");}
      else if (kb_stricmp(spot,"ul") == 0 ) {ul_level++;}
@@ -289,26 +297,242 @@ char *line;
   if ( pre_flag ) strcat(buff,"\n");
 }
 
-/***************************************************************************
+/**************************************************************************
 *
-* function: keyword_help()
+* function: old_html_help()
+*
+* Purpose: old fashioned extraction of help from html doc files,
+*          hopefully superceded by new_text_help()
 *
-* purpose: Print help for keyword from html file.
 */
 #define RELMAX 30
 static struct relinfo { char filename[20];
                       char name[40]; } info[RELMAX];
 
 static char name[100] = "name=\"";
-void keyword_help(keyword)
+
+void old_html_help(keyword)
 char *keyword;
-{ FILE *fd;
-  char helpfilename[100];
-  char line[200];
+{ 
   int printflag = 0;
   int relcount = 0;
   int linecount = 0;
+  int found = 0;
+  char helpfilename[100];
+  FILE *fd;
+  char line[1000];
   int n;
+
+  /* now check html doc files */ 
+
+  helpfilename[0] = 0;
+  fd = path_open("index.htm",NOTDATAFILENAME);
+  if ( fd == NULL )
+  { erroutstring(
+     "Cannot open index.htm.  Are the *.htm doc files on EVOLVERPATH?\n");
+    return; 
+  }
+
+  /* skip header stuff */
+  while ( fgets(line,sizeof(line)-1,fd) )
+     if ( strstr(line,"<a name=\"A\">") ) break;
+
+  while ( fgets(line,sizeof(line)-1,fd)  )
+  { char *sharp = strchr(line,'#');
+    if ( sharp == NULL ) continue;
+    *strchr(sharp,'"') = 0;
+    if ( stricmp(sharp+1,keyword) == 0 )
+    { 
+      *sharp = 0;
+      strcpy(helpfilename,strchr(line,'"')+1);
+    }
+    else if ( strstr(sharp+1,keyword) && (relcount < RELMAX) )
+    { strcpy(info[relcount].filename,strchr(line,'"')+1);
+      strcpy(info[relcount].name,sharp+1);
+      relcount++;
+    }
+  }
+  fclose(fd);
+  if ( found ) goto print_related;
+  if ( helpfilename[0] == 0 )
+  { sprintf(msg,"Cannot find help entry for \"%s\". \n", keyword);
+    outstring(msg);
+    goto print_related;
+  }
+
+
+  fd = path_open(helpfilename,NOTDATAFILENAME);
+  if ( fd == NULL )
+  { perror(helpfilename); return; }
+
+  ul_level = 0;
+  strcpy(name+6,keyword); strcat(name,"\"");
+  while ( fgets(line,sizeof(line)-1,fd)  )
+  {
+     if ( strstr(line,name) ) printflag = 1;
+     else if ( (printflag==1) && 
+        ( ((strstr(line,"<h") || strstr(line,"<H")) && (linecount>2) )
+              || strstr(line,"</dd>") || strstr(line,"EndName") ))
+     { break; }
+
+     strtok(line,"\r\n"); /* remove trailing newline */
+     strcat(line," ");
+     if ( printflag )
+     { if ( !strstr(line,"<a name=") ) linecount++; 
+        do_line(line); 
+     }
+  }
+  printbuff();
+  fclose(fd);
+
+print_related:
+
+  if ( relcount && (strlen(keyword) > 1) )
+  { outstring("Possibly related entries:\n");
+     for ( n = 0 ; n < relcount ; n+=2 ) 
+     { if ( n < relcount-1 )
+          sprintf(msg," %-32s %s\n",info[n].name,info[n+1].name);
+       else sprintf(msg," %-32s \n",info[n].name);
+       outstring(msg);
+     }
+  }
+}
+
+/***************************************************************************
+*
+* Function: new_text_help()
+*
+* Purpose: print help from text help file
+*
+* Return: 0 if can't find help file
+*         1 if found help file
+*/
+
+char *text_help_file = "evhelp.txt";
+
+int new_text_help(keyword)
+char *keyword;
+{ FILE *fd;
+  int found = 0;
+  int keylen = strlen(keyword);
+  char line[1000];
+  int i,n;
+  int relcount = 0;
+
+  /* now check html doc files */ 
+
+  fd = path_open(text_help_file,NOTDATAFILENAME);
+  if ( fd == NULL )
+  { perror(text_help_file);
+    kb_error(4321,"Is the doc subdirectory on EVOLVERPATH?\n",WARNING);
+    return 0; 
+  }
+
+  /* Find keyword in  <--- keyword ---> line */
+  while ( fgets(line,sizeof(line)-1,fd)  )
+  { char *keyspot;
+    if ( strncmp(line,"<---",4) != 0 ) continue;
+    keyspot = strstr(line,keyword);
+    if ( keyspot == NULL ) continue;
+
+    if ( (keyspot[-2] == '-') && (keyspot[keylen+2] == '-') )
+    { int lines_done = 0;
+      int blanklines = 0;
+      /* found exact match */
+      found = 1;   
+
+      /* print nice title bar */
+      outstring("<");
+      for ( i = 1 ; i < (70 - keylen)/2 ; i++ )
+        outstring("-");
+      outstring(" ");
+      outstring(keyword);
+      outstring(" ");
+      for ( i = (70 + keylen)/2 ; i < 70 ;  i++ )
+        outstring("-");
+      outstring(">\n");
+      
+      /* read and echo info */
+      while ( fgets(line,sizeof(line)-1,fd)  )
+      { if ( line[0] != '<' )       
+        { if ( line[1] == 0 )
+          { blanklines++;
+            if ( blanklines < 2 )
+               outstring(line);
+          }
+          else 
+          { outstring(line);
+            blanklines = 0;
+          }
+          lines_done++;
+        }
+        else 
+        { /* skip if just extra title */
+          if ( lines_done > 0 )
+             break;
+        }        
+      }
+
+      /* next title line falls through to relevance recording */
+    }
+    if ( strstr(line,keyword) && (relcount < RELMAX) )
+    { char *spot = strchr(line,' '); 
+      int spaceflag = 0;
+      if ( spot )
+         for ( spot++, i = 0 ; spot[1] != '-' ; spot++, i++ )
+         { info[relcount].name[i] = *spot;
+           if ( *spot == ' ' ) 
+             spaceflag = 1;
+         }
+      if ( spaceflag ) /* enclose in quotes */
+      { char c;
+        c = info[relcount].name[0];
+        info[relcount].name[0] = '"';
+        for ( n = 1 ; n < i ; n++ )
+        { char cc = info[relcount].name[n];
+          info[relcount].name[n] = c;
+          c = cc;
+        }
+        info[relcount].name[i] = c;
+        info[relcount].name[i+1] = '"';
+        i += 2;
+      }
+      info[relcount].name[i] = 0;
+      relcount++;
+    }
+  }
+  fclose(fd);
+  if ( !found ) 
+  { sprintf(msg,"Cannot find help entry for \"%s\". \n", keyword);
+    outstring(msg);
+  }
+
+  if ( relcount && (strlen(keyword) > 1) )
+  { outstring("Possibly related entries:\n");
+    for ( n = 0 ; n < relcount ; n+=2 ) 
+     { if ( n < relcount-1 )
+          sprintf(msg," %-32s %s\n",info[n].name,info[n+1].name);
+       else sprintf(msg," %-32s \n",info[n].name);
+       outstring(msg);
+     }
+  }
+  fclose(fd);
+  return 1;
+}
+
+/***************************************************************************
+*
+* function: keyword_help()
+*
+* purpose: Print help for keyword from html file.
+*/
+
+void keyword_help(keyword)
+char *keyword;
+{ 
+  int printflag = 0;
+  int relcount = 0;
+  int linecount = 0;
   int type;
   int found = 0;
   int entry;
@@ -336,7 +560,7 @@ char *keyword;
   { sprintf(msg,"%s: user-defined element name.\n",keyword);
     outstring(msg);
     found = 1;
-    goto find_related;
+    goto help_exit;
   } 
 
   /* search parameter names */
@@ -402,15 +626,18 @@ char *keyword;
           }
         } 
         else 
-        { if ( gp->type )
+        { if ( gp->flags & OPTIMIZING_PARAMETER )
+           sprintf(msg,"%s: user-defined optimizing parameter\n", keyword);
+          else if ( gp->type )
            sprintf(msg,"%s: user-defined parameter of type %s\n", keyword,
                 datatype_name[gp->type]);
           else
-           sprintf(msg,"%s: user-defined parameter (numeric variable)\n", keyword);
+           sprintf(msg,"%s: user-defined parameter (numeric variable)\n", 
+              keyword);
           outstring(msg);
         }
         found = 1;
-        goto find_related;
+        goto help_exit;
       }
 
         break;
@@ -441,7 +668,7 @@ char *keyword;
              else sprintf(msg,"%s: permanent internal variable.\n",keyword);
              outstring(msg);
              found = 0; /* so prints out regular documentation */
-             goto find_related;
+             goto help_exit;
           } 
           else 
           { sprintf(msg,"%s: permanent user-defined parameter (numeric variable)\n",
@@ -449,7 +676,7 @@ char *keyword;
             outstring(msg);
           }
           found = 1;
-          goto find_related;
+          goto help_exit;
         }
      break;
 
@@ -458,7 +685,7 @@ char *keyword;
         sprintf(msg,"%s: user-defined named quantity.\n",g->name);
         outstring(msg);
         found = 1;
-        goto find_related;} 
+        goto help_exit;} 
         break;
 
       case METHODNAME:
@@ -467,7 +694,7 @@ char *keyword;
         keyword, GEN_QUANT(mi->quant)->name);
         outstring(msg);
         found = 1;
-        goto find_related;} 
+        goto help_exit;} 
         break;
     }
   
@@ -488,86 +715,17 @@ char *keyword;
         strcat(msg,"\n");
         outstring(msg); 
         found = 1;
-        goto find_related;
+        goto help_exit;
       } 
   }
    
 
 
-find_related:
-  /* now check html doc files */ 
+/* find_related: */
 
-  helpfilename[0] = 0;
-  fd = path_open("index.htm",NOTDATAFILENAME);
-  if ( fd == NULL )
-  { erroutstring(
-     "Cannot open index.htm.  Are the *.htm doc files on EVOLVERPATH?\n");
-     goto help_exit; 
-  }
-
-  /* skip header stuff */
-  while ( fgets(line,sizeof(line)-1,fd) )
-     if ( strstr(line,"<a name=\"A\">") ) break;
-
-  while ( fgets(line,sizeof(line)-1,fd)  )
-  { char *sharp = strchr(line,'#');
-    if ( sharp == NULL ) continue;
-    *strchr(sharp,'"') = 0;
-    if ( stricmp(sharp+1,keyword) == 0 )
-    { 
-      *sharp = 0;
-      strcpy(helpfilename,strchr(line,'"')+1);
-    }
-    else if ( strstr(sharp+1,keyword) && (relcount < RELMAX) )
-    { strcpy(info[relcount].filename,strchr(line,'"')+1);
-      strcpy(info[relcount].name,sharp+1);
-      relcount++;
-    }
-  }
-  fclose(fd);
-  if ( found ) goto print_related;
-  if ( helpfilename[0] == 0 )
-  { sprintf(msg,"Cannot find help entry for \"%s\". \n", keyword);
-    outstring(msg);
-    goto print_related;
-  }
-
-
-  fd = path_open(helpfilename,NOTDATAFILENAME);
-  if ( fd == NULL )
-  { perror(helpfilename); goto help_exit; }
-
-  ul_level = 0;
-  strcpy(name+6,keyword); strcat(name,"\"");
-  while ( fgets(line,sizeof(line)-1,fd)  )
-  {
-     if ( strstr(line,name) ) printflag = 1;
-     else if ( (printflag==1) && 
-        ( ((strstr(line,"<h") || strstr(line,"<H")) && (linecount>2) )
-              || strstr(line,"</dd>") || strstr(line,"EndName") ))
-     { break; }
-
-     strtok(line,"\r\n"); /* remove trailing newline */
-     strcat(line," ");
-     if ( printflag )
-     { if ( !strstr(line,"<a name=") ) linecount++; 
-        do_line(line); 
-     }
-  }
-  printbuff();
-  fclose(fd);
-
-print_related:
-
-  if ( relcount && (strlen(keyword) > 1) )
-  { outstring("Possibly related entries:\n");
-     for ( n = 0 ; n < relcount ; n+=2 ) 
-     { if ( n < relcount-1 )
-          sprintf(msg," %-32s %s\n",info[n].name,info[n+1].name);
-       else sprintf(msg," %-32s \n",info[n].name);
-       outstring(msg);
-     }
-  }
+  /* Go to help file */
+  if ( !new_text_help(keyword) )
+    old_html_help(keyword);
 
 help_exit:
 #ifndef NOPIPE
