@@ -681,7 +681,9 @@ struct linsys *M;  /* pointer to empty structure */
 {
   int i,j,k;
   int  Total_entries = 0;
- 
+  struct hess_verlist *vh;
+  vertex_id v_id;
+
   if ( web.lagrange_order > 1 ) { linear_metric_setup(S,M); return; }
 
   if ( ysmp_flag != MINDEG_FACTORING )
@@ -710,16 +712,17 @@ struct linsys *M;  /* pointer to empty structure */
   if ( M->IP == NULL ) M->IP = (int *)temp_calloc(M->N,sizeof(int));
 
   /* go through, vertex by vertex */
-  for ( i = 0, k = 0 ; i <= vhead_count ; i++ )
-  { vertex_id v_id;
+  FOR_ALL_VERTICES(v_id)
+  { 
     REAL area;
+    vh = get_vertex_vhead(v_id);
+
+    if ( vh->freedom == 0 ) continue;
 
-    if ( vhead[i].freedom == 0 ) continue;
-    v_id = vhead[i].v_id;
     area = ((web.representation==STRING)?get_vertex_length_star(v_id):
                                     get_vertex_area_star(v_id))/star_fraction;
 
-    for ( j = 0, k = vhead[i].rownum  ; j < vhead[i].freedom ; j++,k++ )
+    for ( j = 0, k = vh->rownum  ; j < vh->freedom ; j++,k++ )
     { M->IA[k] = M->JA[k] = k + A_OFF;
       M->A[k] = area;
       M->IP[k] = M->P[k] = k;    /* unpermuted */
@@ -920,7 +923,7 @@ if ( optparamcount > 0 )
     { struct hess_verlist *vh[2];
       vertex_id tailv = get_edge_tailv(e_id);
       vertex_id headv = get_edge_headv(e_id);
-      if ( loc_ordinal(ti) > loc_ordinal(hi) )
+      if ( loc_ordinal(tailv) > loc_ordinal(headv) )
       { vh[0] = get_vertex_vhead(headv);
         vh[1] = get_vertex_vhead(tailv);
       }
@@ -975,7 +978,7 @@ if ( optparamcount > 0 )
               for ( kk = 0 ; kk < vv->freedom ; kk++ )  
               { col = vv->rownum + kk + A_OFF ;
                 end = M->IA[v->rownum+jj+1]-A_OFF;
-                for ( j = M->IA[v->rownum+jj]+1-A_OFF ; ; j++ )
+                for ( j = M->IA[v->rownum+jj]+1-A_OFF ; (int)j < end ; j++ )
                   if ( (M->JA[j] < A_OFF) || (M->JA[j] == col)  )
                   { M->JA[j] = col; 
                     M->A[j] += cfudge*mat[jj][kk]*len/6*linear_metric_mix;
