@@ -24,6 +24,9 @@
 #define dmatrix(a,b,c,d) kb_dmatrix(a,b,c,d,__FILE__,__LINE__)
 #define dmatrix3(a,b,c) kb_dmatrix3(a,b,c,__FILE__,__LINE__)
 #define dmatrix4(a,b,c,d) kb_dmatrix4(a,b,c,d,__FILE__,__LINE__)
+#define temp_dmatrix(a,b,c,d) kb_temp_dmatrix(a,b,c,d,__FILE__,__LINE__)
+#define temp_dmatrix3(a,b,c) kb_temp_dmatrix3(a,b,c,__FILE__,__LINE__)
+#define temp_dmatrix4(a,b,c,d) kb_temp_dmatrix4(a,b,c,d,__FILE__,__LINE__)
 #define my_list_calloc(a,b,c) list_calloc(a,b,c,__FILE__,__LINE__)
 #define my_list_realloc(a,b,c) list_realloc(a,b,c,__FILE__,__LINE__)
 #else
@@ -34,6 +37,9 @@
 #define dmatrix(a,b,c,d) kb_dmatrix(a,b,c,d)
 #define dmatrix3(a,b,c) kb_dmatrix3(a,b,c)
 #define dmatrix4(a,b,c,d) kb_dmatrix4(a,b,c,d)
+#define temp_dmatrix(a,b,c,d) kb_temp_dmatrix(a,b,c,d)
+#define temp_dmatrix3(a,b,c) kb_temp_dmatrix3(a,b,c)
+#define temp_dmatrix4(a,b,c,d) kb_temp_dmatrix4(a,b,c,d)
 #define my_list_calloc(a,b,c) list_calloc(a,b,c)
 #define my_list_realloc(a,b,c) list_realloc(a,b,c)
 #endif
@@ -143,11 +149,13 @@ char *getenv();
 /* MAXALLOC is maximum size allocable by calloc() */
 #define    MAXALLOC 0x7FFFFFFFL
 /* Some don't declare calloc, getenv, and bsearch in header files */
+/* Commenting these out now, since too many conflicts with headers.
 #ifndef MAC_OS_X
 char *calloc();
 char *getenv();
 char *bsearch();
 #endif
+*/
 /* PATHCHAR is name-separating character in paths */
 #define PATHCHAR '/'
 /* ENVPATHCHAR is the path separating character in environment strings */
@@ -413,7 +421,9 @@ char *getenv();
 #define farcalloc(num,size)  (void far *)halloc(num,size)
 #define farfree(ptr)            hfree((void huge *)ptr)
 #define FCAST (int(*)(const void*,const void *))
-#define NOPIPE
+#define NOPIPEXX
+#define popen _popen
+#define pclose _pclose
 #define _export
 #define FPRESET _fpreset()
 /* since graphics in parallel thread */
@@ -441,7 +451,7 @@ char *getenv();
 #define    MAXALLOC 0x7FFFFFFFL
 #define PATHCHAR '\\'
 #define ENVPATHCHAR ";"
-#define NOPIPE
+#define NOPIPEXX
 #pragma hdrstop
 /* so qsort() and bsearch() below agree with prototypes */
 #define FCAST (int(*)(const void*,const void *))
@@ -489,9 +499,6 @@ char *getenv();
 typedef int DY_OFFSET ;
 
 
-#include "model.h"
-
-#include "storage.h"
 
 /* Some don't have these manifest constants in math.h (namely, Microsoft) */
 #ifndef M_LN2
@@ -546,6 +553,27 @@ typedef int DY_OFFSET ;
 #define ARGS(x) x
 #endif
 
+/* for converting old-style function defs to new */
+#ifdef __cplusplus
+#define ARGS1(old,a)  (a)
+#define ARGS2(old,a,b)  (a,b)
+#define ARGS3(old,a,b,c)  (a,b,c)
+#define ARGS4(old,a,b,c,d)  (a,b,c,d)
+#define ARGS5(old,a,b,c,d,e)  (a,b,c,d,e)
+#define ARGS6(old,a,b,c,d,e,f)  (a,b,c,d,e,f)
+#define CONST const
+#else
+#define ARGS1(old,a)  old  a;
+#define ARGS2(old,a,b)  old a; b;
+#define ARGS3(old,a,b,c)  old a; b; c;
+#define ARGS4(old,a,b,c,d)  old a; b; c; d;
+#define ARGS5(old,a,b,c,d,e)  old a; b; c; d; e;
+#define ARGS6(old,a,b,c,d,e,f)  old a; b; c; d; e; f;
+#ifndef CONST
+#define CONST
+#endif
+#endif
+
 #ifdef ENABLE_DLL
 #include <dlfcn.h>
 #endif
@@ -561,12 +589,20 @@ typedef int DY_OFFSET ;
 #endif
 
 /* Evolver header files */
+#ifdef __cplusplus
+extern "C" {
+#endif
+#include "model.h"
+#include "storage.h"
 #include "skeleton.h"
 #include "extern.h"
 #include "express.h"
 #include "quantity.h"
 #include "web.h"
 #include "lex.h"
+#ifdef __cplusplus
+}
+#endif
 #include "proto.h"
 
 /* in case of non-parallel machines */
