@@ -30,7 +30,6 @@ void iterate()
   int poptimes = 0;
   int old_flag = iterate_flag;
   REAL old_energy = web.total_energy; /* for estimate */
-  int base_volgrads_flag = 0; /* whether volgrads calculated at 0 scale */
 
 #ifndef MPI_EVOLVER
   if ( web.skel[VERTEX].count == 0 )
@@ -49,7 +48,6 @@ void iterate()
   if ( normal_motion_flag ) begin_normal_motion();
   calc_all_grads(CALC_FORCE|CALC_VOLGRADS);
   volgrads_changed_flag = 0;
-  base_volgrads_flag = 1;
 
   save_coords(&saved,SAVE_IN_ATTR); /* in case of disaster */
   if ( conj_grad_flag )
@@ -84,7 +82,6 @@ void iterate()
     {  calc_all_grads(CALC_VOLGRADS);
        calc_leftside();
        volgrads_changed_flag = 0;
-       base_volgrads_flag = 1;
     }
     web.scale = 0.0; /* to do restoring force */
     move_vertices(TEST_MOVE,web.scale);  /* moving by current scale */
@@ -106,7 +103,6 @@ void iterate()
     {  calc_all_grads(CALC_VOLGRADS);
        calc_leftside();
        volgrads_changed_flag = 0;
-       base_volgrads_flag = 1;
     }
 
     web.scale = tempscale;
@@ -129,7 +125,6 @@ void iterate()
         {  calc_all_grads(CALC_VOLGRADS);
            calc_leftside();
            volgrads_changed_flag = 0;
-           base_volgrads_flag = 1;
         }
         if ( !is_finite(energy2) ) 
            { web.scale /= 2; goto have_scale;} /* use finite motion */ 
@@ -164,7 +159,6 @@ void iterate()
          { calc_all_grads(CALC_VOLGRADS);
            calc_leftside();
            volgrads_changed_flag = 0;
-           base_volgrads_flag = 1;
          }
       }
       while ( energy1 > energy0 );
@@ -203,15 +197,16 @@ have_scale:
 
 
   if ( web.jiggle_flag )  jiggle(); 
-  while ( autopop_flag || autochop_flag ) 
-  { autopop_detect(web.scale);
+  if( autopop_flag || autochop_flag ) 
+  { autopop_init();
+    autopop_detect(web.scale);
     if ( autopop_count || autochop_count )
     { autopop_pop();
       autochop_chop();
-      if ( poptimes++ >= 10 ) break;  
-      calc_all_grads(CALC_FORCE|CALC_VOLGRADS);
+      /*if ( poptimes++ >= 10 ) break;  */
+      /*calc_all_grads(CALC_FORCE|CALC_VOLGRADS); */
     }
-    else break;
+    autopop_cleanup();   
   }
 
   if ( fixed_constraint_flag ) calc_content(Q_FIXED);
@@ -445,9 +440,10 @@ void fix_vertices()
     if ( normal_motion_flag )
     { /* project to normal */
       REAL d;
-      REAL *normal = vertex_normals[loc_ordinal(v_id)];
+      int vnum = loc_ordinal(v_id);
+      REAL *normal = vertex_normals[vnum];
 
-      d = SDIM_dot(force,vertex_normals[loc_ordinal(v_id)]);
+      d = SDIM_dot(force,normal);
       for ( j = 0 ; j < SDIM ; j++ )
          force[j] = d*normal[j];
     }
@@ -744,7 +740,7 @@ int mode2; /* TEST_MOVE or ACTUAL_MOVE */
       }
       volume_restore(stepsize,mode2);
       global_timestamp++;
-      if ( calc_count++ > 10 )
+      if ( calc_count++ > 10 ) 
       { 
         sprintf(errmsg,"Volume or quantity constraints don't converge in 10 projections.\nTotal difference %g times tolerance\n",(DOUBLE)diff);
         if ( mode2 == ACTUAL_MOVE )
@@ -1285,18 +1281,18 @@ void homothety()
   if ( square_curvature_flag )
      scale = 1/pow(web.total_area/homothety_target,1.0/web.dimension);
   else
-     {
-        /* be sure to have current volumes */
-        calc_content(Q_FIXED); 
-        FOR_ALL_BODIES(b_id)
-          vol += get_body_volume(b_id);
-        if ( web.representation == STRING ) scale = 1/sqrt(vol/homothety_target);
-        else scale = 1/pow(vol/homothety_target,1/3.0);
-     }
+  {
+    /* be sure to have current volumes */
+    calc_content(Q_FIXED); 
+    FOR_ALL_BODIES(b_id)
+        vol += get_body_volume(b_id);
+    if ( web.representation == STRING ) scale = 1/sqrt(vol/homothety_target);
+    else scale = 1/pow(vol/homothety_target,1/3.0);
+  }
   FOR_ALL_VERTICES(v_id)
-     {  x = get_coord(v_id);
-         for ( i = 0 ; i < SDIM ; i++ ) x[i] *= scale;
-     }
+  {  x = get_coord(v_id);
+     for ( i = 0 ; i < SDIM ; i++ ) x[i] *= scale;
+  }
 }
 
 /***********************************************************************
