@@ -405,7 +405,7 @@ int mode; /* METHOD_VALUE, METHOD_GRADIENT, or METHOD_HESSIAN */
 
   int i,j,ii,jj;
   
-  REAL detC, detr, JX;
+  REAL detr;
 
 
   lambda = *(REAL*)get_extra(f_info->id,lambda_attr);
@@ -440,8 +440,6 @@ int mode; /* METHOD_VALUE, METHOD_GRADIENT, or METHOD_HESSIAN */
   LW = LambertW(2.0*mu/detr/lambda*exp(2.0*mu/lambda));
  
   c33 = exp((-LW*lambda+2.0*mu)/lambda)/detr;
-  detC = c33*(c11 * c22 - c12 * c21);
-  JX = sqrt(detC);
   
   tmp1 = log(exp(-LW+2.0*mu/lambda));
   tmp2 = exp(-(LW*lambda-2.0*mu)/lambda);
@@ -857,3 +855,359 @@ struct qinfo *f_info;
  return Neo_Hookean_all(f_info,METHOD_HESSIAN);
 }
 
+
+/***********************************************************************/
+/************************************************************************
+     Named method: general_linear_elastic
+
+     Linear elastic strain energy on facets, fully general model.
+
+     Let S be Gram matrix of unstrained facet (dots of sides).
+     Let Q be the inverse of S.
+     Let F be Gram matrix of strained facet.
+     Let C = (FQ-I)/2, the Cauchy-Green strain tensor.
+     Let v be Poisson ratio.
+     Then energy density is 
+        C_ij K_ijkl C_kl / 2
+     Each facet has extra attribute array elastic_coeff[6] in order
+     { xx*xx, yy*yy, xx*yy, xx*xy, yy*xy, xy*xy }
+     for its K_ijkl (should not include factors of 2 and 4 due to symmetry)
+     and extra attribute array form_factors[3] = {s11,s12,s22}
+     where sij = dot(si,sj) and s1 = (v1-v0), s2 = (v2-v0).
+
+************************************************************************/
+
+#define ELASTIC_COEFF_NAME "elastic_coeff"
+#define ELASTIC_BASE_NAME "elastic_basis"
+static int coeff_attr; /* number of elastic_coeff extra attribute */
+static int elastic_base_attr; /* number of form_factors extra attribute */
+
+/***************************************************************
+*
+*  function: general_linear_elastic_init()
+*
+*  purpose: Make sure needed extra attributes are present.
+*              
+*
+*/
+
+void general_linear_elastic_init(mode,mi)
+int mode; /* energy or gradient */
+struct method_instance *mi;
+{ struct extra *ex;
+
+  if ( web.modeltype != LINEAR )
+     kb_error(4015,"general_linear_elastic method only for LINEAR model.\n",
+       RECOVERABLE);
+
+  if ( web.dimension != 2 )
+     kb_error(4022,"general_linear_elastic method only for SOAPFILM model.\n",
+        RECOVERABLE);
+
+  /* extra edge atribute */
+  coeff_attr = find_attribute(FACET,ELASTIC_COEFF_NAME);
+  if ( coeff_attr < 0 ) /* not found */
+     kb_error(2151,"Facet extra attribute elastic_coeff missing. Needed by linear_elastic.\n",RECOVERABLE);
+
+  elastic_base_attr = find_attribute(FACET,ELASTIC_BASE_NAME);
+  if ( elastic_base_attr < 0 ) /* not found */
+     kb_error(2152,"Facet extra attribute form_factors real[3] missing. Needed by linear_elastic.\n",RECOVERABLE);
+
+  ex = &EXTRAS(FACET)[elastic_base_attr];
+  if ( ex->adim != 2 || ex->sizes[0] != 2  || ex->sizes[1] != 2 )
+     kb_error(2153,
+      "Facet extra attribute elastic_base must have dimension [2][2].\n",
+        RECOVERABLE);
+
+  ex = &EXTRAS(FACET)[coeff_attr];
+  if ( ex->adim != 1 || ex->sizes[0] != 6 )
+    kb_error(2153,
+      "Facet extra attribute elastic_coeff must have 1 dimension and size 6.\n",
+        RECOVERABLE);
+}
+
+/************************************************************************
+*
+* function: general_linear_elastic_all()
+*
+* purpose: energy, gradient, and hessian for linear_elastic method.
+*/
+REAL general_linear_elastic_all ARGS((struct qinfo *,int));
+
+REAL general_linear_elastic_all(f_info,mode)
+struct qinfo *f_info;
+int mode; /* METHOD_VALUE, METHOD_GRADIENT, or METHOD_HESSIAN */
+{
+  REAL s[2][2],*sptr;  /* pointer to extra attributes */
+  REAL **side; 
+  REAL q11,q12,q21,q22;  /* Q entries */
+  REAL det;    /* det S */
+  REAL area;  /* reference area of facet */
+  REAL * coeff;  /* elastic coeff extra attribute */
+  REAL elastic_coeff[2][2][2][2];
+  REAL f11,f12,f21,f22;
+  REAL c11,c12,c21,c22;
+  REAL energy;
+  REAL dc11dv[FACET_VERTS][MAXCOORD];
+  REAL dc12dv[FACET_VERTS][MAXCOORD];
+  REAL dc21dv[FACET_VERTS][MAXCOORD];
+  REAL dc22dv[FACET_VERTS][MAXCOORD];
+  REAL ddc11dv[FACET_VERTS][MAXCOORD][FACET_VERTS];
+  REAL ddc12dv[FACET_VERTS][MAXCOORD][FACET_VERTS];
+  REAL ddc21dv[FACET_VERTS][MAXCOORD][FACET_VERTS];
+  REAL ddc22dv[FACET_VERTS][MAXCOORD][FACET_VERTS];
+  int i,j,ii,jj;
+
+  coeff = (REAL*)get_extra(f_info->id,coeff_attr);
+  elastic_coeff[0][0][0][0] = coeff[0];
+  elastic_coeff[1][1][1][1] = coeff[1];
+  elastic_coeff[0][0][1][1] = coeff[2];
+  elastic_coeff[1][1][0][0] = coeff[2];
+  elastic_coeff[0][0][0][1] = coeff[3]/2;
+  elastic_coeff[0][0][1][0] = coeff[3]/2;
+  elastic_coeff[0][1][0][0] = coeff[3]/2;
+  elastic_coeff[1][0][0][0] = coeff[3]/2;
+  elastic_coeff[1][1][1][0] = coeff[4]/2;
+  elastic_coeff[0][1][1][1] = coeff[4]/2;
+  elastic_coeff[1][0][1][1] = coeff[4]/2;
+  elastic_coeff[1][1][0][1] = coeff[4]/2;
+  elastic_coeff[1][0][1][0] = coeff[5]/4;
+  elastic_coeff[1][0][0][1] = coeff[5]/4;
+  elastic_coeff[0][1][0][1] = coeff[5]/4;
+  elastic_coeff[0][1][1][0] = coeff[5]/4;
+
+  /* get unstrained sides in columns of S and get inverse */
+  sptr = (REAL*)get_extra(f_info->id,elastic_base_attr);
+  s[0][0] = sptr[0]; s[1][0] = sptr[1]; s[0][1] = sptr[2]; s[1][1] = sptr[3];
+  det = s[0][0]*s[1][1] - s[0][1]*s[1][0];
+  if ( det == 0.0 )
+  { if ( mode == METHOD_VALUE ) return 0.0;
+    sprintf(errmsg,
+      "general_linear_elastic: Facet %s has unstrained area 0.\n",
+        ELNAME(f_info->id));
+    kb_error(2154,errmsg,RECOVERABLE);
+  }
+  area = fabs(det)/2;
+  q11 = s[1][1]/det; q12 = -s[0][1]/det; q21 = -s[1][0]/det; q22 = s[0][0]/det;
+
+  /* Gram matrix of strained sides */
+  side = f_info->sides[0];
+  f11 = SDIM_dot(side[0],side[0]);
+  f21 = f12 = SDIM_dot(side[0],side[1]);
+  f22 = SDIM_dot(side[1],side[1]);
+
+  /* Strain matrix */
+  c11 = (q11*f11*q11 + q11*f12*q21 + q21*f21*q11 + q21*f22*q21 - 1)/2;
+  c12 = (q11*f11*q12 + q11*f12*q22 + q21*f21*q12 + q21*f22*q22)/2;
+  c21 = c12;
+  c22 = (q12*f11*q12 + q12*f12*q22 + q22*f21*q12 + q22*f22*q22 - 1)/2;
+
+  energy = c11*c11*elastic_coeff[0][0][0][0] 
+         + c11*c12*elastic_coeff[0][0][0][1] 
+         + c11*c22*elastic_coeff[0][0][1][1] 
+         + c11*c21*elastic_coeff[0][0][1][0] 
+         + c12*c11*elastic_coeff[0][1][0][0] 
+         + c12*c12*elastic_coeff[0][1][0][1] 
+         + c12*c22*elastic_coeff[0][1][1][1] 
+         + c12*c21*elastic_coeff[0][1][1][0] 
+         + c21*c11*elastic_coeff[1][0][0][0] 
+         + c21*c12*elastic_coeff[1][0][0][1] 
+         + c21*c22*elastic_coeff[1][0][1][1] 
+         + c21*c21*elastic_coeff[1][0][1][0] 
+         + c22*c11*elastic_coeff[1][1][0][0] 
+         + c22*c12*elastic_coeff[1][1][0][1] 
+         + c22*c22*elastic_coeff[1][1][1][1] 
+         + c22*c21*elastic_coeff[1][1][1][0] ;
+  energy *= area/2;
+   
+
+  if ( mode == METHOD_VALUE ) return energy;
+
+  /* gradient */
+
+  for ( i = 0 ; i < SDIM  ; i++ )
+  {
+
+  dc11dv[1][i] = (q11*2*side[0][i]*q11 + q11*side[1][i]*q21 + q21*side[1][i]*q11                  )/2;
+  dc11dv[2][i] = (q11*side[0][i]*q21 + q21*side[0][i]*q11 + q21*2*side[1][i]*q21                 )/2;
+  dc12dv[1][i] = (q11*2*side[0][i]*q12 + q11*side[1][i]*q22 + q21*side[1][i]*q12                 )/2;
+  dc12dv[2][i] = (q11*side[0][i]*q22 + q21*side[0][i]*q12 + q21*2*side[1][i]*q22                 )/2;
+  dc21dv[1][i] = dc12dv[1][i];
+  dc21dv[2][i] = dc12dv[2][i];
+  dc22dv[1][i] = (q12*2*side[0][i]*q12 + q12*side[1][i]*q22 + q22*side[1][i]*q12                 )/2;
+  dc22dv[2][i] = (q12*side[0][i]*q22 + q22*side[0][i]*q12 + q22*2*side[1][i]*q22                 )/2;
+     dc11dv[0][i] = -(dc11dv[1][i] + dc11dv[2][i]);
+     dc12dv[0][i] = -(dc12dv[1][i] + dc12dv[2][i]);
+     dc21dv[0][i] = -(dc21dv[1][i] + dc21dv[2][i]);
+     dc22dv[0][i] = -(dc22dv[1][i] + dc22dv[2][i]);
+  }
+
+
+  for ( j = 0 ; j < FACET_VERTS  ; j++ )
+     for ( i = 0 ; i < SDIM  ; i++ )
+     { f_info->grad[j][i] = area*(
+           c11*dc11dv[j][i]*elastic_coeff[0][0][0][0] 
+         + c11*dc12dv[j][i]*elastic_coeff[0][0][0][1] 
+         + c11*dc22dv[j][i]*elastic_coeff[0][0][1][1] 
+         + c11*dc21dv[j][i]*elastic_coeff[0][0][1][0] 
+         + c12*dc11dv[j][i]*elastic_coeff[0][1][0][0] 
+         + c12*dc12dv[j][i]*elastic_coeff[0][1][0][1] 
+         + c12*dc22dv[j][i]*elastic_coeff[0][1][1][1] 
+         + c12*dc21dv[j][i]*elastic_coeff[0][1][1][0] 
+         + c21*dc11dv[j][i]*elastic_coeff[1][0][0][0] 
+         + c21*dc12dv[j][i]*elastic_coeff[1][0][0][1] 
+         + c21*dc22dv[j][i]*elastic_coeff[1][0][1][1] 
+         + c21*dc21dv[j][i]*elastic_coeff[1][0][1][0] 
+         + c22*dc11dv[j][i]*elastic_coeff[1][1][0][0] 
+         + c22*dc12dv[j][i]*elastic_coeff[1][1][0][1] 
+         + c22*dc22dv[j][i]*elastic_coeff[1][1][1][1] 
+         + c22*dc21dv[j][i]*elastic_coeff[1][1][1][0]);
+      /* don't need to divide by 2 here, since only doing 
+         derivative of second term and using symmetry */
+     }
+
+  if ( mode == METHOD_GRADIENT ) return energy;
+
+  /* hessian */
+
+  for ( i = 0 ; i < SDIM  ; i++ )
+  {
+     ddc11dv[1][i][1] = (q11*2*q11)/2;
+     ddc11dv[1][i][2] = ( q11*q21 + q21*q11 )/2;
+
+     ddc11dv[2][i][1] = (q11*q21 + q21*q11 )/2;
+     ddc11dv[2][i][2] = (q21*2*q21)/2;
+   
+     ddc12dv[1][i][1] = (q11*2*q12 )/2;
+     ddc12dv[1][i][2] = ( q11*q22 + q21*q12 )/2;
+
+  ddc12dv[2][i][1] = (q11*q22 + q21*q12)/2;
+  ddc12dv[2][i][2] = (q21*2*q22 )/2;
+
+  ddc21dv[1][i][1] = ddc12dv[1][i][1];
+  ddc21dv[1][i][2] = ddc12dv[1][i][2];
+
+     ddc21dv[2][i][1] = ddc12dv[2][i][1];
+     ddc21dv[2][i][2] = ddc12dv[2][i][2];
+
+     ddc22dv[1][i][1] = (q12*2*q12)/2;
+     ddc22dv[1][i][2] = (q12*q22 + q22*q12)/2;
+
+     ddc22dv[2][i][1] = (q12*q22 + q22*q12)/2;
+     ddc22dv[2][i][2] = (q22*2*q22)/2;
+
+     for ( j = 1 ; j < FACET_VERTS;  j++ )
+     { 
+        ddc11dv[0][i][j] = -(ddc11dv[1][i][j] + ddc11dv[2][i][j]);
+        ddc12dv[0][i][j] = -(ddc12dv[1][i][j] + ddc12dv[2][i][j]);
+        ddc21dv[0][i][j] = -(ddc21dv[1][i][j] + ddc21dv[2][i][j]);
+        ddc22dv[0][i][j] = -(ddc22dv[1][i][j] + ddc22dv[2][i][j]);
+        ddc11dv[j][i][0] = -(ddc11dv[j][i][1] + ddc11dv[j][i][2]);
+        ddc12dv[j][i][0] = -(ddc12dv[j][i][1] + ddc12dv[j][i][2]);
+        ddc21dv[j][i][0] = -(ddc21dv[j][i][1] + ddc21dv[j][i][2]);
+        ddc22dv[j][i][0] = -(ddc22dv[j][i][1] + ddc22dv[j][i][2]);
+     }
+     ddc11dv[0][i][0] = -(ddc11dv[1][i][0] + ddc11dv[2][i][0]);
+     ddc12dv[0][i][0] = -(ddc12dv[1][i][0] + ddc12dv[2][i][0]);
+     ddc21dv[0][i][0] = -(ddc21dv[1][i][0] + ddc21dv[2][i][0]);
+     ddc22dv[0][i][0] = -(ddc22dv[1][i][0] + ddc22dv[2][i][0]);
+  }
+
+  for ( j = 0 ; j < FACET_VERTS  ; j++ )
+     for ( i = 0 ; i < SDIM  ; i++ )
+        for ( jj = 0 ; jj < FACET_VERTS  ; jj++ )
+        {
+          f_info->hess[j][jj][i][i] += area*(
+           c11*ddc11dv[j][i][jj]*elastic_coeff[0][0][0][0] 
+         + c11*ddc12dv[j][i][jj]*elastic_coeff[0][0][0][1] 
+         + c11*ddc22dv[j][i][jj]*elastic_coeff[0][0][1][1] 
+         + c11*ddc21dv[j][i][jj]*elastic_coeff[0][0][1][0] 
+         + c12*ddc11dv[j][i][jj]*elastic_coeff[0][1][0][0] 
+         + c12*ddc12dv[j][i][jj]*elastic_coeff[0][1][0][1] 
+         + c12*ddc22dv[j][i][jj]*elastic_coeff[0][1][1][1] 
+         + c12*ddc21dv[j][i][jj]*elastic_coeff[0][1][1][0] 
+         + c21*ddc11dv[j][i][jj]*elastic_coeff[1][0][0][0] 
+         + c21*ddc12dv[j][i][jj]*elastic_coeff[1][0][0][1] 
+         + c21*ddc22dv[j][i][jj]*elastic_coeff[1][0][1][1] 
+         + c21*ddc21dv[j][i][jj]*elastic_coeff[1][0][1][0] 
+         + c22*ddc11dv[j][i][jj]*elastic_coeff[1][1][0][0] 
+         + c22*ddc12dv[j][i][jj]*elastic_coeff[1][1][0][1] 
+         + c22*ddc22dv[j][i][jj]*elastic_coeff[1][1][1][1] 
+         + c22*ddc21dv[j][i][jj]*elastic_coeff[1][1][1][0]);
+
+
+          for ( ii = 0 ; ii < SDIM  ; ii++ )
+             f_info->hess[j][jj][i][ii] += area*(
+                 dc11dv[jj][ii]*dc11dv[j][i]*elastic_coeff[0][0][0][0] 
+               + dc11dv[jj][ii]*dc12dv[j][i]*elastic_coeff[0][0][0][1] 
+               + dc11dv[jj][ii]*dc22dv[j][i]*elastic_coeff[0][0][1][1] 
+               + dc11dv[jj][ii]*dc21dv[j][i]*elastic_coeff[0][0][1][0] 
+               + dc12dv[jj][ii]*dc11dv[j][i]*elastic_coeff[0][1][0][0] 
+               + dc12dv[jj][ii]*dc12dv[j][i]*elastic_coeff[0][1][0][1] 
+               + dc12dv[jj][ii]*dc22dv[j][i]*elastic_coeff[0][1][1][1] 
+               + dc12dv[jj][ii]*dc21dv[j][i]*elastic_coeff[0][1][1][0] 
+               + dc21dv[jj][ii]*dc11dv[j][i]*elastic_coeff[1][0][0][0] 
+               + dc21dv[jj][ii]*dc12dv[j][i]*elastic_coeff[1][0][0][1] 
+               + dc21dv[jj][ii]*dc22dv[j][i]*elastic_coeff[1][0][1][1] 
+               + dc21dv[jj][ii]*dc21dv[j][i]*elastic_coeff[1][0][1][0] 
+               + dc22dv[jj][ii]*dc11dv[j][i]*elastic_coeff[1][1][0][0] 
+               + dc22dv[jj][ii]*dc12dv[j][i]*elastic_coeff[1][1][0][1] 
+               + dc22dv[jj][ii]*dc22dv[j][i]*elastic_coeff[1][1][1][1] 
+               + dc22dv[jj][ii]*dc21dv[j][i]*elastic_coeff[1][1][1][0]);
+
+         }
+
+  return energy;
+}  /* end general_linear_elastic_all() */
+
+/**************************************************************
+*
+*  function: general_linear_elastic_energy()
+*
+*  purpose: calculates energy of one facet due to potential
+*
+*  input: info about vertex is in qinfo structure.
+*
+*/
+
+REAL general_linear_elastic_energy(f_info)
+struct qinfo *f_info;
+{
+ return general_linear_elastic_all(f_info,METHOD_VALUE);
+}
+
+
+
+/**************************************************************
+*
+*  function: general_linear_elastic_gradient()
+*
+*  purpose: calculates gradient of one facet due to potential
+*
+*  input: info about vertex is in qinfo structure.
+*
+*/
+
+REAL general_linear_elastic_gradient(f_info)
+struct qinfo *f_info;
+{
+ return general_linear_elastic_all(f_info,METHOD_GRADIENT);
+}
+
+
+/**************************************************************
+*
+*  function: general_linear_elastic_hessian()
+*
+*  purpose: calculates hessian of one facet due to potential
+*
+*  input: info about vertex is in qinfo structure.
+*
+*/
+
+REAL general_linear_elastic_hessian(f_info)
+struct qinfo *f_info;
+{
+ return general_linear_elastic_all(f_info,METHOD_HESSIAN);
+}
+
