@@ -223,6 +223,7 @@ facetedge_id fe_a;  /* removable vertex is at tail */
   facetedge_id fe_g = get_next_facet(fe_e);
   facetedge_id fe_h = get_next_edge(fe_g);
   facetedge_id fe_i = get_next_edge(fe_h);
+  edge_id e_id = get_fe_edge(fe_a);
   facet_id f_id;
   body_id b_id;
 
@@ -231,6 +232,15 @@ facetedge_id fe_a;  /* removable vertex is at tail */
     outstring(msg);
   }
 
+
+/*
+  if ( eliminate_edge(e_id) )
+  { free_element(e_id);
+    return 1;
+  }
+  else return -1;
+*/
+
   /* some checks on legality */
   if ( get_vertex_evalence(v_id) != 3 )  /* not lonesome star */
   { if ( verbose_flag )
@@ -1169,6 +1179,8 @@ int rebody()
   FOR_ALL_BODIES(b_id)
     { 
       fe = get_body_fe(b_id);
+      if ( !valid_id(fe) )
+        continue;
       f_id = get_fe_facet(fe);
       stack[stacktop++] = f_id;
       bf = (struct fface *)bsearch((char *)&f_id,(char *)flist,
@@ -1213,10 +1225,10 @@ int rebody()
       flist[i].wrapflag = 1;
       f_id = flist[i].f_id;
       old_b = get_facet_body(f_id);
-      b_id = dup_body(old_b);
+      b_id = dup_body(b_id);
       new_bodies++;
-      set_body_fe(b_id,get_facet_fe(f_id));
-      set_facet_body(f_id,b_id);
+      set_body_fe(b_id,NULLID);  /* clear dup'ed value */
+      set_facet_body(f_id,b_id);  /* takes care of set_body_fe() for first one */
       stack[stacktop++] = f_id;
       faces_left--;
       /* pull stuff off stack till empty */
@@ -1613,26 +1625,21 @@ int dissolve_facet(f_id)
 facet_id f_id;
 { facetedge_id fe,fe_id;
   edge_id e_id;
-  int fvalence;
+  body_id b1,b2;
 
   if ( !valid_element(f_id) ) return 0;
 
-  fvalence = valid_id(get_facet_body(f_id)) ? 1 : 0; 
-  fvalence += valid_id(get_facet_body(inverse_id(f_id))) ? 1 : 0;
+  b1 = get_facet_body(f_id);
+  b2 = get_facet_body(inverse_id(f_id));
 
-  if ( fvalence > 1 )
+  if ( !equal_id(b1,b2) && valid_id(b1) && valid_id(b2) )
   { if ( verbose_flag )
-    { sprintf(msg,"Not dissolving facet %s since on two bodies\n",ELNAME(f_id));
+    { sprintf(msg,"Not dissolving facet %s since on two different bodies\n",ELNAME(f_id));
       outstring(msg);
     }
     return 0;
   }
 
-/*
-  if ( (fvalence == 1) && (top_timestamp > bfacet_timestamp) )
-    make_bfacet_lists();
-*/
-
   if ( verbose_flag )
   { sprintf(msg,"Dissolving facet %s\n",ELNAME(f_id));
     outstring(msg);
@@ -1989,16 +1996,16 @@ edge_id keepv,throwv;
   facetedge_id match_kfe=NULLID,match_tfe=NULLID;
 
   if ( (web.modeltype != LINEAR) && (web.modeltype != QUADRATIC) )
-    kb_error(3913,"merge_vertex only in LINEAR or QUADRATIC model so far.\n",
+    kb_error(3913,"vertex_merge only in LINEAR or QUADRATIC model so far.\n",
         RECOVERABLE);
   
   if ( equal_id(keepv,throwv) )
-  { kb_error(3882,"merge_vertex called with identical vertices.\n",WARNING);
+  { kb_error(3882,"vertex_merge called with identical vertices.\n",WARNING);
     return;
   }
  
   if ( web.symmetry_flag && !web.torus_flag )
-  { kb_error(3883,"merge_vertex can't handle wraps in non-torus symmetries.\n",
+  { kb_error(3883,"vertex_merge can't handle wraps in non-torus symmetries.\n",
       RECOVERABLE);
   }
    
@@ -2220,7 +2227,7 @@ edge_id e_id1,e_id2;
   }
  
   if ( web.symmetry_flag && !web.torus_flag )
-  { kb_error(3888,"merge_vertex can't handle wraps in non-torus symmetries.\n",
+  { kb_error(3888,"vertex_merge can't handle wraps in non-torus symmetries.\n",
       RECOVERABLE);
   }
 
@@ -2305,7 +2312,7 @@ edge_id f_id1,f_id2;
   }
  
   if ( web.symmetry_flag && !web.torus_flag )
-  { kb_error(3915,"merge_vertex can't handle wraps in non-torus symmetries.\n",
+  { kb_error(3915,"vertex_merge can't handle wraps in non-torus symmetries.\n",
       RECOVERABLE);
   }
 
@@ -2377,12 +2384,12 @@ edge_id f_id1,f_id2;
       REAL u[MAXCOORD];
   
       for ( i = 0 ; i < SDIM ; i++ )
-        z[i] = (REAL)(f2x[i] - f1x[i]);
+        z[i] = f2x[0][i] - f1x[0][i];
       matvec_mul(web.inverse_periods,z,u,SDIM,SDIM);
       for ( i = SDIM-1, wrap = 0 ; i >= 0 ; i-- )
       { wrap |= (int)(floor(u[i]+0.5)) & WRAPMASK; 
         wrap <<= TWRAPBITS;
-      }
+      } 
       if ( wrap )
       { for ( i = 0 ; i < FACET_VERTS ; i++ )
           torus_wrap(f1x[i],tempx[i],wrap); 
@@ -2469,4 +2476,137 @@ edge_id f_id1,f_id2;
 }
 
 
+/****************************************************************************
+*
+* function: reverse_orientation_edge()
+*
+* purpose: reverse the orientation of an edge
+*/
+
+void reverse_orientation_edge(e_id)
+edge_id e_id;
+{ vertex_id v_id[2],*v;
+  facetedge_id start_fe;
+  int i;
+  struct edge *e_ptr;
+
+  v_id[0] = get_edge_tailv(e_id);
+  v_id[1] = get_edge_headv(e_id);
+  /* swap endpoints */
+  switch ( web.modeltype )
+  { case LINEAR:
+       remove_vertex_edge(v_id[0],e_id);
+       remove_vertex_edge(v_id[1],inverse_id(e_id));
+       set_edge_headv(e_id,v_id[0]);
+       set_edge_tailv(e_id,v_id[1]);
+       break;
+    case QUADRATIC:
+       remove_vertex_edge(v_id[0],e_id);
+       remove_vertex_edge(v_id[1],inverse_id(e_id));
+       set_edge_headv(e_id,v_id[0]);
+       set_edge_tailv(e_id,v_id[1]);
+       break;
+    case LAGRANGE:
+       remove_vertex_edge(v_id[0],e_id);
+       remove_vertex_edge(v_id[1],inverse_id(e_id));
+       set_edge_headv(e_id,v_id[0]);
+       set_edge_tailv(e_id,v_id[1]);
+       v = get_edge_vertices(e_id);
+       for ( i = 1 ; i <= web.lagrange_order/2 ; i++ )
+       { vertex_id vv = v[i];
+         v[i] = v[web.lagrange_order - i];
+         v[web.lagrange_order - i] = vv;
+       }
+       break;
+  }
+  set_vertex_edge(v_id[1],e_id);
+  set_vertex_edge(v_id[0],inverse_id(e_id));
+
+  /* reverse id in facet-edges */
+  start_fe = get_edge_fe(e_id);
+  if ( valid_id(start_fe) )
+  { facetedge_id fe = start_fe;
+    do 
+    { facet_id feprev,fenext,fe_fore,fe_aft;
+      facet_id f_id = get_fe_facet(fe);
+      set_fe_facet(fe,inverse_id(f_id));
+      set_facet_fe(f_id,inverse_id(fe));
+      fenext = get_next_facet(fe);
+      feprev = get_prev_facet(fe);
+      set_next_facet(fe,feprev);
+      set_prev_facet(fe,fenext);
+      fe_fore = get_next_edge(fe);
+      fe_aft = get_prev_edge(fe);
+      set_next_edge(fe,inverse_id(fe_aft));
+      set_prev_edge(fe,inverse_id(fe_fore));
+      set_prev_edge(fe_fore,inverse_id(fe));
+      set_next_edge(fe_aft,inverse_id(fe));
+      set_body_fe(get_facet_body(f_id),inverse_id(fe));
+      set_body_fe(get_facet_body(inverse_id(f_id)),fe);
+      fe = fenext;
+    } while ( !equal_id(fe,start_fe) );
+  }
+
+  /* reverse symmetry wraps */
+  if ( web.symmetry_flag )
+    set_edge_wrap(e_id,(*sym_inverse)(get_edge_wrap(e_id)));
+    
+   
+  /* reverse "orientation" for methods and constraint integrals */
+  e_ptr = eptr(e_id);
+  e_ptr->attr ^= NEGBOUNDARY;
+  
+} /* end edge_reverse_orientation() */
+
+/***************************************************************************
+*
+* Function: reverse_orientation_facet()
+*
+* Purpose: reverse the orientation of one facet.
+*/
+
+void reverse_orientation_facet(f_id)
+facet_id f_id;
+{ facetedge_id start_fe,fe,last_fe;
+  body_id b_id,bb_id;
+  struct facet *e_ptr;
+  
+   /* reverse body attachments (before reversing edges!!) */
+  b_id = get_facet_body(f_id);
+  bb_id = get_facet_body(inverse_id(f_id));
+  set_facet_body(f_id,bb_id);
+  set_facet_body(inverse_id(f_id),b_id);
+
+  /* reverse fe */
+  fe = start_fe = get_facet_fe(f_id);
+  do
+  { set_fe_facet(fe,inverse_id(f_id));
+    last_fe = fe;
+    fe = get_next_edge(fe);
+  } while (valid_id(fe) && !equal_id(fe,start_fe));
+  if (!valid_id(fe)) /* in case of open string facet */
+    set_facet_fe(f_id,inverse_id(last_fe));
+  else
+    set_facet_fe(f_id,inverse_id(start_fe));
+    
+  /* invert Lagrange vertices */
+  if ( (web.representation == SOAPFILM) && (web.modeltype == LAGRANGE) )
+  { vertex_id *v = get_facet_vertices(f_id);
+    int row,spot = web.lagrange_order+1;
+    for ( row = 1 ; row < web.lagrange_order ; row++ )
+    { int rowsize = web.lagrange_order+1-row;
+      int i,mid = rowsize/2;
+      for ( i = 1 ; i < mid ; i++ )
+      { vertex_id vv = v[spot+i];
+        v[spot+i] = v[spot+rowsize-i-1];
+        v[spot+rowsize-i] = vv;
+      }
+      spot += rowsize;
+    }
+  }
+  
+  /* reverse method orientations */
+  e_ptr = fptr(f_id);
+  e_ptr->attr ^= NEGBOUNDARY;
+}
 
