@@ -53,9 +53,8 @@ void pix_start()
          { case 'Y': normflag = 1; break;
            case 'N': normflag = 0; break;
            case '0': 
-              outstring("Pixar file aborted.\n");
-              longjmp(jumpbuf,1);
-            default: outstring("Unintelligible response. Try again.\n");
+              kb_error(3891,"Pixar file aborted.\n",RECOVERABLE);
+           default: outstring("Unintelligible response. Try again.\n");
          }
     }
 
