@@ -13,6 +13,12 @@
 *        of that.
 */
 
+#ifdef  __cplusplus
+
+void loadstub(void);
+
+extern "C" {
+#endif
 
 #ifndef drand48
  extern double drand48 ARGS((void)); /* may not be in header file */
@@ -68,6 +74,17 @@ void mpi_synch_corona ARGS((void));
 void mpi_dissolve_corona ARGS((void));
 #endif
 
+void mark_element_loops ARGS((struct treenode *, int));
+void mem_list_dump ARGS((void));
+struct element * elhash_lookup ARGS((element_id));
+void elhash_delete ARGS((element_id));
+void elhash_bigger ARGS((void));
+void elhash_extend ARGS((int,int));
+struct element *elhash_new_element ARGS((int,element_id));
+void elhash_insert ARGS((element_id,struct element *));
+void elhash_replace ARGS((element_id,struct element *));
+void matrix_multiply_command ARGS((int,int,int));
+void matrix_inverse_command ARGS((int,int));
 void tree_analyze ARGS((struct linsys *));
 void fixup_edge_content_meths ARGS((edge_id));
 void fixup_vertex_content_meths ARGS((vertex_id));
@@ -131,6 +148,7 @@ void partner_hit_volgrad_fix ARGS((void));
 void detect_bdry_hits ARGS((void));
 void thread_move_vertices ARGS((void));
 void torus_period_init ARGS((void));
+void set_eigenvalue_list_global ARGS((REAL*,int));
 void set_view_transforms_global ARGS((void));
 void set_torus_periods_global ARGS((void));
 void set_inverse_periods_global ARGS((void));
@@ -151,6 +169,7 @@ void init_local_scope ARGS((ident_t));
 void exit_local_scope ARGS((void));
 void begin_local_scope ARGS((void));
 void end_local_scope ARGS((void));
+void locals_copy ARGS((struct locallist_t **,struct locallist_t *));
 ident_t lookup_local_var ARGS((char*));
 ident_t add_local_var ARGS((char*));
 void read_attribute_value ARGS((struct extra *, void*));
@@ -238,6 +257,7 @@ void break_check ARGS((void));
 int lagrange_facet_normal  ARGS((vertex_id,facet_id,REAL**));
 void test_axial_points ARGS(( facet_id ));
 int compare_edge_attr ARGS((edge_id, edge_id));
+int compare_edge_facet_attr ARGS((edge_id,facet_id));
 int compare_vertex_edge_attr ARGS((vertex_id, edge_id));
 int compare_vertex_attr ARGS((vertex_id, vertex_id));
 void set_body_volume ARGS((body_id,REAL,int));
@@ -449,7 +469,7 @@ void remove_vertex_edge ARGS((vertex_id,edge_id));
 void insert_vertex_facet ARGS((vertex_id,facet_id));
 void remove_vertex_facet ARGS((vertex_id,facet_id));
 void catfulltext ARGS((char*));
-int identcase ARGS((void));
+int identcase ARGS((char *));
 int eliminate_facet ARGS((facet_id));
 int string_eliminate_facet ARGS((facet_id));
 REAL quantity_attribute ARGS(( element_id, int ));
@@ -488,6 +508,8 @@ void reduce_string ARGS((char *));
 REAL vertex_angle ARGS((vertex_id));
 void fix_ctm ARGS((REAL**,REAL,REAL));
 void my_exit ARGS((int));
+void reverse_orientation_edge ARGS((edge_id));
+void reverse_orientation_facet ARGS((facet_id));
 int dissolve_vertex ARGS((vertex_id));
 int dissolve_edge ARGS((edge_id));
 int dissolve_facet ARGS((facet_id));
@@ -608,6 +630,8 @@ int set_parameters ARGS((void));
 int minus_type ARGS((int));
 void string_fixup ARGS((void));
 void runge_kutta ARGS((void));
+void autopop_init ARGS((void));
+void autopop_cleanup ARGS((void));
 void autopop_pop ARGS((void));
 void autochop_chop ARGS((void));
 void autopop_detect ARGS((REAL));
@@ -936,14 +960,21 @@ void element_id_jiggle ARGS((void ));
 REAL gaussian ARGS((void ));
 void nrerror ARGS((char *));
 void matcopy ARGS((REAL * *,REAL * *,int ,int ));
+REAL **perm_matrix2 ARGS((int,int));
 #ifdef MEMSTRINGS
 REAL * *kb_dmatrix ARGS((int ,int ,int ,int,char*,int ));
 REAL * * * kb_dmatrix3 ARGS((int ,int ,int,char*,int ));
 REAL * * * * kb_dmatrix4 ARGS((int ,int ,int, int,char*,int ));
+REAL * *kb_temp_dmatrix ARGS((int ,int ,int ,int,char*,int ));
+REAL * * * kb_temp_dmatrix3 ARGS((int ,int ,int,char*,int ));
+REAL * * * * kb_temp_dmatrix4 ARGS((int ,int ,int, int,char*,int ));
 #else
 REAL * *kb_dmatrix ARGS((int ,int ,int ,int ));
 REAL * * * kb_dmatrix3 ARGS((int ,int ,int ));
 REAL * * * * kb_dmatrix4 ARGS((int ,int ,int, int ));
+REAL * *kb_temp_dmatrix ARGS((int ,int ,int ,int ));
+REAL * * * kb_temp_dmatrix3 ARGS((int ,int ,int ));
+REAL * * * * kb_temp_dmatrix4 ARGS((int ,int ,int, int ));
 #endif
 REAL *** matrix3_reorder ARGS((REAL***,int,int,int));
 int *ivector ARGS((int ,int ));
@@ -953,6 +984,9 @@ void free_vector ARGS((REAL *,int ,int ));
 void free_matrix ARGS((REAL * *));
 void free_matrix3 ARGS((REAL * * *));
 void free_matrix4 ARGS((REAL * * * *));
+void free_temp_matrix ARGS((REAL * *));
+void free_temp_matrix3 ARGS((REAL * * *));
+void free_temp_matrix4 ARGS((REAL * * * *));
 void vector_add ARGS((REAL*,REAL*,int));
 void vector_add_smul ARGS((REAL*,REAL*,REAL,int));
 void vector_sub ARGS((REAL*,REAL*,int));
@@ -967,6 +1001,7 @@ void vec_mat_mul ARGS((REAL *,REAL ** ,REAL *,int ,int ));
 void mat_mult ARGS((REAL * *,REAL * *,REAL * *,int ,int ,int ));
 REAL quadratic_form ARGS((REAL *,REAL **,REAL *,int));
 int  mat_inv ARGS((REAL * *,int ));
+int  mat_inv_sym ARGS((REAL * *,int ));
 int  mat_inv_sparse ARGS((REAL * *,int ));
 REAL  determinant ARGS((REAL * *,int ));
 REAL  det_adjoint ARGS((REAL * *,int ));
@@ -1015,7 +1050,7 @@ struct volgrad *get_next_vgrad ARGS((struct volgrad *));
 struct volgrad *new_vgrad ARGS((void ));
 struct volgrad *get_bv_vgrad ARGS((int ,element_id ));
 struct volgrad *get_bv_new_vgrad ARGS((int ,element_id ));
-element_id new_element ARGS((int,element_id ));
+element_id new_element ARGS((int,element_id,element_id));
 void free_element ARGS((element_id ));
 void edge_force_l ARGS((element_id ));
 void edge_energy_l ARGS((element_id ));
@@ -1056,7 +1091,7 @@ int articulate ARGS((REAL ));
 int edge_refine ARGS((element_id ));
 int equiangulate ARGS((void ));
 int ridge_notcher ARGS((REAL ));
-void outstring ARGS((char *));
+void outstring ARGS(( CONST char *));
 void erroutstring ARGS((char *));
 void getstring ARGS((char *,int));
 void kb_error ARGS((int, char *,int ));
@@ -1143,7 +1178,6 @@ facetedge_id get_vertex_fe ARGS((vertex_id));
   
 #endif
 
-#ifdef MAC_OS_X
-FILE *popen();
-FILE *fdopen();
+#ifdef  __cplusplus
+}
 #endif
