@@ -175,6 +175,7 @@ void ps_init()
     fputs("        /y2 exch def /x2 exch def /y1 exch def /x1 exch def\n",fd);
     fputs("        /dy y2 y1 sub def  /dx x2 x1 sub def\n",fd);
     fputs("        /mag dx dx mul dy dy mul add sqrt def\n",fd);
+    fputs("        mag 0 ne {                           \n",fd);
     fputs("        /xu dx mag div def /yu dy mag div def\n",fd);
     fputs("        /angle xu yu neg atan def\n",fd);
     fputs("        newpath x2 xu vorad mul sub  y2 yu vorad mul sub moveto\n",fd);
@@ -187,6 +188,7 @@ void ps_init()
     fputs("                y2 yu vorad mul sub  arrowrad xu mul add\n",fd);
     fputs("                arrowrad angle 180 sub angle 225 sub arcn\n",fd);
     fputs("               stroke\n",fd);
+    fputs("        } if \n",fd);
     fputs(" } def\n",fd);
 
     fputs("/Helvetica findfont .03 scalefont setfont\n",fd);
@@ -306,16 +308,17 @@ struct tsort *t;
   if ( ps_colorflag > 0 )
      for ( i = 0 ; i < 3 ; i++ )
         fprintf(fd," %5.3f",(DOUBLE)rgb_colors[t->color][i]);
-  /* width of edge, in descending order of thickness */
+  /* width of edge */
   if ( t->flag & EDGE_ARC )
   { if ( ps_widthattr >= 0 ) 
       fprintf(fd," %5.3f arcedge ",*EREAL(t->f_id,ps_widthattr));
-    else fprintf(fd," %5.3f arcedge\n",ps_gridedgewidth);
+    else fprintf(fd," %5.3f arcedge\n",ps_stringwidth);
   } 
   else
   {
   if ( ps_widthattr >= 0 ) 
     fprintf(fd," %5.3f edge ",*EREAL(t->f_id,ps_widthattr));
+  else if ( web.representation == STRING ) fprintf(fd," ew\n");
   else if ( t->etype[0] & BARE_EDGE ) fprintf(fd," bw\n");
   else if ( t->etype[0] & FIXED_EDGE ) fprintf(fd," fw\n");
   else if ( t->etype[0] & CONSTRAINT_EDGE ) fprintf(fd," ww\n");
@@ -642,15 +645,15 @@ if ( edgeredrawflag )
         if ( strlen(hv) > 5 ) hv[5] = 0;
         if ( strlen(en) > 5 ) en[5] = 0; /* max 5 chars in labels */
 
-        if ( (tv > 0) && (strlen(tv) > 0) )
+        if ( tv && (strlen(tv) > 0) )
         fprintf(fd,"(%s) %f %f vertex%1d\n",tv,(DOUBLE)t->x[ii][0],
                 (DOUBLE)t->x[ii][1],strlen(tv));
-        if ( (hv > 0) && (strlen(hv) > 0) )
+        if ( hv && (strlen(hv) > 0) )
         fprintf(fd,"(%s) %f %f vertex%1d\n",hv,(DOUBLE)t->x[jj][0],
                 (DOUBLE)t->x[jj][1],strlen(hv));
 
         a = .35;
-        if ( (en > 0) && (strlen(en) > 0) )
+        if ( en && (strlen(en) > 0) )
         { 
           if ( inverted(e_id) )
            fprintf(fd," (%s) %f %f edgenum%1d\n",en,
