@@ -33,8 +33,8 @@ int commandverb;
 * purpose:  Appends command to history list, queue fashion.
 */
 
-void new_history(text)
-char *text;
+void new_history ARGS1((text),
+char *text)
 { size_t length;
   int k;
 
@@ -69,8 +69,8 @@ char *text;
 * return:   retval from command()
 */
 
-int old_history(text)
-char *text;
+int old_history ARGS1((text),
+char *text)
 { char *h;
   int k;
   int hnum;
@@ -120,8 +120,8 @@ do_old:
 * purpose:  catenates line to full text of command
 */
 
-void catfulltext(stuff)
-char *stuff;
+void catfulltext ARGS1((stuff),
+char *stuff)
 {
 
   if ( fulltextsize > MAXCMDSIZE-3 ) return; /* too long */
@@ -146,9 +146,9 @@ char *stuff;
 *          0 else
 */
 
-int command(text,mode)
-char *text;
-int mode; /* NO_HISTORY or ADD_TO_HISTORY */
+int command ARGS2((text,mode),
+char *text,
+int mode) /* NO_HISTORY or ADD_TO_HISTORY */
 {
   struct expnode qnode;  /* for query expression */
   int retval = 0;
@@ -156,14 +156,8 @@ int mode; /* NO_HISTORY or ADD_TO_HISTORY */
   struct treenode *old_list;
   int old_listtop, old_listmax;
   int old_brace_depth = brace_depth;
-  struct locallist_t command_locals,*old_query_locals;
 
-  memset(&command_locals,0,sizeof(command_locals));
-
-  qnode.root = NULL;
-  qnode.start = NULL;
-  old_query_locals = query_locals;
-  qnode.locals = query_locals = &command_locals;
+  memset(&qnode,0,sizeof(struct expnode));
   sprintf(qnode.name,"command");
  
   old_listmax = listmax;
@@ -177,6 +171,11 @@ int mode; /* NO_HISTORY or ADD_TO_HISTORY */
   cmdptr = text;
   old_scope = begin_scope();
 
+
+#ifdef __cplusplus
+  try
+  {
+#else
   if ( setjmp(cmdbuf) )
   { set_scope(old_scope);  
     cmdptr = NULL;  
@@ -186,6 +185,8 @@ int mode; /* NO_HISTORY or ADD_TO_HISTORY */
 	local_nest_depth = 0;
     return 0; 
   }
+#endif
+
   verb_flag = 1; /* tell lex we need a verb first */
   old_global_count = web.global_count; /* for error recovery */
   old_perm_global_count = web.perm_global_count; /* for error recovery */
@@ -195,13 +196,17 @@ int mode; /* NO_HISTORY or ADD_TO_HISTORY */
   catfulltext(text);
 
   retval = yybegin();  /* 0 for accept, 1 for error */
+  qnode.locals = localbase;
+  if ( localbase )
+    qnode.locals->flags |= LL_IN_USE;
+  exit_local_scope();
+
   cmdptr = NULL;
 
   if ( (retval == 1) || (parse_error_flag) )
   { myfree((char *)list); 
     if ( list == permlist ) permlist = NULL;
     list = NULL; set_scope(old_scope); 
-    exit_local_scope();
     return 0;  
   }
 
@@ -213,7 +218,7 @@ int mode; /* NO_HISTORY or ADD_TO_HISTORY */
   }
   if ( qnode.root->type != CMDLIST_ )  /* expression or something */
      kb_error(1589,"Illegal command.\n",COMMAND_ERROR);
-
+  
 
   list[0] = list[listtop-1];  /* root also in first spot */
   if ( list[0].left ) list[0].left += listtop - 1;
@@ -225,6 +230,10 @@ int mode; /* NO_HISTORY or ADD_TO_HISTORY */
   /* figure stack usage */
   stack_usage(&qnode);
 
+  /* set element loop flag bit in nodes */
+  if ( !function_kludge_flag )
+    mark_element_loops(&list[0],0);
+
   if ( logfd )
   fprintf(logfd,"%s\n",text);
 
@@ -247,14 +256,20 @@ command_exit:
   list = old_list; listtop = old_listtop; /* for nested parsing */
   listmax = old_listmax;
   brace_depth = old_brace_depth;
-  exit_local_scope();
-  query_locals = old_query_locals;
-  if ( command_locals.list ) 
-  { if ( command_locals.flags & LL_PERMANENT )
-      free((char*)command_locals.list);
-    else myfree((char*)command_locals.list);
-  }
   return retval;
+
+#ifdef __cplusplus
+  }
+  catch(cmd_excep c)
+  { set_scope(old_scope);  
+    cmdptr = NULL;  
+    myfree((char *)list); 
+    list = NULL;
+    memset(local_scope_bases,0,sizeof(local_scope_bases));
+	local_nest_depth = 0;
+    return 0; 
+  }
+#endif
 }
 
 /*************************************************************************
@@ -265,8 +280,8 @@ command_exit:
 *
 */
 
-void stack_usage (ex )
-struct expnode *ex;
+void stack_usage ARGS1((ex ),
+struct expnode *ex)
 { int usage = 0;
   struct treenode *node ;
 #ifdef _DEBUG
@@ -290,3 +305,25 @@ struct expnode *ex;
 #endif
   }
 }
+
+/***********************************************************************
+*
+* Function: mark_element_loops()
+*
+* Purpose: Mark flags of expression nodes that are within 
+*          element loops, so can know when freeing discards is
+*          ill-advised.   Recursively transverses tree pre-order.
+*/
+
+void mark_element_loops(root,mark)
+struct treenode *root;
+int mark;  /* whether root in loop */
+{ if ( mark ) 
+    root->flags |= IN_ELEMENT_LOOP;
+  else mark = root->flags & IN_ELEMENT_LOOP;
+
+  if ( root->left )
+    mark_element_loops(root+root->left,mark);
+  if ( root->right )
+    mark_element_loops(root+root->right,mark);
+}
