@@ -642,3 +642,171 @@ static int twist_var = -1;
  }
 
 
+/**************************************************************************
+    Symmetry group: quarter_turn
+    3D torus with quarter turn in identification of top and bottom.
+    x and y periods taken to be 1.
+    z period is the user-defined variable quarter_turn_period.
+    Generators x,y,z. x and y as in regular torus mode. z is 
+    vertical translation with quarter turn: (x,y,z)->(-y,x,z).
+    Relations: x z = z y^-1,   y z = z x
+    Numerical representation: as in torus, for powers of x,y,z
+      with generators applied in that order.
+***************************************************************************/
+
+#define QUARTERTURNNAME "quarter_turn_period"
+int quarter_turn_var;
+
+void quarter_turn_wrap(x,y,wrap)
+ REAL *x;   /* original coordinates */
+ REAL *y;   /* wrapped coordinates  */
+ WRAPTYPE wrap;  /* encoded symmetry group element */
+ { REAL zper;
+   int xwrap,ywrap,zwrap;
+   if ( quarter_turn_var < 0 )
+   { quarter_turn_var = lookup_global(QUARTERTURNNAME);
+     if ( quarter_turn_var < 0)
+       kb_error(4008,
+        "quarter_turn_period variable not defined for quarter_turn symmetry\n",
+             RECOVERABLE);
+   }
+   xwrap = wrap & WRAPMASK;
+   if ( xwrap & 0x10 ) xwrap -= 0x20;
+   ywrap = (wrap >> TWRAPBITS) & WRAPMASK;
+   if ( ywrap & 0x10 ) ywrap -= 0x20;
+   zwrap = (wrap >> 2*TWRAPBITS) & WRAPMASK;
+   if ( zwrap & 0x10 ) zwrap -= 0x20;
+
+   y[0] = x[0] + xwrap;
+   y[1] = x[1] + ywrap;
+   if ( zwrap )
+   { zper = globals(quarter_turn_var)->value.real;
+     y[2] = x[2] + zwrap * zper;
+     zwrap = (zwrap + 20) % 4;
+     if ( zwrap == 1 )
+     { y[0] = -(x[1] + ywrap);
+       y[1] = x[0] + xwrap;
+     }
+     else if ( zwrap == 2 )
+     { y[0] = -(x[0] + xwrap);
+       y[1] = -(x[1] + ywrap);
+     }
+     else if ( zwrap == 3 )
+     { y[0] = x[1] + ywrap;
+       y[1] = -(x[0] + xwrap);
+     }
+   }
+   else y[2] = x[2];
+}
+
+WRAPTYPE quarter_turn_compose(wrap1,wrap2)
+WRAPTYPE wrap1,wrap2;
+{ 
+  int xwrap1,ywrap1,zwrap1;
+  int xwrap2,ywrap2,zwrap2;
+   if ( quarter_turn_var < 0 )
+   { quarter_turn_var = lookup_global(QUARTERTURNNAME);
+     if ( quarter_turn_var < 0)
+       kb_error(3877,
+        "quarter_turn_period variable not defined for quarter_turn symmetry\n",
+             RECOVERABLE);
+   }
+   xwrap1 = wrap1 & WRAPMASK;
+   if ( xwrap1 & 0x10 ) xwrap1 -= 0x20;
+   ywrap1 = (wrap1 >> TWRAPBITS) & WRAPMASK;
+   if ( ywrap1 & 0x10 ) ywrap1 -= 0x20;
+   zwrap1 = (wrap1 >> 2*TWRAPBITS) & WRAPMASK;
+   if ( zwrap1 & 0x10 ) zwrap1 -= 0x20;
+
+   xwrap2 = wrap2 & WRAPMASK;
+   if ( xwrap2 & 0x10 ) xwrap2 -= 0x20;
+   ywrap2 = (wrap2 >> TWRAPBITS) & WRAPMASK;
+   if ( ywrap2 & 0x10 ) ywrap2 -= 0x20;
+   zwrap2 = (wrap2 >> 2*TWRAPBITS) & WRAPMASK;
+   if ( zwrap2 & 0x10 ) zwrap2 -= 0x20;
+
+   zwrap2 = (zwrap2 + 20) % 4;
+   if ( zwrap2 == 1 )
+   { int temp = xwrap1;
+     xwrap1 = ywrap1;
+     ywrap1 = -temp;
+   } 
+   else if ( zwrap2 == 2 )
+   { xwrap1 = -xwrap1;
+     ywrap1 = -ywrap1;
+   }
+   else if ( zwrap2 == 3 )
+   { int temp = xwrap1;
+     xwrap1 = -ywrap1;
+     ywrap1 = temp;
+   } 
+
+   return (((zwrap1 + zwrap2) & WRAPMASK) << (2*TWRAPBITS))
+        + (((ywrap1 + ywrap2) & WRAPMASK) << TWRAPBITS)
+        + ((xwrap1 + xwrap2) & WRAPMASK); 
+
+ }
+
+WRAPTYPE quarter_turn_inverse(wrap)
+WRAPTYPE wrap;
+{ int temp;
+  int xwrap,ywrap,zwrap;
+  xwrap = wrap & WRAPMASK;
+  if ( xwrap & 0x10 ) xwrap -= 0x20;
+  ywrap = (wrap >> TWRAPBITS) & WRAPMASK;
+  if ( ywrap & 0x10 ) ywrap -= 0x20;
+  zwrap = (wrap >> 2*TWRAPBITS) & WRAPMASK;
+  if ( zwrap & 0x10 ) zwrap -= 0x20;
+  xwrap = -xwrap;
+  ywrap = -ywrap;
+  zwrap = -zwrap;
+  switch ( (zwrap + 20) % 4 )
+  { case 1: 
+     temp = xwrap;
+     xwrap = ywrap;
+     ywrap = -temp;
+     break;
+    case 2:
+     xwrap = -xwrap;
+     ywrap = -ywrap;
+     break;
+    case 3:
+     temp = xwrap;
+     xwrap = -ywrap;
+     ywrap = temp;
+     break;
+  }
+  
+  return ((zwrap  & WRAPMASK) << (2*TWRAPBITS))
+        + ((ywrap & WRAPMASK) << TWRAPBITS)
+        + (xwrap  & WRAPMASK); 
+
+
+ }
+
+ void quarter_turn_form_pullback(x,xform,yform,wrap)
+ REAL *x; REAL *xform; REAL *yform; WRAPTYPE wrap;
+ { int zwrap;
+   zwrap = (wrap >> 2*TWRAPBITS) & WRAPMASK;
+   yform[2] = xform[2];
+   switch ( (zwrap + 20) % 4 )
+   { case 0:
+        yform[0] = xform[0];
+        yform[1] = xform[1];
+        break;
+     case 1:
+        yform[0] = xform[1];
+        yform[1] = -xform[0];
+        break;
+     case 2:
+        yform[0] = -xform[0];
+        yform[1] = -xform[1];
+        break;
+     case 3:
+        yform[0] = -xform[1];
+        yform[1] = xform[0];
+        break;
+        break;
+   }
+ }
+
