@@ -54,7 +54,7 @@ facet_id f_id;
   set_facet_area(f_id,energy);
   if ( get_fattr(f_id) & DENSITY )
          energy *= get_facet_density(f_id);
-  web.total_energy += energy;
+  binary_tree_add(web.total_energy_addends,energy);
       
 }
 
@@ -374,7 +374,7 @@ int sdim;
         }
 
   scount = (1 << sdim);
-  sslist = (struct simplex *)kb_calloc(scount,sizeof(struct simplex));
+  sslist = (struct simplex *)mycalloc(scount,sizeof(struct simplex));
 
   /* do simplices on corners */
   for ( i = 0 ; i <= sdim ; i++ )
@@ -462,8 +462,8 @@ void rehash()
   for ( i = 0, h = oldhashtable ; i < tablesize ; i++,h++ )
      {
         if ( !h->divpt ) continue;
-        hash1 = hash_1(h->endpt[0]) + hash_1(h->endpt[1]);
-        hash2 = hash_2(h->endpt[0]) + hash_2(h->endpt[1]);
+        hash1 = (unsigned int)(hash_1(h->endpt[0]) + hash_1(h->endpt[1]));
+        hash2 = (unsigned int)(hash_2(h->endpt[0]) + hash_2(h->endpt[1]));
         hash = hash1 % hashmask;
         for ( j = 0 ; j < hashentries ; j++ )
           {
@@ -506,8 +506,8 @@ vertex_id v1,v2;
   if ( v1 > v2 ) { vertex_id temp = v1; v1 = v2; v2 = temp;}
 
   /* look up in hash table */
-  hash1 = hash_1(v1) + hash_1(v2);
-  hash2 = hash_2(v1) + hash_2(v2);
+  hash1 = (unsigned int)(hash_1(v1) + hash_1(v2));
+  hash2 = (unsigned int)(hash_2(v1) + hash_2(v2));
   hash = hash1 % hashmask;
   for ( i = 0 ; i <= hashentries ; i++ )
      { 
@@ -585,7 +585,7 @@ void refine_all_simplices()
         for ( svcount = 0,flip=0 ; svcount <= web.dimension ; svcount++ )
         { /* insertion sort */
           REAL sortv = SDIM_dot(sort_vector,get_coord(v[svcount]))
-             + 10*machine_eps * ordinal(v[svcount]); 
+             + 10*machine_eps * loc_ordinal(v[svcount]); 
           for ( i = svcount ; i > 0 ; i-- )
              if ( sortval[i-1] > sortv ) 
                 {vlist[i] = vlist[i-1]; sortval[i] = sortval[i-1]; flip++;}
@@ -764,7 +764,8 @@ edge_id e_id;
 
     }
 
-  web.total_energy += energy/web.simplex_factorial*web.dimension;
+  binary_tree_add(web.total_energy_addends,
+            energy/web.simplex_factorial*web.dimension);
 
 }
 
@@ -1074,7 +1075,7 @@ REAL max_len;
            v1 = we_ptr->v[1] = se_ptr->v[1];
          }
       we_ptr->f[we_ptr->fcount++] = se_ptr->f;
-      fl=fedge+edges_per*ordinal(se_ptr->f) ; while ( *fl ) fl++;
+      fl=fedge+edges_per*loc_ordinal(se_ptr->f) ; while ( *fl ) fl++;
       *fl = we_ptr; /* inverse list */
     }
   temp_free((char*)se);  /* don't need anymore */
@@ -1122,20 +1123,18 @@ REAL max_len;
              v = get_facet_vertices(newf);
              oldv = get_facet_vertices(f_id);
              for ( j = 0 ; j <= web.dimension ; j++ )
-                { if ( equal_id(v0,oldv[j]) ) v[j] = newv;
-                     else v[j] = oldv[j];
-                  if ( equal_id(v1,oldv[j]) ) oldv[j] = newv;
-                }
+             { if ( equal_id(v0,oldv[j]) ) v[j] = newv;
+               else v[j] = oldv[j];
+               if ( equal_id(v1,oldv[j]) ) oldv[j] = newv;
+             }
              /* update working list */
              if ( web.skel[FACET].max_ord >= fedge_max )
-                { int old = fedge_max;
-                  fedge_max = web.skel[FACET].max_ord+100;
-                  fedge = (struct w_edge **)temp_realloc((char*)fedge,
-                                 fedge_max*edges_per*sizeof(struct w_edge *),
-                                 old*edges_per*sizeof(struct w_edge *));
-                }
-             fl = fedge + ordinal(f_id)*edges_per;
-             newfl = fedge + ordinal(newf)*edges_per;
+             { fedge_max = web.skel[FACET].max_ord+100;
+               fedge = (struct w_edge **)temp_realloc((char*)fedge,
+                               fedge_max*edges_per*sizeof(struct w_edge *));
+             }
+             fl = fedge + loc_ordinal(f_id)*edges_per;
+             newfl = fedge + loc_ordinal(newf)*edges_per;
              for ( m = 0 ; m<edges_per ; fl++,m++ )
                 { if ( !*fl || (*fl)->v[0] == v0 ) continue; /* is ok */
                   if ( (*fl)->v[1] == v1 )
@@ -1350,7 +1349,7 @@ REAL *rhs;
         vertex_id *v_id = get_edge_vertices(e_id);
 
         for ( i = 0 ; i < web.dimension ; i++ )
-          v[i] = vhead + ordinal(v_id[i]);
+          v[i] = get_vertex_vhead(v_id[i]);
 
         memset((char*)first[0],0,sizeof(REAL)*web.dimension*SDIM);
         memset((char*)second[0][0][0],0,sizeof(REAL)*
@@ -1375,7 +1374,7 @@ REAL *rhs;
          for ( j = i ; j < web.dimension ; j++ )
           {
              if ( (v[i]->freedom==0) || (v[j]->freedom==0) ) continue;
-             if ( ordinal(v_id[i]) > ordinal(v_id[j]) )
+             if ( loc_ordinal(v_id[i]) > loc_ordinal(v_id[j]) )
                  { tail = i; head = j; }
              else { tail = j; head = i; }
 
@@ -1456,7 +1455,7 @@ REAL *rhs;
         coeff = get_facet_density(v_id[i])*sqrt(Q)/web.simplex_factorial;
 
         for ( i = 0 ; i <= web.dimension ; i++ )
-          v[i] = vhead + ordinal(v_id[i]);
+          v[i] = get_vertex_vhead(v_id[i]);
 
         /* first derivatives of area */
         for ( i = 0 ; i <= web.dimension ; i++ )
@@ -1479,7 +1478,7 @@ REAL *rhs;
          for ( j = i ; j <= web.dimension ; j++ )
           {
              if ( (v[i]->freedom==0) || (v[j]->freedom==0) ) continue;
-             if ( ordinal(v_id[i]) > ordinal(v_id[j]) )
+             if ( loc_ordinal(v_id[i]) > loc_ordinal(v_id[j]) )
                  { tail = i; head = j; }
              else { tail = j; head = i; }
 
@@ -1634,9 +1633,7 @@ int mode;
   struct gauss_lag *gl = (web.modeltype == LAGRANGE) ? 
        &gauss_lagrange[web.dimension][web.gauss2D_order] : NULL;
   int cpts = (web.modeltype == LAGRANGE) ? gl->lagpts : ctrl_num;
-  int gpts = (web.modeltype == LAGRANGE) ? gl->gnumpts : gauss2D_num;
   REAL **gp = (web.modeltype == LAGRANGE) ? gl->gpoly : gpoly;
-  REAL ***gpp = (web.modeltype == LAGRANGE) ? gl->gpolypart : gpolypartial;
 
   ada = dmatrix4(cpts,SDIM,SDIM,SDIM);
   tr = dmatrix(0,cpts,0,SDIM);
