@@ -425,7 +425,7 @@ element_id parent; /* for inherited stuff */
   vertex_id v_id;
   REAL *y;
 
-  v_id = new_element(VERTEX,parent);
+  v_id = new_element(VERTEX,parent,NULLID);
   if ( x )
   { y = get_coord(v_id);
     for ( i = 0 ; i < SDIM ; i++ ) y[i] = x[i];
@@ -447,7 +447,7 @@ vertex_id old_v;
   vertex_id v_id;
   struct vertex *v_id_p;
 
-  v_id = new_element(VERTEX,NULLID);
+  v_id = new_element(VERTEX,NULLID,NULLID);
   v_id_p = vptr(v_id);
   memcpy((char *)&(v_id_p->attr),(char *)&(elptr(old_v)->attr),
               web.sizes[VERTEX] - ((char*)&(v_id_p->attr)-(char*)v_id_p));
@@ -465,7 +465,7 @@ element_id parent; /* for inherited stuff */
   REAL *x,*h,*t;
   int i,k;
 
-  e_id = new_element(EDGE,parent);
+  e_id = new_element(EDGE,parent,NULLID);
   set_edge_fe(e_id,NULLFACETEDGE);
   set_edge_color(e_id,DEFAULT_EDGE_COLOR);
   if ( valid_id(tail_id) && valid_id(head_id) )
@@ -473,7 +473,7 @@ element_id parent; /* for inherited stuff */
      set_edge_headv(e_id,head_id);
      if ( (web.modeltype == QUADRATIC) && valid_id(head_id) )
      { /* quadratic version; linear interpolation of midpoint */
-        v_id = new_element(VERTEX,parent);
+        v_id = new_element(VERTEX,parent,NULLID);
         set_edge_midv(e_id,v_id);
         h = get_coord(head_id);
         t = get_coord(tail_id);
@@ -487,7 +487,7 @@ element_id parent; /* for inherited stuff */
         h = get_coord(head_id);
         t = get_coord(tail_id);
         for ( k = 1 ; k < web.lagrange_order ; k++ )
-        { v[k] = new_element(VERTEX,parent);
+        { v[k] = new_element(VERTEX,parent,NULLID);
           set_attr(v[k],Q_MIDEDGE);
           set_vertex_edge(v[k],e_id);
           x = get_coord(v[k]);
@@ -506,7 +506,7 @@ edge_id old_e;
   struct edge *e_id_p,*old_e_p;
   vertex_id newmid; 
 
-  e_id = new_element(EDGE,NULLID);
+  e_id = new_element(EDGE,NULLID,NULLID);
   e_id_p = eptr(e_id);
   old_e_p = eptr(old_e);
   memcpy((char *)&(e_id_p->attr),(char *)&(old_e_p->attr),
@@ -543,7 +543,7 @@ facet_id  new_facet ()
 { 
   facet_id f_id;
 
-  f_id = new_element(FACET,NULLID);
+  f_id = new_element(FACET,NULLID,NULLID);
   set_facet_color(f_id,DEFAULT_FACET_COLOR);
   set_facet_density(f_id,1.0);
   return f_id;
@@ -559,12 +559,12 @@ facet_id old_f;
 
   old_f = positive_id(old_f);  /* since new facet id will be positive */
   
-  f_id = new_element(FACET,NULLID);
+  f_id = new_element(FACET,NULLID,NULLID);
   f_id_p = fptr(f_id);
   memcpy((char *)&(f_id_p->attr),(char *)&(elptr(old_f)->attr),
               web.sizes[FACET] - ((char*)&(f_id_p->attr)-(char*)f_id_p));
-  set_attr(f_id,NEWELEMENT);
   f_id_p->self_id = f_id; /* restore new id */
+  set_attr(f_id,NEWELEMENT);
 
   /* update body facet lists */
   if ( web.skel[BODY].count )
@@ -590,7 +590,7 @@ body_id new_body()
 
   expand_attribute(FACET,F_BODY_LIST_ATTR,&two);
   expand_attribute(FACET,F_NEXT_BFACET_ATTR,&four);
-  b_id = new_element(BODY,NULLID);
+  b_id = new_element(BODY,NULLID,NULLID);
   set_body_fe(b_id,NULLFACETEDGE);
   if ( everything_quantities_flag ) 
       convert_new_body_to_quantity(b_id);
@@ -604,11 +604,14 @@ body_id old_b;
   body_id b_id;
   struct body *b_id_p;
 
-  b_id = new_element(BODY,NULLID);
+  b_id = new_element(BODY,NULLID,NULLID);
   b_id_p = bptr(b_id);
   memcpy((char *)&(b_id_p->attr),(char *)&(elptr(old_b)->attr),
               web.sizes[BODY] - ((char*)&(b_id_p->attr)-(char*)b_id_p));
   b_id_p->self_id = b_id; /* restore new id */
+  set_body_fe(b_id,NULLFACETEDGE);
+  if ( everything_quantities_flag ) 
+      convert_new_body_to_quantity(b_id);
   web.bodycount++;
   return b_id;
 }
@@ -620,7 +623,7 @@ edge_id e_id;
   facetedge_id fe_id;
   vertex_id headv,tailv;
 
-  fe_id = new_element(FACETEDGE,NULLID);
+  fe_id = new_element(FACETEDGE,NULLID,NULLID);
   set_fe_edge(fe_id,e_id);
   set_fe_facet(fe_id,f_id);
   set_prev_edge(fe_id,NULLFACETEDGE);
@@ -830,6 +833,48 @@ edge_id ea,eb;
   return INCOMPARABLE;
 }
 
+
+/********************************************************************
+*
+* Function: compare_edge_facet_attr(ea,fb)
+* 
+* Purpose: Compare constraints of edge and facet.
+*
+* Return value: 0 INCOMPARABLE  Incomparable
+*               1 A_SUB_B Edge proper subset of facet
+*               2 A_EQ_B Edge equal facet
+*               3 A_SUPER_B Edge superset of facet
+*/
+
+int compare_edge_facet_attr(ea,fb)
+edge_id ea;
+facet_id fb;
+{ int i,j;
+  conmap_t * con1,*con2;
+  int same = 0;
+
+  /* check constraints */
+  con1 = get_e_constraint_map(ea);
+  con2 = get_f_constraint_map(fb);
+
+  for ( i = 1 ; i <= (int)con2[0] ; i++ )
+  { for ( j = 1 ; j <= (int)con1[0] ; j++ )
+      if ( (con1[j]&CONMASK) == (con2[i]&CONMASK) )
+      { same++;
+        break;
+      }
+  }
+
+  if ( (same==(int)con1[0]) && (same==(int)con2[0]) )
+    return A_EQ_B;
+  if ( (same==(int)con1[0]) && (same < (int)con2[0]) )
+    return A_SUB_B;
+  if ( (same < (int)con1[0]) && (same==(int)con2[0]) )
+    return A_SUPER_B;
+  return INCOMPARABLE;
+
+}
+
 /**************************************************************
 *
 *  Function:  equal_constr()
@@ -1462,8 +1507,9 @@ int n;
   constr = get_constraint(n);
   if ( constr->attr & CON_ENERGY )
   { set_attr(e_id, BDRY_ENERGY);
-     if ( everything_quantities_flag )
-        apply_method_num(e_id,constr->energy_method);
+    if ( everything_quantities_flag )
+       apply_method_num(positive_id(e_id),constr->energy_method);
+/* positive_id() to agree with old way */
   }
   if ( constr->attr & CON_CONTENT )
   { set_attr(e_id, BDRY_CONTENT);  /* BIG PROBLEM HERE GETTING RIGHT BODY!!! */
