@@ -61,7 +61,7 @@ void softimage()
   fprintf(fd,"PMSH    \"%s\"\n  {\n  ",name);
 
   /* vertex list */
-  vnumber = (int*)mycalloc(web.skel[VERTEX].max_ord+1,sizeof(int*));
+  vnumber = (int*)temp_calloc(web.skel[VERTEX].max_ord+1,sizeof(int*));
 
   fprintf(fd,"    vertex %ld\n",web.skel[VERTEX].count);
   n = 1;
@@ -79,7 +79,7 @@ void softimage()
   FOR_ALL_FACETS(f_id)
   { facetedge_id fe = get_facet_fe(f_id);
     facetedge_id next_fe = get_next_edge(fe);
-    int oh = loc_ordinal(get_fe_headv(fe))+1;
+    int oh = vnumber[loc_ordinal(get_fe_headv(fe))];
     fprintf(fd,"     %10d %10d %10d %10d\n",oh,
             vnumber[loc_ordinal(get_fe_headv(next_fe))],
             vnumber[loc_ordinal(get_fe_tailv(fe))],
@@ -88,6 +88,8 @@ void softimage()
   fprintf(fd,"    }\n\n");
  
   fclose(fd);
+
+  temp_free((char*)vnumber);
 }
 
 
