@@ -1125,7 +1125,7 @@ struct qinfo *v_info;
   a2 = (s1 + s2)/2;
   if ( (a1 <= 0.0) || (a2 == 0.0) ) return 0.0;
 
-  hh = (2*a1+12*a1*a1)/a2/a2;
+  hh = (2*a1+12*a1*a1/36)/a2/a2;
   energy = a2*pow(hh,power/2);
 
   return energy;
@@ -1167,7 +1167,7 @@ struct qinfo *v_info;
   a2 = (s1 + s2)/2;
   if (a2 == 0.0) return 0.0;
 
-  hh = (a1+3*a1*a1)/a2/a2;
+  hh = (a1+3*a1*a1/36)/a2/a2;
   energy = a2*pow(hh,power/2);
 
   side[0] = side1; side[1] = side2; s[0] = s1; s[1] = s2;
@@ -1184,7 +1184,7 @@ struct qinfo *v_info;
      for ( i = 0 ; i < SDIM ; i++ )
      { REAL f;
         f = a2g[k][i]*energy/a2 + term*power/2
-              *(a1g[k][i]*(1+6*a1)/a2/a2 - (2*a1+6*a1*a1)/a2/a2/a2*a2g[k][i]);
+              *(a1g[k][i]*(1+6*a1/36)/a2/a2 - (2*a1+6*a1*a1/36)/a2/a2/a2*a2g[k][i]);
         v_info->grad[k+1][i] = f; 
         v_info->grad[0][i] += -f; 
      }
@@ -1230,7 +1230,7 @@ struct qinfo *v_info;
   a2 = (s1 + s2)/2;
   if ( a2 == 0.0 ) return 0.0;
 
-  energy = 2*(a1+6*a1*a1)/a2;
+  energy = 2*(a1+6*a1*a1/36)/a2;
 
   side[0] = side1; side[1] = side2; s[0] = s1; s[1] = s2;
   for ( k = 0 ; k < 2 ; k++ )
@@ -1241,10 +1241,10 @@ struct qinfo *v_info;
         }
   }
   for ( i = 0 ; i < SDIM ; i++ ) v_info->grad[0][i] = 0.0;
-  term = (a1+6*a1*a1)/a2/a2;
+  term = (a1+6*a1*a1/36)/a2/a2;
   for ( k = 0 ; k < 2 ; k++ )
      for ( i = 0 ; i < SDIM ; i++ )
-     { REAL f = 2*(a1g[k][i]*(1+12*a1)/a2 - term*a2g[k][i]);
+     { REAL f = 2*(a1g[k][i]*(1+12*a1/36)/a2 - term*a2g[k][i]);
         v_info->grad[k+1][i] = f; 
         v_info->grad[0][i] += -f; 
      }
@@ -1281,11 +1281,11 @@ struct qinfo *v_info;
     for ( kk = 0 ; kk < 2 ; kk++ )
      for ( i = 0 ; i < SDIM ; i++ )
       for ( ii = 0 ; ii < SDIM ; ii++ )
-      { REAL f = 2*((a1h[k][kk][i][ii]*(1+12*a1)+12*a1g[k][i]*a1g[kk][ii])/a2
-                         - a1g[k][i]*(1+12*a1)*a2g[kk][ii]/a2/a2
-                         - a1g[kk][ii]*(1+12*a1)*a2g[k][i]/a2/a2
-                         - (a1+6*a1*a1)*a2h[k][kk][i][ii]/a2/a2
-                         + 2*(a1+6*a1*a1)*a2g[k][i]*a2g[kk][ii]/a2/a2/a2);
+      { REAL f = 2*((a1h[k][kk][i][ii]*(1+12*a1/36)+12*a1g[k][i]*a1g[kk][ii]/36)/a2
+                         - a1g[k][i]*(1+12*a1/36)*a2g[kk][ii]/a2/a2
+                         - a1g[kk][ii]*(1+12*a1/36)*a2g[k][i]/a2/a2
+                         - (a1+6*a1*a1/36)*a2h[k][kk][i][ii]/a2/a2
+                         + 2*(a1+6*a1*a1/36)*a2g[k][i]*a2g[kk][ii]/a2/a2/a2);
          h[k+1][kk+1][i][ii] += f;
          h[k+1][0][i][ii] += -f;
          h[0][kk+1][i][ii] += -f;
@@ -1294,6 +1294,129 @@ struct qinfo *v_info;
   return energy;
 }
 
+/*************************************************************************
+    Square curvature for string with intrinsic curvature
+    Plane only; intrinsic curvature in vertex attribute 
+      "intrinsic_curvature".  Edges assumed to all be oriented
+    in the same direction.
+
+
+**************************************************************************/
+#define SQCURVE2_ATTR_NAME "intrinsic_curvature"
+static int sqcurve2_attr;  /* index number of attribute */
+/************************************************************************
+*
+*  function: sqcurve2_string_init()
+*
+*  purpose:  
+*/
+void sqcurve2_string_init(mode,mi)
+int mode;
+struct method_instance *mi;
+{ 
+  if ( web.modeltype != LINEAR )
+    kb_error(2864,"Method sqcurve_string only for LINEAR model.\n",
+       RECOVERABLE);
+
+  sqcurve2_attr = find_attribute(VERTEX,SQCURVE2_ATTR_NAME);
+  if ( sqcurve2_attr < 0 ) /* not found, so create */
+  { int one = 1;
+    sqcurve2_attr = add_attribute(VERTEX,SQCURVE2_ATTR_NAME,REAL_TYPE,0,&one,
+             DUMP_ATTR, NULL);
+  }
+}
+
+/************************************************************************
+*
+*  function: sqcurve2_string_value()
+*
+*  purpose:  Calculate square curvature energy for string model
+*                Works locally vertex by vertex. Assumes two edges per vertex.
+*
+*/
+
+REAL sqcurve2_string_value(v_info)
+struct qinfo *v_info;
+{
+  REAL s1,s2,s1xs2;  /* edge lengths */
+  REAL *side1 = v_info->sides[0][0],*side2 = v_info->sides[0][1];
+  REAL energy;
+  REAL h,h0,hh,a1,a2;
+
+  if ( v_info->vcount < 3 ) return 0.;
+
+  s1 = sqrt(SDIM_dot(side1,side1));
+  s2 = sqrt(SDIM_dot(side2,side2));
+  s1xs2 = side1[0]*side2[1] - side1[1]*side2[0];
+
+  a1 = s1xs2/s1/s2;  /* sin(theta) */
+  a2 = (s1 + s2)/2;
+  if ( a2 == 0.0 ) return 0.0;
+
+  h = a1/a2;  /* curvature */
+  h0 = *VREAL(v_info->id,sqcurve2_attr);
+  hh = (h - h0)*(h - h0);  /* square net curvature */
+  energy = a2*hh;
+
+  return energy;
+}
+
+/************************************************************************
+*
+*  function: sqcurve2_string_grad()
+*
+*  purpose:  Calculate square curvature force for string model
+*                Works locally vertex by vertex. 
+*
+*/
+
+REAL sqcurve2_string_grad(v_info)
+struct qinfo *v_info;
+{
+  REAL s1,s2,s1xs2;  /* edge lengths */
+  REAL *side1 = v_info->sides[0][0],*side2 = v_info->sides[0][1];
+  int  i,k;
+  REAL hh,h,h0,a1,a2,energy;
+  REAL a1g[2][MAXCOORD],a2g[2][MAXCOORD];
+  REAL s[2];
+  REAL *side[2];
+
+  if ( v_info->vcount < 3 ) return 0.;
+
+  s1 = sqrt(SDIM_dot(side1,side1));
+  s2 = sqrt(SDIM_dot(side2,side2));
+  s1xs2 = side1[0]*side2[1] - side1[1]*side2[0];
+
+  a1 = s1xs2/s1/s2;  /* sin(theta) */
+  a2 = (s1 + s2)/2;
+  if ( a2 == 0.0 ) return 0.0;
+
+  h = a1/a2;  /* curvature */
+  h0 = *VREAL(v_info->id,sqcurve2_attr);
+  hh = (h - h0)*(h - h0);  /* square net curvature */
+  energy = a2*hh;
+
+
+  side[0] = side1; side[1] = side2; s[0] = s1; s[1] = s2;
+  for ( k = 0 ; k < 2 ; k++ )
+  { REAL coeff = s1xs2/s[k]/s[k]/s[k]/s[1-k];
+    for ( i = 0 ; i < SDIM ; i++ )
+    { a1g[k][i] = (k==i?1:-1)*side[1-k][1-i]/s1/s2 - coeff*side[k][i];
+      a2g[k][i] = 0.5*side[k][i]/s[k];
+    }
+  }
+  for ( i = 0 ; i < SDIM ; i++ ) v_info->grad[0][i] = 0.0;
+  for ( k = 0 ; k < 2 ; k++ )
+    for ( i = 0 ; i < SDIM ; i++ )
+    { REAL f;
+      f = a2g[k][i]*hh + a2*2*(h - h0)*(a1g[k][i]/a2 - a1/a2/a2*a2g[k][i]);
+      v_info->grad[k+1][i] = f; 
+      v_info->grad[0][i] += -f; 
+    }
+  return energy;
+
+}
+
 /*************************************************************************
               Square mean curvature for cylinder in string model
 
@@ -1339,46 +1462,6 @@ struct method_instance *mi;
 
 }
 
-#ifdef XXXX
-/************************************************************************
-*
-*  function: sq_mean_curv_cyl_all()
-*
-*  purpose:  Calculate square curvature energy, grad, hess for string model
-*            Works locally vertex by vertex. Assumes two edges per vertex.
-*
-*/
-
-void xxxcheck(double x[3], double y[3], double *h, double *a, double *dax,
-      double *dar)
-{ double side1[2],side2[2];
-  double r0,r1,r2,s1,s2,area,dadx[2],dadr[2],root;
-  int sign;
-
-  side1[0] = x[1]-x[0];
-  side1[1] = y[1]-y[0];
-  side2[0] = x[2]-x[0];
-  side2[1] = y[2]-y[0];
-
-  r0 = y[1];
-  r1 = y[0];
-  r2 = y[2];
-  s1 = sqrt(SDIM_dot(side1,side1));
-  s2 = sqrt(SDIM_dot(side2,side2));
-  area = s1*(r0 + r1)/2 + s2*(r1 + r2)/2;
-
-  dadx[0] = -side1[0]*(r0+r1)/2/s1 - side2[0]*(r1+r2)/2/s2;
-  dadr[0] = (r1*r1 - r0*r0)/s1/2 + s1/2 + (r1*r1 - r2*r2)/s2/2 + s2/2;
-
-  sign = ( dadr[0] < 0 ) ? -1 : 1;
-  root = sqrt(dadx[0]*dadx[0] + dadr[0]*dadr[0]); 
-  *a = area;
-  *dax = dadx[0];
-  *dar = dadr[0];
-  *h = sign*root/area; /* half factor included here */
-}
-#endif
-
 REAL sq_mean_curv_cyl_all(v_info,mode,power)
 struct qinfo *v_info;
 int mode; /* METHOD_VALUE, METHOD_GRADIENT, METHOD_HESSIAN */
@@ -1406,9 +1489,59 @@ REAL power;
   int sign; /* of curvature */
 
 
+  if ( (v_info->vcount == 2) && (power == 2.0) && !h0_flag &&
+          (fabs(v_info->x[0][1]) < 1e-10) )
+  { /* assume this is an endpoint on the axis, so we do special
+       concentrated squared curvature */
+
+    REAL y1,cosangle;
+    s1 = sqrt(SDIM_dot(side1,side1));
+    y1 = v_info->x[1][1];
+    cosangle = y1/s1;
+    energy = 2*M_PI*(1 - cosangle);
+    if ( mode == METHOD_VALUE )
+      return energy;
+
+    /* gradient */
+    v_info->grad[0][0] = -2*M_PI*y1/s1/s1/s1*side1[0];
+    v_info->grad[0][1] = 0.0; /* assuming constrained to axis */
+    v_info->grad[1][0] = 2*M_PI*y1/s1/s1/s1*side1[0];
+    v_info->grad[1][1] = -2*M_PI/s1 + 2*M_PI*y1/s1/s1/s1*side1[1];
+    if ( mode == METHOD_GRADIENT )
+      return energy;
+
+    /* hessian */
+    v_info->hess[0][0][0][0] = 2*M_PI*(-3*y1/s1/s1/s1/s1/s1*side1[0]*side1[0]
+           + y1/s1/s1/s1);
+    v_info->hess[0][0][0][1] = 0.0;
+    v_info->hess[0][1][0][0] =  -v_info->hess[0][0][0][0];
+    v_info->hess[0][1][0][1] =  2*M_PI*(3*y1/s1/s1/s1/s1/s1*side1[0]*side1[1]
+            - side1[0]/s1/s1/s1);
+
+    v_info->hess[0][0][1][0] = 0.0; /* assuming constrained to axis */
+    v_info->hess[0][0][1][1] = 0.0; /* assuming constrained to axis */
+    v_info->hess[0][1][1][0] = 0.0; /* assuming constrained to axis */
+    v_info->hess[0][1][1][1] = 0.0; /* assuming constrained to axis */
+
+    v_info->hess[1][0][0][0] = v_info->hess[0][1][0][0];
+    v_info->hess[1][0][0][1] = 0.0;
+    v_info->hess[1][1][0][0] = -v_info->hess[1][0][0][0];
+    v_info->hess[1][1][0][1] = 2*M_PI*( 1/s1/s1/s1*side1[0]  -
+           3*y1/s1/s1/s1/s1/s1*side1[0]*side1[1]);
+
+    v_info->hess[1][0][1][0] = v_info->hess[0][1][0][1];
+    v_info->hess[1][0][1][1] = v_info->hess[0][1][1][1];
+    v_info->hess[1][1][1][0] = v_info->hess[1][1][0][1];
+    v_info->hess[1][1][1][1]=  2*M_PI*(2/s1/s1/s1*side1[1] 
+      - 3*y1/s1/s1/s1/s1/s1*side1[1]*side1[1] + y1/s1/s1/s1);
+
+    return energy;
+  } 
 
   if ( v_info->vcount < 3 ) return 0.;
 
+  /* now regular case */
+
   r0 = v_info->x[1][1];
   r1 = v_info->x[0][1];
   r2 = v_info->x[2][1];
@@ -1437,7 +1570,7 @@ REAL power;
     return energy;
 
   /* Gradient */
-
+  /* Note: v_info->grad[i][j] is component j of gradient at vertex i */
 
   dadx[1] = side1[0]*(r0+r1)/2/s1;
   dadr[1] = -(r1*r1 - r0*r0)/s1/2 + s1/2;
@@ -1473,43 +1606,6 @@ REAL power;
          - h/area*dadr[i];
   }
   
-#ifdef XXXX/* for debug */
-{
-double xbug[3];
-double ybug[3];
-double hbug[2],abug[2];
-double dx = 1e-4, dy = 1e-4;
-/*debug*/
-xbug[0] = 0;
-xbug[1] = side1[0];
-xbug[2] = side2[0];
-ybug[0] = r1;
-ybug[1] = r0;
-ybug[2] = r2;
-
-/* check numerically */
-for ( i = 0 ; i < 3; i++ )
-{ REAL dummy;
-  xbug[i] -= dx;
-  xxxcheck(xbug,ybug,&hbug[0],&abug[0],&dummy,&dummy);
-  xbug[i] += 2*dx;
-  xxxcheck(xbug,ybug,&hbug[1],&abug[1],&dummy,&dummy);
-  xbug[i] -= dx;
-  printf("diffs: dadx[%d]: %13.10f  dhdx[%d]: %13.10f\n",
-        i,dadx[i]-(abug[1]-abug[0])/2/dx, i,dhdx[i]-(hbug[1]-hbug[0])/2/dx);
-
-  ybug[i] -= dy;
-  xxxcheck(xbug,ybug,&hbug[0],&abug[0],&dummy,&dummy);
-  ybug[i] += 2*dy;
-  xxxcheck(xbug,ybug,&hbug[1],&abug[1],&dummy,&dummy);
-  ybug[i] -= dy;
-  printf("diffs: dadr[%d]: %13.10f  dhdr[%d]: %13.10f\n",
-        i,dadr[i]-(abug[1]-abug[0])/2/dy, i,dhdr[i]-(hbug[1]-hbug[0])/2/dy);
-
-}
-}
-#endif
-
   if ( power == 2.0 ) 
   { for ( i = 0 ; i < 3 ; i++ )
     { v_info->grad[i][0] = 2*M_PI*(2*hh*dhdx[i]*area + hh*hh*dadx[i])/2;
@@ -1534,6 +1630,8 @@ for ( i = 0 ; i < 3; i++ )
     return energy;
 
   /*  Hessian */
+  /*  Note: v_info->hess[i][ii][j][jj] is component j of vertex i and 
+      component jj of vertex ii */
 
   /* dadx[1] = side1[0]*(r0+r1)/2/s1; */
 
@@ -1804,123 +1902,6 @@ for ( i = 0 ; i < 3; i++ )
          - h/area*ddadrdr[i][j];
   } 
 
-#ifdef XXXX
-/* check numerically */ 
-for ( i = 0 ; i < 3; i++ )
-for ( j = 0 ; j < 3; j++ )
-{ REAL hbug[3][3],abug[3][3],daxbug[3][3],darbug[3][3]; 
-  int ii,jj;
-
-  for ( ii = -1 ; ii <= 1 ; ii ++ )
-   for ( jj = -1 ; jj <= 1 ; jj ++ )
-   { xbug[i] += ii*dx;
-     xbug[j] += jj*dx;
-     xxxcheck(xbug,ybug,&hbug[ii+1][jj+1],&abug[ii+1][jj+1],&daxbug[ii+1][jj+1],
-         &darbug[ii+1][jj+1]);
-     xbug[i] -= ii*dx;
-     xbug[j] -= jj*dx;
-   }
-
-  printf("diffs:  ddadxdx[%d][%d]: %13.10f \n",
-     i,j,ddadxdx[i][j]-(abug[0][0]-abug[0][2]-abug[2][0]+abug[2][2])/4/dx/dx);
-  printf("        dddadxdxdx[0][%d][%d]: %13.10f \n",
-     i,j,dddadxdxdx[0][i][j]-
-          (daxbug[0][0]-daxbug[0][2]-daxbug[2][0]+daxbug[2][2])/4/dx/dx);
-  printf("        dddadrdxdx[0][%d][%d]: %13.10f \n ",
-     i,j,dddadrdxdx[0][i][j]-
-          (darbug[0][0]-darbug[0][2]-darbug[2][0]+darbug[2][2])/4/dx/dx);
-
-  printf("        ddhdxdx[%d][%d]: %13.10f \n",
-     i,j,ddhdxdx[i][j]-(hbug[0][0]-hbug[0][2]-hbug[2][0]+hbug[2][2])/4/dx/dx);
-}
-
-for ( i = 0 ; i < 3; i++ )
-for ( j = 0 ; j < 3; j++ )
-{ REAL hbug[3][3],abug[3][3],daxbug[3][3],darbug[3][3]; 
-  int ii,jj;
-
-  for ( ii = -1 ; ii <= 1 ; ii ++ )
-   for ( jj = -1 ; jj <= 1 ; jj ++ )
-   { xbug[i] += ii*dx;
-     ybug[j] += jj*dy;
-     xxxcheck(xbug,ybug,&hbug[ii+1][jj+1],&abug[ii+1][jj+1],&daxbug[ii+1][jj+1],
-         &darbug[ii+1][jj+1]);
-     xbug[i] -= ii*dx;
-     ybug[j] -= jj*dy;
-   }
-
-  printf("diffs:  ddadxdr[%d][%d]: %13.10f \n",
-     i,j,ddadxdr[i][j]-(abug[0][0]-abug[0][2]-abug[2][0]+abug[2][2])/4/dx/dx);
-  printf("        dddadxdxdr[0][%d][%d]: %13.10f \n",
-     i,j,dddadxdxdr[0][i][j]-
-          (daxbug[0][0]-daxbug[0][2]-daxbug[2][0]+daxbug[2][2])/4/dx/dx);
-  printf("        dddadrdxdr[0][%d][%d]: %13.10f \n ",
-     i,j,dddadrdxdr[0][i][j]-
-          (darbug[0][0]-darbug[0][2]-darbug[2][0]+darbug[2][2])/4/dx/dx);
-
-  printf("        ddhdxdr[%d][%d]: %13.10f \n",
-     i,j,ddhdxdr[i][j]-(hbug[0][0]-hbug[0][2]-hbug[2][0]+hbug[2][2])/4/dx/dx);
-}
-
-for ( i = 0 ; i < 3; i++ )
-for ( j = 0 ; j < 3; j++ )
-{ REAL hbug[3][3],abug[3][3],daxbug[3][3],darbug[3][3]; 
-  int ii,jj;
-
-  printf("\n");
-  for ( ii = -1 ; ii <= 1 ; ii ++ )
-   for ( jj = -1 ; jj <= 1 ; jj ++ )
-   { ybug[i] += ii*dy;
-     xbug[j] += jj*dx;
-     xxxcheck(xbug,ybug,&hbug[ii+1][jj+1],&abug[ii+1][jj+1],&daxbug[ii+1][jj+1],
-         &darbug[ii+1][jj+1]);
-     ybug[i] -= ii*dy;
-     xbug[j] -= jj*dx;
-   }
-
-  printf("diffs:  ddadrdx[%d][%d]: %13.10f \n",
-     i,j,ddadrdx[i][j]-(abug[0][0]-abug[0][2]-abug[2][0]+abug[2][2])/4/dx/dx);
-  printf("        dddadxdrdx[0][%d][%d]: %13.10f \n",
-     i,j,dddadxdrdx[0][i][j]-
-          (daxbug[0][0]-daxbug[0][2]-daxbug[2][0]+daxbug[2][2])/4/dx/dx);
-  printf("        dddadrdrdx[0][%d][%d]: %13.10f \n ",
-     i,j,dddadrdrdx[0][i][j]-
-          (darbug[0][0]-darbug[0][2]-darbug[2][0]+darbug[2][2])/4/dx/dx);
-
-  printf("        ddhdrdx[%d][%d]: %13.10f \n",
-     i,j,ddhdrdx[i][j]-(hbug[0][0]-hbug[0][2]-hbug[2][0]+hbug[2][2])/4/dx/dx);
-
-}
-
-for ( i = 0 ; i < 3; i++ )
-for ( j = 0 ; j < 3; j++ )
-{ REAL hbug[3][3],abug[3][3],daxbug[3][3],darbug[3][3]; 
-  int ii,jj;
-
-  printf("\n");
-  for ( ii = -1 ; ii <= 1 ; ii ++ )
-   for ( jj = -1 ; jj <= 1 ; jj ++ )
-   { ybug[i] += ii*dy;
-     ybug[j] += jj*dy;
-     xxxcheck(xbug,ybug,&hbug[ii+1][jj+1],&abug[ii+1][jj+1],&daxbug[ii+1][jj+1],
-         &darbug[ii+1][jj+1]);
-     ybug[i] -= ii*dy;
-     ybug[j] -= jj*dy;
-   }
-  printf("diffs:  ddadrdr[%d][%d]: %13.10f \n",
-     i,j,ddadrdr[i][j]-(abug[0][0]-abug[0][2]-abug[2][0]+abug[2][2])/4/dx/dx);
-  printf("        dddadxdrdr[0][%d][%d]: %13.10f \n",
-     i,j,dddadxdrdr[0][i][j]-
-          (daxbug[0][0]-daxbug[0][2]-daxbug[2][0]+daxbug[2][2])/4/dx/dx);
-  printf("        dddadrdrdr[0][%d][%d]: %13.10f \n ",
-     i,j,dddadrdrdr[0][i][j]-
-          (darbug[0][0]-darbug[0][2]-darbug[2][0]+darbug[2][2])/4/dx/dx);
-
-  printf("        ddhdrdr[%d][%d]: %13.10f \n",
-     i,j,ddhdrdr[i][j]-(hbug[0][0]-hbug[0][2]-hbug[2][0]+hbug[2][2])/4/dx/dx);
-}
-#endif
-
   if ( power == 2.0 ) 
   { for ( i = 0 ; i < 3 ; i++ )
      for ( j = 0 ; j < 3 ; j++ )
@@ -2022,3 +2003,119 @@ struct qinfo *v_info;
 {
   return sq_mean_curv_cyl_all(v_info,METHOD_HESSIAN,2.0);
 }
+
+/*************************************************************************
+   Square gaussian curvature for surface of revolution in string model.
+   Contributed by John Frank, jrf@mit.edu
+
+   Cylindrical axis is x axis.
+   Includes 2*pi factor.
+**************************************************************************/
+
+REAL sq_gauss_curv_cyl_all ARGS(( struct qinfo *, int));
+
+/************************************************************************
+*
+*  function: sq_gauss_curv_cyl_init()
+*
+*  purpose:  Any overall initialization needed each time the method
+*            is evaluated.
+*/
+void sq_gauss_curv_cyl_init(mode,mi)
+int mode;
+struct method_instance *mi;
+{ 
+  if ( web.modeltype != LINEAR )
+     kb_error(4486,"sq_gaussian_curv_cyl method only for LINEAR model.\n",
+        RECOVERABLE);
+
+}
+
+/************************************************************************
+*
+* function: sq_gauss_curv_cyl_all()
+*
+* purpose: Combined calculations for one vertex for sq_gaussian_curv_cyl
+*          method.
+*/
+
+REAL sq_gauss_curv_cyl_all(v_info,mode)
+struct qinfo *v_info;
+int mode; /* METHOD_VALUE, METHOD_GRADIENT, or METHOD_HESSIAN */
+{ REAL energy = 0.0;
+
+  /* Notes on incoming data:
+       v_info->vcount is number of vertices in v_info; it will be 3
+           for an interior vertex and 2 for an endpoint.
+       v_info->x[0][] is the coordinates of the vertex in question.
+       v_info->x[1][] is the coordinates of one neighbor vertex.
+       v_info->x[2][] is the coordinates of the other neighbor vertex.
+       v_info->sides[0][0][] is the vector from vertex 0 to vertex 1.
+       v_info->sides[0][1][] is the vector from vertex 0 to vertex 2.
+  */
+
+  /* calculate value for single vertex here */
+
+  if ( mode == METHOD_VALUE )
+    return energy;
+
+  /* Gradient */
+
+  /* calculate gradient for the energy of the vertex here */
+
+  if ( mode == METHOD_GRADIENT )
+    return energy;
+
+  /*  Hessian */
+
+  /* calculate hessian for the energy of the vertex here */
+
+  return energy;
+
+}
+
+/************************************************************************
+*
+*  function: sq_gauss_curv_cyl_value()
+*
+*  purpose:  Calculate square gaussian curvature energy for surface
+*            of revolution in string model.
+*            Works locally vertex by vertex. Assumes two edges per vertex.
+*
+*/
+
+REAL sq_gauss_curv_cyl_value(v_info)
+struct qinfo *v_info;
+{
+  return sq_gauss_curv_cyl_all(v_info,METHOD_VALUE);
+}
+
+/************************************************************************
+*
+*  function: sq_gauss_curv_cyl_grad()
+*
+*  purpose:  Calculate square gaussian curvature gradient for 
+*            surface of revolution in string model.
+*
+*/
+
+REAL sq_gauss_curv_cyl_grad(v_info)
+struct qinfo *v_info;
+{
+  return sq_gauss_curv_cyl_all(v_info,METHOD_GRADIENT);
+}
+
+/************************************************************************
+*
+*  function: sq_gauss_curv_cyl_hess()
+*
+*  purpose:  Calculate square curvature hessian for surface of revolution
+*            in string model.
+*
+*/
+
+REAL sq_gauss_curv_cyl_hess(v_info)
+struct qinfo *v_info;
+{
+  return sq_gauss_curv_cyl_all(v_info,METHOD_HESSIAN);
+}
