@@ -187,7 +187,7 @@ int mode; /* METHOD_VALUE, METHOD_GRADIENT, or METHOD_HESSIAN */
 
   if ( v_info->vcount > MAXV )
   { a = (REAL*)mycalloc(5*pairs,sizeof(REAL));
-     ds1 = dmatrix(0,12*v_info->vcount,0,SDIM-1);
+    ds1 = dmatrix(0,12*v_info->vcount,0,SDIM-1);
   } else
   { memset((char*)aa,0,sizeof(REAL)*5*pairs);
     a = aa;
@@ -429,15 +429,18 @@ int mode; /* METHOD_VALUE, METHOD_GRADIENT, or METHOD_HESSIAN */
       if ( fn != 0.0 )
         for ( k = 0 ; k < pairs ; k++ )
         { REAL *grad2 = (k+1==pairs)?v_info->grad[final_edge+1]:v_info->grad[k+2];
+          REAL g1,g2,g3;
           for ( i = 0 ; i < SDIM ; i++ )
-          { g = 0.75*hh0*hh0*dAdv1[k][i] 
-                    + 1.5*area/fn*hh0*dffdv1[k][i]
-                    - 1.5*area*ff*hh0/fn/fn*dfndv1[k][i]; 
+          { g1 = 0.75*hh0*hh0*dAdv1[k][i]; 
+                g2 = 1.5*area/fn*hh0*dffdv1[k][i];
+                 g3 =   - 1.5*area*ff*hh0/fn/fn*dfndv1[k][i]; 
+             g = g1 + g2 + g3;
              v_info->grad[k+1][i] += g;
              v_info->grad[0][i] -= g;
-             g = 0.75*hh0*hh0*dAdv2[k][i] 
-                    + 1.5*area/fn*hh0*dffdv2[k][i]
-                    - 1.5*area*ff*hh0/fn/fn*dfndv2[k][i]; 
+             g1 = 0.75*hh0*hh0*dAdv2[k][i] ;
+                  g2 =  1.5*area/fn*hh0*dffdv2[k][i];
+                 g3 =    - 1.5*area*ff*hh0/fn/fn*dfndv2[k][i]; 
+             g = g1 + g2 + g3;
              grad2[i] += g;
              v_info->grad[0][i] -= g;
             }
@@ -513,8 +516,8 @@ int mode; /* METHOD_VALUE, METHOD_GRADIENT, or METHOD_HESSIAN */
           break;
       case NORMAL_SQ:
           afnfn = 1.5*area/fn/fn;
-          afffnfn = 1.5*area*hh0/fn;
           fffnfn = 1.5*hh0/fn;
+          afffnfn = 1.5*area*hh0/fn;
           ffafnfnfn = 1.5*area*(ff/fn + hh0)/fn/fn;
           afffffnfnfnfn = 1.5*area*ff*(ff/fn+2*hh0)/fn/fn/fn;
           fffffnfnfna = 1.5*ff*hh0/fn/fn*area;
@@ -524,6 +527,7 @@ int mode; /* METHOD_VALUE, METHOD_GRADIENT, or METHOD_HESSIAN */
           break;
     }
 
+
     for ( k = 0 ; k < pairs ; k++ ) 
     { REAL co1 = 0.25/a[k]/a[k]/a[k];
       REAL co2 = 0.125/a[k]/a[k];
@@ -1411,3 +1415,128 @@ struct qinfo *v_info;
 {
    return star_sqcurve_method_all(v_info,METHOD_HESSIAN);  
 }
+
+/*************************************************************************
+**************************************************************************
+ 
+  Method circle_willmore
+
+  Alexander Bobenko's circle-based discrete Willmore energy.
+  Is conformally invariant.
+  At each vertex, energy is the sum of the angles between facet
+  circumcircles - 2*pi.  More simply done as edge quantity, since
+  angles at each end are the same.
+
+  For edge e, if adjacent facet edge loops are a e d and b c -e, then
+  circle angle beta for edge has
+  
+  cos(beta) = (<a,c><b,c>-<a,b><c,d>-<b,c><d,a>)/|a|/|b|/|c|/|d|
+
+  For now, assumes all vertices are faceted, and fully starred.
+  
+   Severe numerical difficulties: Not smooth when angle beta is zero,
+  which is all too common.  Set of zero angles should be codimension 2,
+  which means generally avoided, but still crops up.
+
+**************************************************************************/
+
+void circle_willmore_init(mode,mi)
+int mode; /* METHOD_VALUE or METHOD_GRADIENT */
+struct method_instance *mi;
+{ REAL vertex_adjust;
+
+  if ( web.modeltype != LINEAR )
+    kb_error(4007,"circle_willmore method requires the LINEAR model.\n",
+      RECOVERABLE);
+  if ( web.representation != SOAPFILM )
+    kb_error(3880,"circle_willmore method requires the SOAPFILM model.\n",
+      RECOVERABLE);
+
+  vertex_adjust = -M_PI*web.skel[VERTEX].count;
+
+  binary_tree_add(mi->value_addends,vertex_adjust);
+}
+
+REAL circle_willmore_all(e_info,mode)
+struct qinfo *e_info;
+int mode; /* METHOD_VALUE, METHOD_GRADIENT, or METHOD_HESSIAN */
+{ REAL *q = e_info->sides[0][0];
+  REAL *p = e_info->sides[0][1];
+  REAL *r = e_info->sides[0][2];
+  REAL pp = dot(p,p,SDIM);
+  REAL qq = dot(q,q,SDIM);
+  REAL rr = dot(r,r,SDIM);
+  REAL pq = dot(p,q,SDIM);
+  REAL pr = dot(p,r,SDIM);
+  REAL qr = dot(q,r,SDIM);
+  REAL value;
+  REAL cosbeta;
+  REAL dcdp[MAXCOORD],dcdq[MAXCOORD],dcdr[MAXCOORD];
+  REAL coeff;
+  int i;
+
+  cosbeta = (rr*pq - qq*pr - pp*rr + pp*qr)/sqrt(pp)/sqrt(rr)/
+              sqrt(qq-2*pq+pp)/sqrt(rr-2*qr+qq);
+  if ( fabs(cosbeta) > 1.0 )
+  { sprintf(errmsg,"edge %s cosine of angle out of range in circular_willmore",
+      ELNAME(e_info->id));
+     kb_error(3850,errmsg,WARNING);
+  }
+  value = acos(cosbeta);
+
+  if ( mode == METHOD_VALUE ) 
+    return value;
+ 
+  /* Gradient */
+  if ( cosbeta*cosbeta >= 1.0 ) coeff = 0.0;
+  coeff = -1/sqrt(1 - cosbeta*cosbeta);
+  for ( i = 0 ; i < SDIM ; i++ )
+  { dcdp[i] = (rr*q[i] - qq*r[i] - 2*rr*p[i] + 2*qr*p[i])/sqrt(pp)/sqrt(rr)/
+              sqrt(qq-2*pq+pp)/sqrt(rr-2*qr+qq)
+            - cosbeta/pp*p[i]
+            - cosbeta/(qq-2*pq+pp)*(-q[i] + p[i]);
+    dcdq[i] = (rr*p[i] - 2*pr*q[i] + pp*r[i])/sqrt(pp)/sqrt(rr)/
+              sqrt(qq-2*pq+pp)/sqrt(rr-2*qr+qq)
+            - cosbeta/(qq-2*pq+pp)*(q[i] - p[i])
+            - cosbeta/(rr-2*qr+qq)*(-r[i] + q[i]);
+    dcdr[i] = (2*r[i]*pq - qq*p[i] - 2*pp*r[i]+ pp*q[i])/sqrt(pp)/sqrt(rr)/
+              sqrt(qq-2*pq+pp)/sqrt(rr-2*qr+qq)
+            - cosbeta/rr*r[i]
+            - cosbeta/(rr-2*qr+qq)*(r[i] - q[i]);
+    e_info->grad[1][i] = coeff*dcdq[i]; 
+    e_info->grad[0][i] -= coeff*dcdq[i]; 
+    e_info->grad[2][i] = coeff*dcdp[i]; 
+    e_info->grad[0][i] -= coeff*dcdp[i]; 
+    e_info->grad[3][i] = coeff*dcdr[i]; 
+    e_info->grad[0][i] -= coeff*dcdr[i]; 
+  }
+
+  if ( mode == METHOD_GRADIENT ) 
+    return value;
+
+
+  /* Hessian */
+ 
+  
+
+    return value;
+  
+}
+
+REAL circle_willmore_value(e_info)
+struct qinfo *e_info;
+{ return circle_willmore_all(e_info,METHOD_VALUE);
+}
+
+REAL circle_willmore_grad(e_info)
+struct qinfo *e_info;
+{ return circle_willmore_all(e_info,METHOD_GRADIENT);
+}
+
+REAL circle_willmore_hess(e_info)
+struct qinfo *e_info;
+{ return circle_willmore_all(e_info,METHOD_HESSIAN);
+}
+
+
+
