@@ -20,6 +20,9 @@ struct blocklist_struct *blocklist[NUMELEMENTS]; /* list of allocated blocks */
 int blockcount[NUMELEMENTS];  /* how many blocks allocated */
 int blockmax[NUMELEMENTS];  /* length of blocklist */
 
+
+int sparse_ibase_flag; /* for permitting sparse ibase */
+
 /* individual indirect block pointer arrays, handy for debugging */
 /* set to web.skel[].ibase */
 INDIRECT_TYPE *vibase;
@@ -28,16 +31,8 @@ INDIRECT_TYPE *fibase;
 INDIRECT_TYPE *bibase;
 INDIRECT_TYPE *feibase;
 
-/* Starting web structure */
-struct webstruct web = {
-{ { VERTEX, 1, 0,  NULL, 0, NULLID, NULLID, 0, NULLID },
- { EDGE  , 2, 0,  NULL, 0, NULLID, NULLID, 0, NULLID },
- { FACET , 3, 0,  NULL, 0, NULLID, NULLID, 0, NULLID },
- { BODY  , 0, 0,   NULL, 0, NULLID, NULLID, 0, NULLID },
- { FACETEDGE, 0,0, NULL, 0, NULLID, NULLID, 0, NULLID } },
- {0,0,0,0,0},{0,0,0,0,0},3,2,LINEAR
-     };
-
+/* The web structure */
+struct webstruct web; 
 
 element_id
   NULLVERTEX  =  (element_id)VERTEX  << TYPESHIFT, 
@@ -86,29 +81,33 @@ struct facetedge * Feptr(id) facetedge_id id; { return feptr(id); }
 void expand(type,newsize)
 int type;  /* VERTEX, etc. */
 int newsize; /* new size of structure */
-{ char *newblock;
+{ char *newblock,*oldblock;
   int oldsize = web.sizes[type];
   int count = web.skel[type].maxcount;
   int i,n;                                   
   char *newptr,*oldptr;
+  int copysize;
 
   /* round up newsize to alignment for doubles */
   newsize = ((newsize+sizeof(REAL)-1)/sizeof(REAL))*sizeof(REAL);
   if ( newsize == web.sizes[type] ) return; /* don't have to expand */
   web.sizes[type] = newsize;
   if ( count == 0 ) return; /* don't have to reallocate space */
+  copysize = (newsize < oldsize) ? newsize : oldsize;
 
   ENTER_GRAPH_MUTEX;
   /* Don't mess with structures while graph thread using them */
 
+#ifdef HASH_ID
+  web.skel[type].freehead = NULL;
+#endif
+
   for ( n = 0 ; n < blockcount[type] ; n++ )
   { INDIRECT_TYPE *iptr;
      char *spot;
      struct blocklist_struct *b = blocklist[type] + n;
-     int copysize;
 
      newblock = mycalloc(b->count,newsize);
-     copysize = (newsize < oldsize) ? newsize : oldsize;
      for ( i = 0, oldptr = (char*)(b->blockptr), newptr = newblock; 
              i < b->count ; i++, oldptr += oldsize, newptr += newsize )
      { memcpy(newptr,oldptr,copysize);
@@ -119,24 +118,38 @@ int newsize; /* new size of structure */
            remote_elements[type].ibase[id_machine(id)][ordinal(id)] = 
                (struct element *)newptr;   
           }
-        }
-        #endif
+       }
+       #endif
      }
-     myfree((char*)b->blockptr);
+     oldblock = (char*)b->blockptr;
      b->blockptr = (struct element*)newblock;
      /* update indirect pointers */
      /* keeping in mind structures not necessarily in id order 
         due to reorder command */
      for ( i=0,spot=newblock, iptr=web.skel[type].ibase+b->start_ord ; 
                 i<b->count ; i++,spot+=newsize,iptr++ )
-	 {  
+     {  
 #ifdef MPI_EVOLVER
          element_id id = ((struct element *)spot)->local_id;
 #else
          element_id id = ((struct element *)spot)->self_id;
 #endif
+
+#ifdef HASH_ID
+      { struct element *el_ptr;
+        el_ptr = (struct element *)spot;
+        if ( el_ptr->self_id ) /* replace in hash table */
+          elhash_replace(id,el_ptr);
+        else /* add to freelist */
+        { *(struct element **)&(el_ptr->forechain) = web.skel[type].freehead;
+          web.skel[type].freehead = el_ptr;
+        }
+      }
+#else
         web.skel[type].ibase[ordinal(id)] = (struct element *)spot;
- 	 }
+#endif
+     }
+     myfree(oldblock);
   }
 
   LEAVE_GRAPH_MUTEX;
@@ -155,10 +168,13 @@ void extend(type,mode)
 int type;
 int mode; /* EXTEND_BATCH or EXTEND_FOR_REFINE */
 {
+#ifdef HASH_ID
+  elhash_extend(type,mode);
+#else
   int number=0;  /* of new items to allocate */
   char *newblock;
   INDIRECT_TYPE *newiblock;
-  element_id id,oldfreehead;
+  element_id id,backid;
   struct element *newptr = NULL;
   int k;
   int allocsize;
@@ -226,41 +242,68 @@ int mode; /* EXTEND_BATCH or EXTEND_FOR_REFINE */
   }
 
   /* add to end of freelist */
-  neword = web.skel[type].maxcount;
+
+  /* find first empty slot */
+  if ( sparse_ibase_flag )
+  { /* search for empty ibase slots from start */
+    for ( neword = 0 ; neword < web.skel[type].ialloc ; neword++ )
+      if ( web.skel[type].ibase[neword] == NULL )
+        break;
+  }
+  else /* dense ibase */
+    neword = web.skel[type].maxcount;
+
   id = ((element_id)type << TYPESHIFT) | VALIDMASK | neword; 
 #ifdef MPI_EVOLVER
-  id |= this_machine << MACH_ID_SHIFT;  
+  id |= (element_id)this_machine << MACH_ID_SHIFT;  
 #endif
-  oldfreehead = web.skel[type].free;
-  if ( valid_id(oldfreehead) )
-  { element_id nextid = oldfreehead;
-    while ( valid_id(elptr(nextid)->forechain) )
-      nextid = elptr(nextid)->forechain;
-    elptr(nextid)->forechain = id;
+
+  if ( valid_id(web.skel[type].freelast) )
+  { backid = web.skel[type].freelast;
+    elptr(backid)->forechain = id;
   }
-  else web.skel[type].free = id;
+  else
+  { backid = NULLID;
+    web.skel[type].free = id;
+  }
+
   for ( k = 0 ; k < number ; k++ )
   { newptr = (struct element *)(newblock + k*web.sizes[type]); 
     web.skel[type].ibase[neword] = newptr;
     newptr->self_id = id;
     #ifdef MPI_EVOLVER
-    newptr->local_id = id;
+      newptr->local_id = id;
     #endif
     if ( k < number-1 )
-    { neword++;
+    {
+      if ( sparse_ibase_flag )
+      { for ( neword = 0 ; neword < web.skel[type].ialloc ; neword++ )
+          if ( web.skel[type].ibase[neword] == NULL )
+            break;
+      }
+      else 
+        neword++;
       id = ((element_id)type << TYPESHIFT) | VALIDMASK | neword; 
 #ifdef MPI_EVOLVER
-      id |= this_machine << MACH_ID_SHIFT;  
+      id |= (element_id)this_machine << MACH_ID_SHIFT;  
 #endif
 
       newptr->forechain = id;
     }
+    newptr->backchain = backid;
+    backid = newptr->self_id;
+
   }
 
+  web.skel[type].freelast = backid;
   web.skel[type].maxcount += number;
   
 extend_exit:
   LEAVE_GRAPH_MUTEX;
+  
+/* end ndef HASH_ID */
+#endif
+
 
 }
 
@@ -272,14 +315,19 @@ extend_exit:
 *
 */
 
-element_id new_element(type,parent)
+element_id new_element(type,parent,desired_id)
 int type;
 element_id parent; /* for inherited stuff */
+element_id desired_id; /* or NULLID */
 {
   element_id newid;
   struct element *newptr,*last;
   ORDTYPE ord;
   
+#ifdef HASH_ID
+  newptr = elhash_new_element(type,desired_id);
+  newid = newptr->self_id;
+#else
   newid = web.skel[type].free;
   
   if ( !valid_id(newid) ) /* free list empty */
@@ -297,6 +345,11 @@ element_id parent; /* for inherited stuff */
 
   /* remove from free chain */
   web.skel[type].free = newptr->forechain;
+  if ( valid_id(web.skel[type].free) )
+    elptr(web.skel[type].free)->backchain = NULLID;
+  else
+    web.skel[type].freelast = NULLID;
+#endif
 
   /* clean out old info */
   ord = ordinal(newid);
@@ -306,14 +359,14 @@ element_id parent; /* for inherited stuff */
   }
   newptr->original = NULLID;
 
-
+#ifndef HASH_ID
   if ( match_id_flag && datafile_flag )
   { /* link in numerical order */
     int i;
     struct element *prev,*next;
     for ( i = ord-1 ; i >= 0 ; i-- )
     { prev = web.skel[type].ibase[i];
-      if ( prev->attr & ALLOCATED )
+      if ( prev && (prev->attr & ALLOCATED) )
       { if ( valid_id(prev->forechain) )
         { next = elptr(prev->forechain);
           next->backchain = newid;
@@ -335,6 +388,7 @@ element_id parent; /* for inherited stuff */
     if ( !valid_id(newptr->forechain) ) web.skel[type].last = newid;
   }
   else
+  #endif
   { /* add to end of in-use chain */
     newptr->forechain = NULLID;
     newptr->backchain = web.skel[type].last;
@@ -416,7 +470,7 @@ element_id parent; /* for inherited stuff */
 * purpose: mark element as unallocated, but leave it in the chain of
 *          used elements.  Moved to freelist later by free_discards.
 */
-void free_element(id) 
+void free_element(id)  
 element_id id;
 {
   struct element *ptr;
@@ -425,8 +479,8 @@ element_id id;
   if ( !valid_element(id) )
   {
     sprintf(errmsg,
-      "Internal error: Trying to free invalid element type %d id %s \n",
-            type,ELNAME(id));
+      "Internal error: Trying to free invalid element %s id %s \n",
+            typenames[type],ELNAME(id));
     kb_error(1311,errmsg,WARNING);
     return;
   }
@@ -549,6 +603,7 @@ element_id get_ordinal_id(type,ord)
 int type; /* type of element */
 int ord;  /* ordinal of element, signed */
 { element_id id;
+  struct element *ep;
 
   if ( (type < 0) || (type > NUMELEMENTS) ) return NULLID;
   if ( abs(ord) > web.skel[type].max_ord ) return NULLID;
@@ -556,11 +611,12 @@ int ord;  /* ordinal of element, signed */
   if ( ord < 0 ) invert(id);
 
 #ifdef MPI_EVOLVER
-  id |= this_machine << MACH_ID_SHIFT;
+  id |= (element_id)this_machine << MACH_ID_SHIFT;
 #endif
-  if ( !(get_attr(id) & ALLOCATED) )
-     return NULLID;
- return id;
+  ep = elptr(id);
+  if ( ep == NULL ) return 0;
+  if ( ep->attr & ALLOCATED ) return id;
+  return NULLID;
 }
 
 int generate_all(type,idptr,sentinel)  /* re-entrant */
@@ -631,11 +687,14 @@ void memory_report()
     if ( verbose_flag )
     { /* report element sizes */
       for ( k = 0 ; k < NUMELEMENTS ; k++ )
-      { sprintf(msg,"%10.10s size: %4d bytes\n",typenames[k],web.sizes[k]);
+      { sprintf(msg,"%10.10s size: %4d bytes;  number allocated: %d\n",
+          typenames[k],web.sizes[k],web.skel[k].maxcount);
         outstring(msg);
       }
       sprintf(msg,"quantity size:   %4d bytes\n",sizeof(struct gen_quant));
+      outstring(msg);
       sprintf(msg,"instance size:   %4d bytes\n",sizeof(struct method_instance));
+      outstring(msg);
     }
 
 #if defined(_WIN32) && defined(_HEAPOK)
@@ -651,7 +710,7 @@ void memory_report()
       size_t mem_use=0,mem_free=0;
       char * heaptop = NULL; 
       char * heapstart = NULL;
-      MEMORYSTATUSEX memstat;
+      MEMORYSTATUS memstat;
 
       hinfo._pentry = NULL;
       while ( _heapwalk(&hinfo) == _HEAPOK )
@@ -679,16 +738,16 @@ void memory_report()
       sprintf(errmsg,"blocks free:    %d    memory free:    %ld \n",
                  b_free,mem_free);
       outstring(errmsg);
-      sprintf(errmsg,"Heap top: %P\n",heaptop);
+      sprintf(errmsg,"Heap top: %p\n",heaptop);
       outstring(errmsg);
       sprintf(errmsg,"Heap size: %4.2f MB\n",
              (heaptop-heapstart)/1024./1024.);
       outstring(errmsg);
 
       memstat.dwLength = sizeof(memstat);
-      GlobalMemoryStatusEx(&memstat);
-      sprintf(errmsg,"Physical memory size: %I64d bytes   Virtual memory top: %I64X\n",
-           memstat.ullTotalPhys,memstat.ullTotalVirtual);
+      GlobalMemoryStatus(&memstat);
+      sprintf(errmsg,"Physical memory size: %d bytes   Virtual memory top: %X\n",
+           memstat.dwTotalPhys,memstat.dwTotalVirtual);
       outstring(errmsg);
     }
 #endif
@@ -750,11 +809,13 @@ void reset_skeleton()
   int maxperm   = web.max_perm_globals;
   struct global *perm = perm_globals(0);
 
+  ENTER_GRAPH_MUTEX
+
   gauss1Dpt = NULL; 
   gauss1Dwt = NULL;
 
   memset((char *)&web,0,sizeof(web));  /* total clean out */
-
+  
   web.perm_global_count = permcount;
   web.max_perm_globals = maxperm;
   dy_perm_globals = perm;
@@ -785,6 +846,8 @@ void reset_skeleton()
   web.skel[FACET].dimension = 2;
   web.skel[BODY].dimension = 3;
 
+  LEAVE_GRAPH_MUTEX 
+
   /* set up permanent attributes, empty to start with */
   /* have REAL attributes first, so can align on 8-byte */
   /* Be sure the order given here is same as order in skeleton.h */
@@ -845,7 +908,7 @@ void reset_skeleton()
 *
 * function: free_discards()
 *
-* purpose: totally free up discard list.  to be called only when
+* purpose: Totally free up discard list.  To be called only when
 *          no lists are being used.
 */
 void free_discards(mode)
@@ -863,18 +926,33 @@ int mode; /* DISCARDS_ALL or DISCARDS_SOME */
       next_id = ptr->forechain;
       if ( !(ptr->attr & ALLOCATED) )
       { /* move to free list */
+#ifdef MPI_EVOLVER
+        element_id loc_id = ptr->local_id;
+#else
+        element_id loc_id = id;
+#endif
         if ( valid_id(ptr->forechain) )
           elptr(ptr->forechain)->backchain = ptr->backchain;
         else web.skel[type].last = ptr->backchain;
         if ( valid_id(ptr->backchain) )
           elptr(ptr->backchain)->forechain = ptr->forechain;
         else web.skel[type].used = ptr->forechain;
+
+#ifdef HASH_ID
+        *(struct element **)&(ptr->forechain) = web.skel[type].freehead; 
+        web.skel[type].freehead = ptr;
+        elhash_delete(id);
+        ptr->self_id = NULLID;
+        web.skel[type].alloc--;
+#else
+        if ( valid_id(web.skel[type].free) )
+          elptr(web.skel[type].free)->backchain = loc_id;
+        else
+          web.skel[type].freelast = loc_id;
+
         ptr->forechain = web.skel[type].free;
         ptr->backchain = NULLID;
-#ifdef MPI_EVOLVER
-        web.skel[type].free = ptr->local_id;
-#else
-        web.skel[type].free = id;
+        web.skel[type].free = loc_id;
 #endif
       }
       id = next_id;
@@ -894,34 +972,90 @@ int mode; /* DISCARDS_ALL or DISCARDS_SOME */
 
 void move_to_free_front(type,id)
 int type; /* element type */
-int id;    /* element id    */
+int id;    /* element number    */
 { int ord = id - 1;
-  element_id eid;
-  struct element *pptr,*bbptr;
-  element_id prev;
 
+
+#ifdef HASH_ID
+ if ( !match_id_flag ) return; /* for old way */
+ web.skel[type].free_spot = ord;
+#else
+
+  element_id eid;
+  struct element *eptr,*fptr,*bptr;
+ 
   if ( !match_id_flag ) return; /* for old way */
-  while ( id > web.skel[type].maxcount ) extend(type,EXTEND_BATCH);
-  /* linear search through free list */
-  eid = web.skel[type].free;
-  prev = NULLID;
-  while ( valid_id(eid) )
-  { if ( ordinal(eid) == ord ) break;
-    prev = eid;
-    eid = elptr(eid)->forechain;
+
+  if ( sparse_ibase_flag )
+  { /* extend ibase with empty slots far enough */
+    if ( id > web.skel[type].ialloc )
+    { int imax = web.skel[type].ialloc > 1024 ? 2*web.skel[type].ialloc : 1024;
+      while ( imax < id ) 
+        imax *= 2;
+      web.skel[type].ibase = (INDIRECT_TYPE*)kb_realloc((char*)web.skel[type].ibase,
+                                 imax*sizeof(INDIRECT_TYPE));
+      web.skel[type].ialloc = imax;
+    }
+    if ( !valid_id(web.skel[type].free) )
+      extend(type,EXTEND_BATCH); /* refill freelist */ 
+    if ( web.skel[type].ibase[ord] == NULL ) 
+    { /* move first of freelist to desired spot */
+      element_id free_id = web.skel[type].free;
+      int free_ord;
+      free_ord = ordinal(free_id);
+      eptr = web.skel[type].ibase[ord] = web.skel[type].ibase[free_ord];
+      web.skel[type].ibase[free_ord] = NULL;
+      eptr->self_id = ((element_id)type << TYPESHIFT) | VALIDMASK | ord; 
+#ifdef MPI_EVOLVER
+      eptr->self_id |= (element_id)this_machine << MACH_ID_SHIFT;  
+#endif
+      web.skel[type].free = eptr->self_id;
+      if ( !valid_id(eptr->forechain) )
+        web.skel[type].freelast = eptr->self_id;
+         
+      return; 
+    } 
   }
-  if ( !valid_id(eid) )
-  {  kb_error(2187,"Internal error: Cannot find element in free list.\n",
-        DATAFILE_ERROR);
-      return;
+  else
+  { /* dense ibase, extend and allocate empty structures */
+    while ( id > web.skel[type].maxcount ) 
+     extend(type,EXTEND_BATCH);
   }
+
+  eid = ((element_id)type << TYPESHIFT) | VALIDMASK | abs(ord);
+  #ifdef MPI_EVOLVER
+  eid |= (element_id)this_machine << MACH_ID_SHIFT;  
+  #endif
+
+  eptr = elptr(eid);
+  if ( valid_element(eid) )
+  { kb_error(2187,"Internal error: Cannot find element in free list.\n",
+       DATAFILE_ERROR);
+    return;
+  }
+
   if ( eid == web.skel[type].free ) return; /* already in place */
-  /* now cut and paste */
-  bbptr = elptr(eid);
-  pptr = elptr(prev);
-  pptr->forechain = bbptr->forechain; /* cut */
-  bbptr->forechain = web.skel[type].free; /* paste */
+  if ( eid == web.skel[type].freelast ) 
+    web.skel[type].freelast = eptr->backchain;
+
+  /* now cut out of free list */
+  if ( valid_id(eptr->backchain) )
+  { bptr = elptr(eptr->backchain);
+    bptr->forechain = eptr->forechain; /* cut */
+  }
+  if ( valid_id(eptr->forechain) )
+  { fptr = elptr(eptr->forechain);
+    fptr->backchain = eptr->backchain;
+  }
+
+  /* paste at front of free list */
+  if ( valid_id(web.skel[type].free) )
+  { fptr = elptr(web.skel[type].free);
+    fptr->backchain = eid;
+  }
+  eptr->forechain = web.skel[type].free; 
   web.skel[type].free = eid;
+#endif
 }
 
 /*********************************************************************
@@ -970,6 +1104,7 @@ void reorder_storage()
       id = ep->forechain;
       spot += web.sizes[n];
     }
+    #ifndef HASH_ID
     /* free list at end */
     id = web.skel[n].free;
     while ( valid_id(id) )
@@ -978,6 +1113,7 @@ void reorder_storage()
       id = ep->forechain;
       spot += web.sizes[n];
     }
+    #endif
   }
 
   /* sort elements in use */
@@ -994,8 +1130,8 @@ void reorder_storage()
       }
     if ( key_offset < 0 )
     { for ( n = 0 ; n < NUMELEMENTS ; n++ ) myfree((char*)newblock[n]);
-      kb_error(2188,"reorder_storage: Key_order attribute not defined.\n",
-         RECOVERABLE);
+      sprintf(errmsg,"reorder_storage: %s attribute not defined.\n",keynames[i]);
+      kb_error(2188,errmsg, RECOVERABLE);
     }
     qsort((char*)newblock[i],web.skel[i].count,web.sizes[i],FCAST esort);
   }
@@ -1060,7 +1196,7 @@ void reorder_storage()
 void renumber_all()
 { int type;
   struct element *ep=NULL;
-  element_id id,newid;
+  element_id id;
   int k;
   int dim,off,dima,offa,dimb,offb,dimc,offc;
   struct element **newibase[NUMELEMENTS];
@@ -1090,7 +1226,8 @@ void renumber_all()
   FOR_ALL_FACETEDGES(id)
   { struct facetedge *ep = feptr(id);
     ep->fe_edge_id = copy_sign(elptr(ep->fe_edge_id)->self_id,ep->fe_edge_id);
-    ep->fe_facet_id = copy_sign(elptr(ep->fe_facet_id)->self_id,ep->fe_facet_id);
+    ep->fe_facet_id = valid_id(ep->fe_facet_id) ? 
+		copy_sign(elptr(ep->fe_facet_id)->self_id,ep->fe_facet_id) : ep->fe_facet_id;
     ep->nextedge[0] = copy_sign(elptr(ep->nextedge[0])->self_id,ep->nextedge[0]);
     ep->nextedge[1] = copy_sign(elptr(ep->nextedge[1])->self_id,ep->nextedge[1]);
     ep->nextfacet[0] = copy_sign(elptr(ep->nextfacet[0])->self_id,ep->nextfacet[0]);
@@ -1108,8 +1245,10 @@ void renumber_all()
   { struct edge *ep = eptr(id);
     vertex_id *vp;
     ep->fe_id = copy_sign(elptr(ep->fe_id)->self_id,ep->fe_id);
-    ep->next_vedge[0] = copy_sign(elptr(ep->next_vedge[0])->self_id,ep->next_vedge[0]);
-    ep->next_vedge[1] = copy_sign(elptr(ep->next_vedge[1])->self_id,ep->next_vedge[1]);
+    ep->next_vedge[0] = copy_sign(elptr(ep->next_vedge[0])->self_id,
+      ep->next_vedge[0]);
+    ep->next_vedge[1] = copy_sign(elptr(ep->next_vedge[1])->self_id,
+      ep->next_vedge[1]);
     vp = (vertex_id*)((char*)ep + off);
     for ( k = 0 ; k < dim ; k++ )
       vp[k] = vptr(vp[k])->self_id;
@@ -1147,42 +1286,45 @@ void renumber_all()
 
   FOR_ALL_BODIES(id)    
   { struct body * ep = bptr(id);
-     ep->fe_id = copy_sign(elptr(ep->fe_id)->self_id,ep->fe_id);
+    ep->fe_id = copy_sign(elptr(ep->fe_id)->self_id,ep->fe_id);
   }
 
   /* now new pointers in newibase */
   for ( type = VERTEX ; type <= FACETEDGE ; type++ )
-  {  int count = 0;
-      if ( web.skel[type].count == 0 ) continue; 
-      FOR_ALL_ELEMENTS(type,id)
-          newibase[type][count++] = elptr(id);
+  { int count = 0;
+    if ( web.skel[type].count == 0 ) continue; 
+    FOR_ALL_ELEMENTS(type,id)
+      newibase[type][count++] = elptr(id);
   } 
 
 
   /* fix up links */
   for ( type = VERTEX ; type <= FACETEDGE ; type++ )
   { if ( web.skel[type].count == 0 ) continue;
-     if ( web.skel[type].count == 1 )
-     { newibase[type][0]->forechain = NULLID;
-        newibase[type][0]->backchain = NULLID;
-        continue;
-     }
-     newibase[type][0]->forechain = newibase[type][0]->self_id+1;
-     newibase[type][0]->backchain = NULLID;
-     for ( k = 1 ; k < web.skel[type].count-1; k++ )
-     {
-        newibase[type][k]->forechain = newibase[type][k]->self_id+1;
-        newibase[type][k]->backchain = newibase[type][k]->self_id-1;
-     }
-     newibase[type][k]->forechain = NULLID;
-     newibase[type][k]->backchain = newibase[type][k]->self_id - 1;
-     web.skel[type].used = newibase[type][0]->self_id;
-     web.skel[type].last = newibase[type][k]->self_id;
+    if ( web.skel[type].count == 1 )
+    { newibase[type][0]->forechain = NULLID;
+      newibase[type][0]->backchain = NULLID;
+      continue;
+    }
+    newibase[type][0]->forechain = newibase[type][0]->self_id+1;
+    newibase[type][0]->backchain = NULLID;
+    for ( k = 1 ; k < web.skel[type].count-1; k++ )
+    {
+      newibase[type][k]->forechain = newibase[type][k]->self_id+1;
+      newibase[type][k]->backchain = newibase[type][k]->self_id-1;
+    }
+    newibase[type][k]->forechain = NULLID;
+    newibase[type][k]->backchain = newibase[type][k]->self_id - 1;
+    web.skel[type].used = newibase[type][0]->self_id;
+    web.skel[type].last = newibase[type][k]->self_id;
   }
 
+  #ifndef HASH_ID
   /* fix up freelists */
   for ( type = VERTEX ; type <= FACETEDGE ; type++ )
   { int count;
+    element_id backid = NULLID,newid;
+
     count = web.skel[type].count;
     id = web.skel[type].free;
     if ( !valid_id(id) ) continue;
@@ -1193,12 +1335,16 @@ void renumber_all()
     { ep = elptr(id);
       id = ep->forechain;
       ep->forechain = ++newid;
+      ep->backchain = backid;
+      backid = id;
       newibase[type][count] = ep;
       ep->self_id = (ep->self_id & (~OFFSETMASK)) | count;
       count++;
     } 
     ep->forechain = NULLID;
+    web.skel[type].freelast = backid;
   }
+  #endif
 
   /* miscellaneous elements */
   if ( valid_id(web.zoom_v) ) web.zoom_v = vptr(web.zoom_v)->self_id;
@@ -1350,6 +1496,16 @@ int inx;
   return 0.0;  /* shouldn't get here. */
 }
 
+/***********************************************************************/
+/***********************************************************************
+
+ Routines for locating element structures through hash of element ID.
+ Meant for handling sparse distributions of IDs, for example in large
+ parallel distributed surfaces.  ID is hashed on machine, element type,
+ and ordinal; not on valid bit or orientation.  Unified list for all
+ elements.
+*************************************************************************/
+
 /************************************************************************
 *
 * Function: int32hash()
@@ -1398,8 +1554,322 @@ unsigned long long key;
 }
 
 
+/*************************************************************************
+*
+* Function: elhash_lookup()
+*
+* Purpose: Retrieve element structure pointer from hash table
+*
+*/
+
+struct element * elhash_lookup(id)
+element_id id;
+{ element_id key;
+  int inx;
+
+  key = id_hash(id);
+  inx = ((int)key) & web.elhashmask;
+  for (;;)
+  { if ( web.elhashtable[inx] == NULL )
+      return NULL;
+    if ( web.elhashtable[inx] != ELHASH_FREE )
+      if ( equal_element(web.elhashtable[inx]->self_id,id) )
+        return web.elhashtable[inx];
+    inx++;
+    inx &= web.elhashmask; /* wraparound, maybe */
+  }
+
+}
+
+/*************************************************************************
+*
+* Function: elhash_bigger()
+*
+* Purpose: Double size of element hash table.
+*
+*/
+
+void elhash_bigger()
+{ int oldsize = web.elhashsize;
+  struct element ** oldtable;
+  int n;
+
+  if ( web.elhashsize == 0 )
+  { web.elhashsize = 4096;
+    web.elhashmask = 0xFFF;
+  }
+  else
+  {
+    web.elhashmask |= web.elhashsize; /* depending on power of 2 */
+    web.elhashsize *= 2;
+  }
+
+  oldtable = web.elhashtable;
+  web.elhashtable = (struct element **)mycalloc(web.elhashsize,sizeof(struct element*));
+
+  /* rehash */
+  web.elhashcount = 0;
+  for ( n = 0 ; n < oldsize ; n++ )
+    if ( oldtable[n] && (oldtable[n] != ELHASH_FREE) )
+      elhash_insert(oldtable[n]->self_id,oldtable[n]);
+}
+
+
+/***********************************************************************
+*
+*  function: elhash_extend()
+*
+*  purpose: allocate more empty element structures using pointer freelist
+*/
+
+void elhash_extend(type,mode)
+int type;
+int mode; /* EXTEND_BATCH or EXTEND_FOR_REFINE */
+{
+  int number=0;  /* of new items to allocate */
+  char *newblock;
+  INDIRECT_TYPE *newiblock;
+  struct element *newptr = NULL, *oldptr;
+  int k;
+  int allocsize;
+  long  oldnum = web.skel[type].maxcount;
+  long  newnum=0;
+
+  ENTER_GRAPH_MUTEX;
+  /* Don't mess with structures while graph thread using them */
+
+  if ( blockcount[type] >= blockmax[type] )
+  { blocklist[type] = (struct blocklist_struct*)
+      kb_realloc((char*)(blocklist[type]),
+         (blockcount[type]+BATCHSIZE)*sizeof(struct blocklist_struct));
+     blockmax[type] += BATCHSIZE;
+  }
+  if ( mode == EXTEND_BATCH )
+  {
+    /* calculate number of structures to fit in block size just under 2^n */
+    allocsize = BATCHSIZE*web.sizes[type];
+    k = 0x100 ; while ( k < allocsize ) k <<= 1 ;
+    number = (k-16)/web.sizes[type]; /* maybe room for block header */
+    newnum = web.skel[type].maxcount + number;
+  }
+  else if ( mode == EXTEND_FOR_REFINE )
+  { /* increase by 2^surface_dimension factor */
+    number = web.skel[type].count*(1<<web.dimension)
+                - web.skel[type].maxcount + 100;
+    newnum = web.skel[type].maxcount + number;
+    if ( number <= 0 ) { goto elhash_extend_exit; }/* don't need any more */
+  }
+  else kb_error(2476,"Internal error: bad mode for elhash_extend().\n",
+    RECOVERABLE);
+  
+  if ( (web.skel[type].maxcount + number) > OFFSETMASK )
+  { sprintf(errmsg, "Trying to allocate more %s than ID format allows, %Ld\n",
+          typenames[type],OFFSETMASK);
+    kb_error(3712,errmsg,RECOVERABLE);
+  }
+
+  newblock = mycalloc(number,web.sizes[type]);
+  blocklist[type][blockcount[type]].start_ord = oldnum ;
+  blocklist[type][blockcount[type]].count = number ;
+  blocklist[type][blockcount[type]++].blockptr = (struct element *)newblock;
+
+  while ( newnum > web.skel[type].ialloc )
+  {
+    if ( web.skel[type].ibase == NULL )
+    { newiblock = (INDIRECT_TYPE*)mycalloc(number,sizeof(INDIRECT_TYPE));
+      web.skel[type].ialloc = number;
+    }
+    else
+    { newiblock = (INDIRECT_TYPE*)kb_realloc((char*)(web.skel[type].ibase),
+        2*web.skel[type].ialloc*sizeof(INDIRECT_TYPE));
+      web.skel[type].ialloc *= 2;
+    }
+    web.skel[type].ibase = newiblock;
+    switch(type)
+    {    case VERTEX: vibase = newiblock; break;
+         case EDGE:    eibase = newiblock; break;
+         case FACET:  fibase = newiblock; break;
+         case BODY:    bibase = newiblock; break;
+         case FACETEDGE: feibase = newiblock; break;
+    }
+  }
+
+  /* add to freelist */
+  oldptr = (struct element *)newblock;
+  for ( k = 0 ; k < number ; k++ )
+  { newptr = (struct element *)(newblock + k*web.sizes[type]); 
+    *(struct element**)&(oldptr->forechain) = newptr;
+    oldptr = newptr;
+  }
+  *(struct element**)&(oldptr->forechain) = web.skel[type].freehead;
+  web.skel[type].freehead = (struct element *)newblock;
+  
+elhash_extend_exit:
+  LEAVE_GRAPH_MUTEX;
+  
+
+}
+
+/*************************************************************************
+*
+* Function: elhash_new_element()
+*
+* Purpose: Allocate new element from freelist.  If desired_id is non-null,
+*          it will allocate that id.
+*/
+
+struct element *elhash_new_element(type,desired_id)
+int type; /* of element */
+element_id desired_id;
+{ struct element *newptr,*oldptr;
+  element_id newid;
+  int spot;
+
+  if ( web.skel[type].freehead == NULL )
+    /* allocate another batch */
+    elhash_extend(type,EXTEND_BATCH);
+  newptr = web.skel[type].freehead;
+  web.skel[type].freehead = *(struct element**)&(newptr->forechain);
+
+  if ( web.skel[type].maxcount == 0 )
+    web.skel[type].maxcount = 1024;
+  if ( web.skel[type].alloc > 0.8*web.skel[type].maxcount )
+    web.skel[type].maxcount *= 2;
+    
+  /* find next available element number, using persistant scan */
+  if ( desired_id ) 
+  { spot = (int)(desired_id & OFFSETMASK);
+    newid = ((element_id)type << TYPESHIFT) | VALIDMASK | spot;
+    oldptr = elhash_lookup(newid); 
+    if ( oldptr )
+      kb_error(5433,"Duplicate element number.\n",RECOVERABLE);
+    elhash_insert(newid,newptr);
+  }
+  else 
+  {  spot = web.skel[type].free_spot;
+    do 
+    {
+      newid = ((element_id)type << TYPESHIFT) | VALIDMASK | spot;
+      oldptr = elhash_lookup(newid); 
+      spot++;
+      if ( spot >= web.skel[type].maxcount )
+        spot = 0; 
+    } while (oldptr != NULL);
+    elhash_insert(newid,newptr);
+    web.skel[type].free_spot = spot;
+  }
+  newptr->self_id = newid;
+  web.skel[type].alloc++;
+ 
+  return newptr;
+} 
+
+/*************************************************************************
+*
+* Function: elhash_insert()
+*
+* Purpose: Add one id to element hash table.  Not an error if already
+*          there.
+*/
+
+void elhash_insert(id,eptr)
+element_id id;
+struct element * eptr;
+{ element_id key;
+  int inx;
+  
+  if ( web.elhashcount >= web.elhashsize/2 )
+    elhash_bigger();
+
+  key = id_hash(id);
+  inx = ((int)key) & web.elhashmask;
+  for (;;)
+  { if ( (web.elhashtable[inx] == NULL)
+      || (web.elhashtable[inx] == ELHASH_FREE) )
+    { web.elhashtable[inx] = eptr;
+      web.elhashcount++;
+      return;
+    }
+    if ( equal_element(web.elhashtable[inx]->self_id,id))
+    { web.elhashtable[inx] = eptr;  /* maybe change due to expand() */
+      return;
+    }
+    inx++;
+    inx &= web.elhashmask; /* wraparound, maybe */
+  }
+   
+}
+/*************************************************************************
+*
+* Function: elhash_insert()
+*
+* Purpose: Replace pointer for one id to element hash table.  
+*          NOTE: Old pointer must still be valid!
+*/
+
+void elhash_replace(id,eptr)
+element_id id;
+struct element * eptr;
+{ element_id key;
+  int inx;
+  
+  key = id_hash(id);
+  inx = ((int)key) & web.elhashmask;
+  for (;;)
+  { if ( web.elhashtable[inx] == NULL)
+      kb_error(4382,"INTERNAL ERROR - elhash_replace(): entry not found.\n", RECOVERABLE);
+    if ( web.elhashtable[inx] != ELHASH_FREE )
+      if ( equal_element(web.elhashtable[inx]->self_id,id))
+      { web.elhashtable[inx] = eptr;  /* maybe change due to expand() */
+        return;
+      }
+    inx++;
+    inx &= web.elhashmask; /* wraparound, maybe */
+  }
+   
+}
+
+/*************************************************************************
+*
+* Function: elhash_delete()
+*
+* Purpose: Delete an entry from the element hash table.
+*/
+
+void elhash_delete(id)
+element_id id;
+{ element_id key;
+  int inx;
+
+  /* First, find it */
+  key = id_hash(id);
+  inx = ((int)key) & web.elhashmask;
+  for (;;)
+  { if ( web.elhashtable[inx] == NULL )
+      return; /* wasn't there */
+    if ( (web.elhashtable[inx] != ELHASH_FREE) &&
+            (equal_element(web.elhashtable[inx]->self_id,id)) )
+      break;
+    inx++;
+    inx &= web.elhashmask; /* wraparound, maybe */
+  }
+
+  /* mark as freed, not empty, so searches will continue past */
+  web.elhashtable[inx] = ELHASH_FREE;
+
+  web.elhashcount--;
+}
+
+/* end element id hashing ********************************************/
 
 #ifdef MPI_EVOLVER
+/*************************************************************************
+*
+* Function: mpi_remote_present()
+*
+* purpose: test if a particular remote element id exists locally.
+*/
+
 int mpi_remote_present(id)
 element_id id;
 { struct element *eptr = mpi_remote_elptr(id);
