@@ -70,6 +70,9 @@ typedef struct xelement_id {
 typedef unsigned long long int
       element_id, vertex_id, edge_id, facet_id, body_id, facetedge_id; 
 
+/* appropriate ID hash function */
+#define id_hash(id) int64hash((id)&~(VALIDMASK|SIGNMASK))
+
 #else
 /* masks for fields */
 #define TYPEMASK    0xE0000000
@@ -88,6 +91,9 @@ typedef unsigned long long int
 typedef unsigned int
       element_id, vertex_id, edge_id, facet_id, body_id, facetedge_id; 
 
+/* appropriate ID hash function */
+#define id_hash(id) int32hash((id)&~(VALIDMASK|SIGNMASK))
+
 #endif
 
 #define NULLID 0L 
@@ -107,7 +113,21 @@ typedef unsigned int
 /* machine number of id, for MPI */
 #define id_machine(id) (int)(((id)&MACHINEMASK) >> MACH_ID_SHIFT)
 
+/* for denoting freed spot in hash table */
+#define ELHASH_FREE ((struct element *)1)
+
 /* Macros for getting structure pointer from id */
+#ifdef HASH_ID
+#define elptr(id)    ((struct element *)(elhash_lookup(id)))
+#define vptr(v_id)   ((struct vertex *)(elhash_lookup(v_id)))
+#define eptr(e_id)   ((struct edge   *)(elhash_lookup(e_id)))
+#define fptr(f_id)   ((struct facet  *)(elhash_lookup(f_id)))
+#define bptr(b_id)   ((struct body   *)(elhash_lookup(b_id)))
+#define feptr(fe_id) ((struct facetedge *)(elhash_lookup(fe_id)))
+
+
+#else
+/* lookup in ibase pointer list for each element */
 #ifdef MPI_EVOLVER
 #define elptr(id)   ((struct element *)((id_machine(id)==this_machine?web.skel[id_type(id)].ibase[(id)&OFFSETMASK]:mpi_remote_elptr(id))))
 #define vptr(v_id)   ((struct vertex *)((id_machine(v_id)==this_machine?web.skel[VERTEX].ibase[(v_id)&OFFSETMASK]:mpi_remote_elptr(v_id))))
@@ -124,6 +144,9 @@ typedef unsigned int
 #define feptr(fe_id) ((struct facetedge *)(web.skel[FACETEDGE].ibase[(fe_id)&OFFSETMASK]))
 #endif
 
+/* end HASH_ID */
+#endif
+
 #define ordinal(id)  (valid_id(id) ? (int)((id) & OFFSETMASK) : -1 )
 
 #ifdef MPI_EVOLVER
@@ -138,47 +161,47 @@ typedef unsigned int
 /* ELNAME is unsigned, SELNAME is signed */
 extern char elnames[10][30];
 #ifdef MPI_EVOLVER
-#define ELNAME(id) (valid_id(id) ? (sprintf(elnames[0],"%d@%d",((id)&OFFSETMASK)+1,id_machine(id)),elnames[0]) : "")
-#define ELNAME1(id) (valid_id(id) ? (sprintf(elnames[1],"%d@%d",((id)&OFFSETMASK)+1,id_machine(id)),elnames[1]) : "")
-#define ELNAME2(id) (valid_id(id) ? (sprintf(elnames[2],"%d@%d",((id)&OFFSETMASK)+1,id_machine(id)),elnames[2]) : "")
-#define ELNAME3(id) (valid_id(id) ? (sprintf(elnames[3],"%d@%d",((id)&OFFSETMASK)+1,id_machine(id)),elnames[3]) : "")
-#define ELNAME4(id) (valid_id(id) ? (sprintf(elnames[4],"%d@%d",((id)&OFFSETMASK)+1,id_machine(id)),elnames[4]) : "")
-#define ELNAME5(id) (valid_id(id) ? (sprintf(elnames[5],"%d@%d",((id)&OFFSETMASK)+1,id_machine(id)),elnames[5]) : "")
-#define ELNAME6(id) (valid_id(id) ? (sprintf(elnames[6],"%d@%d",((id)&OFFSETMASK)+1,id_machine(id)),elnames[6]) : "")
-#define ELNAME7(id) (valid_id(id) ? (sprintf(elnames[7],"%d@%d",((id)&OFFSETMASK)+1,id_machine(id)),elnames[7]) : "")
-#define ELNAME8(id) (valid_id(id) ? (sprintf(elnames[8],"%d@%d",((id)&OFFSETMASK)+1,id_machine(id)),elnames[8]) : "")
-#define ELNAME9(id) (valid_id(id) ? (sprintf(elnames[9],"%d@%d",((id)&OFFSETMASK)+1,id_machine(id)),elnames[9]) : "")
-#define SELNAME(id) (valid_id(id) ? (sprintf(elnames[0],"%s%d@%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1,id_machine(id)),elnames[0]) : "")
-#define SELNAME1(id) (valid_id(id) ? (sprintf(elnames[1],"%s%d@%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1,id_machine(id)),elnames[1]) : "")
-#define SELNAME2(id) (valid_id(id) ? (sprintf(elnames[2],"%s%d@%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1,id_machine(id)),elnames[2]) : "")
-#define SELNAME3(id) (valid_id(id) ? (sprintf(elnames[3],"%s%d@%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1,id_machine(id)),elnames[3]) : "")
-#define SELNAME4(id) (valid_id(id) ? (sprintf(elnames[4],"%s%d@%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1,id_machine(id)),elnames[4]) : "")
-#define SELNAME5(id) (valid_id(id) ? (sprintf(elnames[5],"%s%d@%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1,id_machine(id)),elnames[5]) : "")
-#define SELNAME6(id) (valid_id(id) ? (sprintf(elnames[6],"%s%d@%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1,id_machine(id)),elnames[6]) : "")
-#define SELNAME7(id) (valid_id(id) ? (sprintf(elnames[7],"%s%d@%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1,id_machine(id)),elnames[7]) : "")
-#define SELNAME8(id) (valid_id(id) ? (sprintf(elnames[8],"%s%d@%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1,id_machine(id)),elnames[8]) : "")
-#define SELNAME9(id) (valid_id(id) ? (sprintf(elnames[9],"%s%d@%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1,id_machine(id)),elnames[9]) : "")
+#define ELNAME(id) (valid_id(id) ? (sprintf(elnames[0],"%d@%d",(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[0]) : "")
+#define ELNAME1(id) (valid_id(id) ? (sprintf(elnames[1],"%d@%d",(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[1]) : "")
+#define ELNAME2(id) (valid_id(id) ? (sprintf(elnames[2],"%d@%d",(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[2]) : "")
+#define ELNAME3(id) (valid_id(id) ? (sprintf(elnames[3],"%d@%d",(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[3]) : "")
+#define ELNAME4(id) (valid_id(id) ? (sprintf(elnames[4],"%d@%d",(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[4]) : "")
+#define ELNAME5(id) (valid_id(id) ? (sprintf(elnames[5],"%d@%d",(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[5]) : "")
+#define ELNAME6(id) (valid_id(id) ? (sprintf(elnames[6],"%d@%d",(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[6]) : "")
+#define ELNAME7(id) (valid_id(id) ? (sprintf(elnames[7],"%d@%d",(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[7]) : "")
+#define ELNAME8(id) (valid_id(id) ? (sprintf(elnames[8],"%d@%d",(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[8]) : "")
+#define ELNAME9(id) (valid_id(id) ? (sprintf(elnames[9],"%d@%d",(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[9]) : "")
+#define SELNAME(id) (valid_id(id) ? (sprintf(elnames[0],"%s%d@%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[0]) : "")
+#define SELNAME1(id) (valid_id(id) ? (sprintf(elnames[1],"%s%d@%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[1]) : "")
+#define SELNAME2(id) (valid_id(id) ? (sprintf(elnames[2],"%s%d@%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[2]) : "")
+#define SELNAME3(id) (valid_id(id) ? (sprintf(elnames[3],"%s%d@%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[3]) : "")
+#define SELNAME4(id) (valid_id(id) ? (sprintf(elnames[4],"%s%d@%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[4]) : "")
+#define SELNAME5(id) (valid_id(id) ? (sprintf(elnames[5],"%s%d@%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[5]) : "")
+#define SELNAME6(id) (valid_id(id) ? (sprintf(elnames[6],"%s%d@%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[6]) : "")
+#define SELNAME7(id) (valid_id(id) ? (sprintf(elnames[7],"%s%d@%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[7]) : "")
+#define SELNAME8(id) (valid_id(id) ? (sprintf(elnames[8],"%s%d@%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[8]) : "")
+#define SELNAME9(id) (valid_id(id) ? (sprintf(elnames[9],"%s%d@%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1,id_machine(id)),elnames[9]) : "")
 #else
-#define ELNAME(id) (valid_id(id) ? (sprintf(elnames[0],"%d",((id)&OFFSETMASK)+1),elnames[0]) : "")
-#define ELNAME1(id) (valid_id(id) ? (sprintf(elnames[1],"%d",((id)&OFFSETMASK)+1),elnames[1]) : "")
-#define ELNAME2(id) (valid_id(id) ? (sprintf(elnames[2],"%d",((id)&OFFSETMASK)+1),elnames[2]) : "")
-#define ELNAME3(id) (valid_id(id) ? (sprintf(elnames[3],"%d",((id)&OFFSETMASK)+1),elnames[3]) : "")
-#define ELNAME4(id) (valid_id(id) ? (sprintf(elnames[4],"%d",((id)&OFFSETMASK)+1),elnames[4]) : "")
-#define ELNAME5(id) (valid_id(id) ? (sprintf(elnames[5],"%d",((id)&OFFSETMASK)+1),elnames[5]) : "")
-#define ELNAME6(id) (valid_id(id) ? (sprintf(elnames[6],"%d",((id)&OFFSETMASK)+1),elnames[6]) : "")
-#define ELNAME7(id) (valid_id(id) ? (sprintf(elnames[7],"%d",((id)&OFFSETMASK)+1),elnames[7]) : "")
-#define ELNAME8(id) (valid_id(id) ? (sprintf(elnames[8],"%d",((id)&OFFSETMASK)+1),elnames[8]) : "")
-#define ELNAME9(id) (valid_id(id) ? (sprintf(elnames[9],"%d",((id)&OFFSETMASK)+1),elnames[9]) : "")
-#define SELNAME(id) (valid_id(id) ? (sprintf(elnames[0],"%s%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1),elnames[0]) : "")
-#define SELNAME1(id) (valid_id(id) ? (sprintf(elnames[1],"%s%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1),elnames[1]) : "")
-#define SELNAME2(id) (valid_id(id) ? (sprintf(elnames[2],"%s%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1),elnames[2]) : "")
-#define SELNAME3(id) (valid_id(id) ? (sprintf(elnames[3],"%s%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1),elnames[3]) : "")
-#define SELNAME4(id) (valid_id(id) ? (sprintf(elnames[4],"%s%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1),elnames[4]) : "")
-#define SELNAME5(id) (valid_id(id) ? (sprintf(elnames[5],"%s%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1),elnames[5]) : "")
-#define SELNAME6(id) (valid_id(id) ? (sprintf(elnames[6],"%s%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1),elnames[6]) : "")
-#define SELNAME7(id) (valid_id(id) ? (sprintf(elnames[7],"%s%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1),elnames[7]) : "")
-#define SELNAME8(id) (valid_id(id) ? (sprintf(elnames[8],"%s%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1),elnames[8]) : "")
-#define SELNAME9(id) (valid_id(id) ? (sprintf(elnames[9],"%s%d",(inverted(id)?"-":""),((id)&OFFSETMASK)+1),elnames[9]) : "")
+#define ELNAME(id) (valid_id(id) ? (sprintf(elnames[0],"%d",(int)((id)&OFFSETMASK)+1),elnames[0]) : "")
+#define ELNAME1(id) (valid_id(id) ? (sprintf(elnames[1],"%d",(int)((id)&OFFSETMASK)+1),elnames[1]) : "")
+#define ELNAME2(id) (valid_id(id) ? (sprintf(elnames[2],"%d",(int)((id)&OFFSETMASK)+1),elnames[2]) : "")
+#define ELNAME3(id) (valid_id(id) ? (sprintf(elnames[3],"%d",(int)((id)&OFFSETMASK)+1),elnames[3]) : "")
+#define ELNAME4(id) (valid_id(id) ? (sprintf(elnames[4],"%d",(int)((id)&OFFSETMASK)+1),elnames[4]) : "")
+#define ELNAME5(id) (valid_id(id) ? (sprintf(elnames[5],"%d",(int)((id)&OFFSETMASK)+1),elnames[5]) : "")
+#define ELNAME6(id) (valid_id(id) ? (sprintf(elnames[6],"%d",(int)((id)&OFFSETMASK)+1),elnames[6]) : "")
+#define ELNAME7(id) (valid_id(id) ? (sprintf(elnames[7],"%d",(int)((id)&OFFSETMASK)+1),elnames[7]) : "")
+#define ELNAME8(id) (valid_id(id) ? (sprintf(elnames[8],"%d",(int)((id)&OFFSETMASK)+1),elnames[8]) : "")
+#define ELNAME9(id) (valid_id(id) ? (sprintf(elnames[9],"%d",(int)((id)&OFFSETMASK)+1),elnames[9]) : "")
+#define SELNAME(id) (valid_id(id) ? (sprintf(elnames[0],"%s%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1),elnames[0]) : "")
+#define SELNAME1(id) (valid_id(id) ? (sprintf(elnames[1],"%s%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1),elnames[1]) : "")
+#define SELNAME2(id) (valid_id(id) ? (sprintf(elnames[2],"%s%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1),elnames[2]) : "")
+#define SELNAME3(id) (valid_id(id) ? (sprintf(elnames[3],"%s%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1),elnames[3]) : "")
+#define SELNAME4(id) (valid_id(id) ? (sprintf(elnames[4],"%s%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1),elnames[4]) : "")
+#define SELNAME5(id) (valid_id(id) ? (sprintf(elnames[5],"%s%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1),elnames[5]) : "")
+#define SELNAME6(id) (valid_id(id) ? (sprintf(elnames[6],"%s%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1),elnames[6]) : "")
+#define SELNAME7(id) (valid_id(id) ? (sprintf(elnames[7],"%s%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1),elnames[7]) : "")
+#define SELNAME8(id) (valid_id(id) ? (sprintf(elnames[8],"%s%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1),elnames[8]) : "")
+#define SELNAME9(id) (valid_id(id) ? (sprintf(elnames[9],"%s%d",(inverted(id)?"-":""),(int)((id)&OFFSETMASK)+1),elnames[9]) : "")
 #endif
 
 /* Macros for manipulating and testing element ids */
@@ -188,8 +211,8 @@ extern char elnames[10][30];
 #define invert(id)         ((id) ^= SIGNMASK)
 #define equal_id(a,b)      ((a)==(b))
 #define equal_element(a,b) (((a)|SIGNMASK) == ((b)|SIGNMASK))
-#define valid_id(id)       ((id)&VALIDMASK)
-#define inverted(id)       ((id)&SIGNMASK)
+#define valid_id(id)       ((int)(((id)&VALIDMASK) >> VALIDSHIFT))
+#define inverted(id)       ((int)(((id)&SIGNMASK) >> SIGNSHIFT))
 #define inverse_id(id)     ((id) ^ SIGNMASK)
 #define positive_id(id)    ((id) & ~SIGNMASK)
 typedef int ORDTYPE;       /* element numbering type */
