@@ -342,6 +342,7 @@ char *name;
   slot = web.global_count;
   iid = slot | EPHGLOBAL;
   g = globals(iid);
+  memset((char*)g,0,sizeof(struct global));
   strncpy(g->name,name,GLOBAL_NAME_SIZE);
   g->flags |= GLOB_USED;
   
@@ -546,7 +547,6 @@ int perm_rewind_spot; /* where to rewind to */
 
 struct locallist_t *local_scope_bases[LOCALSCOPEMAX]; /* for reference during parsing */
 int local_nest_depth;
-struct locallist_t *query_locals; /* for unnamed commands */
 struct locallist_t *localbase;
 
 /**************************************************************************
@@ -560,34 +560,12 @@ void init_local_scope(iid)
 ident_t iid;  /* identifier of procedure to begin new scope */
 {
   struct locallist_t *locals;
-  struct global *proc=NULL;  
     
-  if ( iid )
-  { proc = globals(iid);    
-    locals = proc->attr.procstuff.locals; 
-  } 
-  else 
-  { locals = query_locals;
-    /*locals->flags |= LL_PERMANENT; *//* potentially execute a permanent command */
-  }
-    
-  if ( locals && valid_id(iid) )
-  { /* free old stuff */
-    if ( locals->list ) 
-    { if ( locals->flags & LL_PERMANENT ) free((char*)locals->list);
-      else myfree((char*)locals->list);
-    }
-    memset(locals,0,sizeof(struct locallist_t));
-  }
-  else if ( locals == NULL )
-  { locals = (iid & PERMGLOBAL ) ?
+  locals = (iid & PERMGLOBAL ) ?
        (struct locallist_t *)calloc(1,sizeof(struct locallist_t)) :
        (struct locallist_t *)mycalloc(1,sizeof(struct locallist_t)) ;
-    if ( iid )   
-      proc->attr.procstuff.locals = locals;
-    if ( iid & PERMGLOBAL ) 
+  if ( iid & PERMGLOBAL ) 
       locals->flags |= LL_PERMANENT;
-  }
 
   if ( local_nest_depth < 0 ) 
      local_nest_depth = 0;
@@ -603,6 +581,7 @@ ident_t iid;  /* identifier of procedure to begin new scope */
                                  sizeof(struct localvar_t)) ;
   localbase = locals;
   locals->list[0].prev = -1;
+
 }
 
 /**************************************************************************
@@ -610,13 +589,26 @@ ident_t iid;  /* identifier of procedure to begin new scope */
 * function: exit_local_scope()
 *
 * purpose: finish procedure list local definitions.
+* return: sets global variable localbase to the completed 
+*                  locals structure, for parser to attach
+*                  to code block.
 */
 
 void exit_local_scope()
 {
+  struct locallist_t *locals;
+  
   local_nest_depth--;
-  if ( local_nest_depth >= 0 )
-  localbase = local_scope_bases[local_nest_depth];
+
+  locals = local_scope_bases[local_nest_depth];
+  if ( locals && !(locals->flags & LL_IN_USE) )
+  { myfree((char*)(locals->list));
+    myfree((char*)locals);
+  }
+  local_scope_bases[local_nest_depth] = NULL;
+  if ( local_nest_depth > 0 )
+    localbase = local_scope_bases[local_nest_depth-1];
+  else localbase = NULL;
 }
 
 /**************************************************************************
@@ -632,8 +624,8 @@ void begin_local_scope()
   if ( local_nest_depth == 0 )
   { /* coming in from unnamed command */
     /*init_local_scope(0); */
-	  kb_error(3501,"INTERNAL ERROR: local_scope not initialized.\n",
-		   RECOVERABLE);
+    kb_error(3501,"INTERNAL ERROR: local_scope not initialized.\n",
+       RECOVERABLE);
   }
     
   locals = local_scope_bases[local_nest_depth-1]; 
@@ -715,7 +707,7 @@ char *name; /* NULL if just reserving space for anonymous value */
     v = locals->list + locals->count;   /* reset dangling pointer */
   }
  
-  /* move counter up */
+  /* move counter up */ 
   v[1].prev = locals->count;
   v[1].scope_depth = v->scope_depth;
   locals->count++;
@@ -751,6 +743,35 @@ char *name;
 
   return 0; /* not found */
 }
+
+/**************************************************************************
+*
+* function: locals_copy()
+*
+* purpose: make a copy of local variable structure when assigning
+*          a code block in eval().
+*/
+void locals_copy(dest,src)
+struct locallist_t **dest;
+struct locallist_t *src;
+{
+  if ( *dest )
+  { /* have old structure */
+    myfree((char*)(*dest)->list);
+  }
+  else
+    *dest = (struct locallist_t *)mycalloc(1,sizeof(struct locallist_t));
+  if ( src )
+  {
+    **dest = *src;
+    (*dest)->list = (struct localvar_t *)mycalloc(src->count,
+                       sizeof(struct localvar_t));
+    memcpy((*dest)->list,src->list,src->count*sizeof(struct localvar_t));
+  }
+  else 
+    memset((char*)(*dest),0,sizeof(struct locallist_t));
+} 
+
 /**************************************************************************
 *
 * function: initialize_perm_globals()
@@ -843,6 +864,19 @@ void initialize_perm_globals()
   g->value.dataptr = &estimated_change;
   g->flags = INTERNAL_NAME|PERMANENT|REALVAL|READONLY;
 
+  k = add_perm_global("eigenvalues");
+  eigenvalues_list_global = k;
+  g = globals(eigenvalues_list_global);
+  g->value.arrayptr = 
+     (struct array*)calloc(sizeof(struct array)+1*sizeof(int),1); 
+  g->value.arrayptr->dim = 1;
+  g->attr.varstuff.dim = 1;
+  g->value.arrayptr->itemsize = sizeof(REAL);
+  g->value.arrayptr->datatype = REAL_TYPE;
+  g->value.arrayptr->size[0] = 0;
+  g->value.arrayptr->datasize = 0;
+  g->flags = INTERNAL_NAME|PERMANENT|ARRAY_PARAM|READONLY;
+
 }
 
 
@@ -940,3 +974,22 @@ void set_view_matrix_global()
         (char*)&view[0][0] - (char*)(g->value.arrayptr);
 }
 
+
+/*************************************************************************
+*
+* function: set_eigenvalue_list_global()
+*
+* purpose: update eigenvalue[] global variable info.
+*/
+void set_eigenvalue_list_global(evalues,count)
+REAL *evalues;
+int count;
+{ struct global *g;
+
+  g = globals(eigenvalues_list_global);
+  g->value.arrayptr->size[0] = count;
+  g->value.arrayptr->datasize = count;
+  g->value.arrayptr->datastart = 
+        (char*)evalues - (char*)(g->value.arrayptr);
+}
+
