@@ -30,12 +30,12 @@ DWORD WINAPI winthread_worker( void * arg );
 CRITICAL_SECTION thread_cs;  /* main thread critical section */
 CRITICAL_SECTION element_cs;  /* per element critical section */
 CRITICAL_SECTION web_cs;  /* web structure critical section */
-void *element_mutex_ptr = (void*)&element_cs; /* for use in other files */
-void *web_mutex_ptr = (void*)&web_cs; /* for use in other files */
+LPCRITICAL_SECTION element_mutex_ptr = &element_cs; /* for use in other files */
+LPCRITICAL_SECTION web_mutex_ptr = &web_cs; /* for use in other files */
 HANDLE workthread_wakeup; /* worker thread wake-up event */
 HANDLE mainthread_wakeup; /* resume main thread event */
 HANDLE barrier_event;  /* for synchronizing worker threads */
-int barrier_count;
+volatile LONG barrier_count;
 HANDLE *threadlist;   /* thread handles */
 DWORD thread_data_key;
 #endif
@@ -125,12 +125,12 @@ void m_set_idlist()
 #define main old_main 
 #endif
  
-int main(argc,argv)  
-int argc;
-char *argv[]; 
+int main ARGS2((argc,argv),
+int argc,
+char *argv[]) 
 { 
-  msgmax = 200; 
-  msg = my_list_calloc(1,msgmax,ETERNAL_BLOCK); 
+  msgmax = 2000; 
+  if ( !msg ) msg = my_list_calloc(1,msgmax,ETERNAL_BLOCK); 
   set_ctypes();
 
   outfd = stdout;  
@@ -154,12 +154,6 @@ char *argv[];
   procs_requested = 1;  /* default at John's request.  m_get_numprocs(); */
 #endif
 
-#ifdef LONG_ID
-id_hash = int64hash;
-#else 
-id_hash = int32hash;
-#endif
-
 #ifdef WIN32
 /* Mutex for use between graph thread and main thread. */
 /* Nameless to prevent interference between separate processes. */
@@ -194,6 +188,7 @@ id_hash = int32hash;
                     break;
           case 'a': auto_convert_flag = (argv[0][2]=='-') ? 0 : 1; break;
           case 'q': option_q = (argv[0][2]=='-') ? 0 : 1; break; 
+          case 'Q': quiet_load_flag = 1; break;
           case 'e': echo_flag = 1; break;
 #ifdef MPI_EVOLVER
           case 'z': mpi_debug = 1; break;
@@ -211,6 +206,7 @@ id_hash = int32hash;
                  yydebug = 1;
                  break;
           case 'i' : match_id_flag = 1; break;
+          case 'I' : sparse_ibase_flag = 1; break;
           case 'p' : procs_requested = atoi(argv[0]+2); 
 #if defined(SGI_MULTI) || defined (THREADS)
                 if ( procs_requested < 1 )
@@ -328,7 +324,7 @@ id_hash = int32hash;
     barrier_count = procs_requested;
     ResetEvent(barrier_event);
     for ( i = 0 ; i < procs_requested ; i++ )
-    { int tid;  /* dummy for thread id */
+    { DWORD tid;  /* dummy for thread id */
       thread_data_ptrs[i] = 
            (struct thread_data *)my_list_calloc(1,sizeof(struct thread_data),
                  ETERNAL_BLOCK);
@@ -364,15 +360,15 @@ id_hash = int32hash;
     pthread_key_create(&thread_data_key,NULL);
     pthread_mutex_init(&thread_mutex, NULL);
     pthread_mutex_init(&element_mutex, NULL);
-    threadlist = (pthread_t *)list_calloc(procs_requested+10,sizeof(int),
+    threadlist = (pthread_t *)my_list_calloc(procs_requested+10,sizeof(int),
               ETERNAL_BLOCK);
-    thread_data_ptrs = (struct thread_data**)list_calloc(procs_requested,
+    thread_data_ptrs = (struct thread_data**)my_list_calloc(procs_requested,
                       sizeof(struct thread_data *),ETERNAL_BLOCK);
     busythreads = procs_requested;  /* countdown barrier */
     pthread_mutex_lock(&thread_mutex); 
     for ( i = 0 ; i < procs_requested ; i++ )
     { int ret;  /* dummy for thread id */
-      thread_data_ptrs[i] = (struct thread_data *)list_calloc(1,
+      thread_data_ptrs[i] = (struct thread_data *)my_list_calloc(1,
                 sizeof(struct thread_data),ETERNAL_BLOCK);
       thread_data_ptrs[i]->worker_id = i;
       ret = pthread_create(threadlist+i,NULL,pthread_worker,
@@ -424,26 +420,60 @@ id_hash = int32hash;
 
   if ( cmdfilename )    /* trap back to here if error and skip cmdfilename */
   { if ( !setjmp(jumpbuf) )    
-          push_commandfd(NULL,cmdfilename);
+      push_commandfd(NULL,cmdfilename);
   }
 
+#ifdef __cplusplus
+  char *to_load = (argc > 0) ? argv[0] : NULL; 
+  for ( ;; )
+  {
+    try
+    {
+      startup(to_load);
+      datafile_flag = 0;
+      /* use C++ exception mechanism instead of setjmp/longjmp */
+      for (;;)
+      {
+        try
+        { 
+          exec_commands(NULL,"Enter command: "); /* command read/execute loop */
+        }
+        catch ( int k )
+        { 
+        }
+      }
+    }
+    catch ( loadexcep k )  /* corresponds to loadjmpbuf */
+    { /* LOAD command returns here */
+      if ( list && (list != permlist)) 
+      { myfree((char*)list); list = NULL; }/* plug memory leak */
+      to_load = loadfilename[0] ? loadfilename : NULL;
+    }
+    catch ( int j )   /* corresponds to jumpbuf */
+    { to_load = NULL;
+    }
+  }
+
+#else
   if ( setjmp(loadjumpbuf) )
   { /* LOAD command returns here */
     if ( list && (list != permlist)) 
     { myfree((char*)list); list = NULL; }/* plug memory leak */
     startup(loadfilename);
+
   }
   else
-  {
-  if ( setjmp(jumpbuf) )    /* return here after datafile errors */
-     { 
+  { if ( setjmp(jumpbuf) )    /* return here after datafile errors */
+    { 
         startup(NULL);
-     }
-  else
-  if ( argc > 0 ) startup(argv[0]);
-  else startup(NULL);
+    }
+    else
+    { if ( argc > 0 ) startup(argv[0]);
+      else startup(NULL);
+    }
   }
   datafile_flag = 0;
+#endif
 
 #ifdef MPI_EVOLVER
   MPI_Barrier(MPI_COMM_WORLD);
@@ -452,13 +482,24 @@ id_hash = int32hash;
   calc_energy();  /* initial energy */
 #endif
 
+#ifdef __cplusplus
+#else
   while ( setjmp(jumpbuf) != 0 );    /* return here after commandfile  errors */
   exec_commands(NULL,"Enter command: ");  /* command read and execute loop */
+#endif
+
   my_exit(0);
 
   return 0; /* success return code */
 }
 
+#ifdef __cplusplus
+void loadstub()
+{ struct loadexcep something;
+  throw something;
+}
+#endif
+
 /********************************************************************
 *
 *  function: my_exit()
@@ -466,8 +507,8 @@ id_hash = int32hash;
 *  purpose: graceful exit from program
 */
 
-void my_exit(code)
-int code;
+void my_exit ARGS1((code),
+int code)
 {
   if ( OOGL_flag ) End_OOGL();
 
@@ -498,9 +539,9 @@ int code;
 *              pops command file stack whenever end of file.
 */
 
-void exec_commands(basefd,promptstring)
-FILE *basefd;  /* stop when get back to this input source */
-char *promptstring;  /* prompt to use */
+void exec_commands ARGS2((basefd,promptstring),
+FILE *basefd,  /* stop when get back to this input source */
+char *promptstring)  /* prompt to use */
 { /* main event loop of program */
   while ( commandfd && (commandfd != basefd))
   {
@@ -521,6 +562,7 @@ char *promptstring;  /* prompt to use */
   }
 }
 
+#ifdef MAC_OS_X
 /********************************************************************
 *
 * function: mac_exec_commands()
@@ -528,11 +570,86 @@ char *promptstring;  /* prompt to use */
 * purpose: wrapper for exec_commands so Mac main thread (which is
 *          the draw thread) can call with one argument.
 */
-void *mac_exec_commands(arg)
-FILE *arg;
-{ exec_commands(arg,"Enter command: ");
+void *mac_exec_commands ARGS1((arg),
+FILE *arg)
+{
+
+#ifdef __cplusplus
+  char *to_load = NULL;
+  int flag; /* so doesn't prompt user for input file first time, if have */
+  flag = !datafilename[0];
+  for ( ;; )
+  {
+    if ( flag )
+       startup(to_load);
+    flag = 1;
+    try
+    {
+      /* use C++ exception mechanism instead of setjmp/longjmp */
+      for (;;)
+      {
+        try
+        { 
+          chdir(curdir);
+          exec_commands(NULL,"Enter command: "); /* command read/execute loop */
+        }
+        catch ( int k )
+        { 
+        }
+      }
+    }
+    catch ( loadexcep k )  /* corresponds to loadjmpbuf */
+    { /* LOAD command returns here */
+      if ( list && (list != permlist)) 
+      { myfree((char*)list); list = NULL; }/* plug memory leak */
+      to_load = loadfilename[0] ? loadfilename : NULL;
+    }
+    catch ( int j )   /* corresponds to jumpbuf */
+    { to_load = NULL;
+    }
+  }
+
+#else
+  if ( setjmp(loadjumpbuf) )
+  { /* LOAD command returns here */
+    if ( list && (list != permlist)) 
+    { myfree((char*)list); list = NULL; }/* plug memory leak */
+    startup(loadfilename);
+
+  }
+  else
+  { if ( setjmp(jumpbuf) )    /* return here after datafile errors */
+    { 
+       exec_commands(NULL,"Enter command: "); /* command read/execute loop */
+       // startup(NULL);
+    }
+    else
+    { if ( do_show_flag == 1 )
+         do_show(); 
+      chdir(curdir);   /* so command thread gets proper directory */
+      exec_commands(NULL,"Enter command: "); /* command read/execute loop */
+     //  startup(NULL);
+    }
+  }
+  datafile_flag = 0;
+#endif
+
+#ifdef MPI_EVOLVER
+  MPI_Barrier(MPI_COMM_WORLD);
+  if ( this_machine > 0 )
+    return 0;   /* return to mpi main() */
+  calc_energy();  /* initial energy */
+#endif
+
+#ifdef __cplusplus
+#else
+  while ( setjmp(jumpbuf) != 0 );    /* return here after commandfile  errors */
+  exec_commands(NULL,"Enter command: ");  /* command read and execute loop */
+#endif
+
   return NULL;
 }
+#endif
 
 /********************************************************************
 *
@@ -543,9 +660,9 @@ FILE *arg;
 *              can be used in the middle of executing a command.
 */
 
-void exec_file(fd,name)
-FILE *fd;  /* file, if already opened, like stdin */
-char *name; /* file name, if not already opened */
+void exec_file ARGS2((fd,name),
+FILE *fd,  /* file, if already opened, like stdin */
+char *name) /* file name, if not already opened */
 { int old_read_depth = read_depth;
   push_commandfd(fd,name);
   do  /* main event loop of program */
@@ -571,13 +688,17 @@ char *name; /* file name, if not already opened */
 *
 *****************************************************************/
 
-void startup(file_name)
-char *file_name;  /* NULL if need to ask for name */
+void startup ARGS1((file_name),
+char *file_name)  /* NULL if need to ask for name */
 {
   char *name = file_name;
   char response[100];
   FILE *newfd;
 
+  /* be sure graphics thread is ok before loading new file */
+  ENTER_GRAPH_MUTEX;
+  LEAVE_GRAPH_MUTEX;
+
   datafile_flag = 0;
   /* close leftover input files */
   for ( ; read_depth > (cmdfilename ? 2 : 1) ; read_depth-- ) 
@@ -622,6 +743,7 @@ printf("Task %d: ",this_machine);
   SetConsoleTitle(msg);
 #endif
 
+  ENTER_GRAPH_MUTEX;
   reset_web(); 
   init_view();
 
@@ -648,6 +770,7 @@ if ( heapcheck() < 0 )
   }
 
   initialize();
+  LEAVE_GRAPH_MUTEX;
 
   datafile_flag = 0;
   if ( read_depth > 0 )
@@ -700,8 +823,8 @@ void thread_stage_setup ARGS((void));
 * purpose: provide common place for various type worker threads to
 *          call the desired task.
 */
-void task_caller(thread_tasknum)
-int thread_tasknum;
+void task_caller ARGS1((thread_tasknum),
+int thread_tasknum)
 { 
   struct thread_data *data = GET_THREAD_DATA;
 __int32 task_caller_elapsed_time[2];
@@ -770,7 +893,7 @@ PROF_PRINT(task_caller)
 *
 */
 
-DWORD WINAPI winthread_worker( void * arg )
+DWORD WINAPI winthread_worker ( void * arg )
 { struct thread_data *data = (struct thread_data *)arg;
 
   /* set per-thread data */
@@ -857,14 +980,14 @@ void *pthread_worker( void * arg )
 *
 * purpose: comparison function for sorting partition coordinates.
 */
-struct pcoord { REAL pcoord; vertex_id v_id; };
+struct pcoord { REAL p_coord; vertex_id v_id; };
 int pcomp ARGS((struct pcoord*,struct pcoord*));
 
-int pcomp(a,b)
-struct pcoord *a,*b;
+int pcomp ARGS2((a,b),
+struct pcoord *a, struct pcoord *b)
 {
-  if ( a->pcoord < b->pcoord ) return -1;
-  if ( a->pcoord > b->pcoord ) return  1;
+  if ( a->p_coord < b->p_coord ) return -1;
+  if ( a->p_coord > b->p_coord ) return  1;
   return 0;
 }
 
@@ -937,7 +1060,7 @@ void thread_stage_setup()
                  sizeof(struct pcoord));
   pcount = 0;
   FOR_ALL_VERTICES(v_id)
-  { pcoords[pcount].pcoord = *((REAL*)(get_extra(v_id,v_partition_coord_attr)));
+  { pcoords[pcount].p_coord = *((REAL*)(get_extra(v_id,v_partition_coord_attr)));
     pcoords[pcount].v_id = v_id;
     pcount++;
   } 
@@ -1153,7 +1276,8 @@ void thread_launch(int task, int element_type)
 
   global_id = web.skel[element_type].used;
   for ( i = 0 ; i < nprocs ; i++ )
-  { while ( valid_id(global_id) && !valid_element(global_id) )
+  { /* make sure we are starting with an actual element */
+    while ( valid_id(global_id) && !valid_element(global_id) )
       global_id = elptr(global_id)->forechain;
  
     thread_data_ptrs[i]->iteration_id = global_id;
@@ -1376,3 +1500,6 @@ void find_cpu_speed()
    cpu_speed = 3e9;  /* rough guess */
 #endif
 }
+
+
+
