@@ -422,12 +422,12 @@ void local_refine()     /* REAL the resolution of triangulation */
       for ( i = 0 ; i < oldctrl ; i++)
       { REAL *vx = get_oldcoord(v[i]);
         for ( k = 0 ; k < SDIM ; k++ ) 
-		  oldx[i][k] = vx[k];
-		if ( get_vattr(v[i]) & BOUNDARY )
-		{ REAL *vp = get_param(v[i]);
-		  for ( k = 0 ; k < web.maxparam ; k++ )
-				oldp[i][k] = vp[k];
-		}
+      oldx[i][k] = vx[k];
+    if ( get_vattr(v[i]) & BOUNDARY )
+    { REAL *vp = get_param(v[i]);
+      for ( k = 0 ; k < web.maxparam ; k++ )
+    oldp[i][k] = vp[k];
+    }
       }
       if ( w2 ) (*sym_wrap)(get_coord(v[N]),oldx[N],w2);
       if ( w3 ) (*sym_wrap)(get_coord(v[oldctrl-1]),oldx[oldctrl-1],w3);
@@ -462,7 +462,7 @@ void local_refine()     /* REAL the resolution of triangulation */
             }
           }
          
-		  /* whole facet boundary parameters */
+          /* whole facet boundary parameters */
           if ( facet_bdry_flag && (get_vattr(allv[j][i]) & Q_MIDFACET) )
             { REAL * vp = get_param(allv[j][i]);
               for ( n = 0 ; n < 2 ; n++ )
@@ -609,15 +609,17 @@ REAL min_area;    /* criterion for weeding out small triangles */
   /* main loop sweeping over all triangles */
   f_id = NULLFACET;
   while ( generate_all(FACET,&f_id,&sentinel) )
-  { REAL side[MAXCOORD];  /* side vector */
+  { REAL side[FACET_EDGES][MAXCOORD];  /* side vector */
     REAL sside[FACET_EDGES]; /* squares of side lengths */
-    REAL temp;      /* temporary variable in area calculation */
     REAL area;      /* area of triangle */
     facetedge_id fe[FACET_EDGES]; /* edges of triangle */
     edge_id e_id;
     int i;              /* side number */
     int elimcount;
-
+#ifdef _DEBUGXX
+    if ( (f_id & 0xFFFFFF) == 0xb3b )
+      break;
+#endif
     /* skip already modified triangles */
     if ( get_fattr(f_id) & NEWFACET )
        continue;
@@ -625,32 +627,48 @@ REAL min_area;    /* criterion for weeding out small triangles */
     /* find sides and area */
     fe[0] = get_facet_fe(f_id);
     for ( i = 0 ; i < FACET_EDGES ; i++ ) 
-    { get_fe_side(fe[i],side);
-      sside[i] = SDIM_dot(side,side);
+    { get_fe_side(fe[i],side[i]);
+      sside[i] = SDIM_dot(side[i],side[i]);
       fe[i+1] = get_next_edge(fe[i]);
     }
+    
+    area = get_facet_area(f_id);
 
-    temp = sside[0] + sside[1] - sside[2];
-    temp = 4*sside[0]*sside[1] - temp*temp;
-    area = (temp > 0.0) ? sqrt(temp)/4 : 0.0;
     if ( area > min_area )  /* skip big triangles */
-      continue;
-
-    /* find longest side */
-/*        i = (sside[0] > sside[1]) ? 0 : 1; */
-/*        i = (sside[i] > sside[2]) ? i : 2; */
-/*        weedcount += edge_refine(get_fe_edge(fe[i])); */
-/*  this doesn't really work too well */
+      continue;  
 
     /* weed by eliminating shortest side. eliminate obtuse vertex,
        the one between the two shortest sides, if possible.
        Follow with equiangulation  */
     i = (sside[0] < sside[1]) ? 0 : 1;
     i = (sside[i] < sside[2]) ? i : 2;
+    /*
     if ( sside[(i+1)%3] < sside[(i+2)%3] )
       e_id = inverse_id(get_fe_edge(fe[i]));
-    else e_id = get_fe_edge(fe[i]);
+    else 
+      e_id = get_fe_edge(fe[i]);
     elimcount = eliminate_edge(e_id);
+    */
+
+    /* copied from eliminate_facet() */
+    elimcount = eliminate_edge(e_id);
+    if ( elimcount ) goto elimdone;
+
+    if ( sside[(i+1)%3] < sside[(i+2)%3] )
+    { e_id = get_fe_edge(fe[(i+1)%3]);
+      elimcount = eliminate_edge(e_id);
+      if ( elimcount ) goto elimdone;
+      e_id = get_fe_edge(fe[(i+2)%3]);
+      elimcount = eliminate_edge(e_id);
+    }
+    else
+    { e_id = get_fe_edge(fe[(i+2)%3]);
+      elimcount = eliminate_edge(e_id);
+      if ( elimcount ) goto elimdone;
+      e_id = get_fe_edge(fe[(i+1)%3]);
+      elimcount = eliminate_edge(e_id);
+    }
+elimdone:
     if ( elimcount )
     { free_element(e_id);
       if ( verbose_flag )
@@ -955,13 +973,12 @@ void skinny_histogram()
   int n; 
  
   for ( n = 0 ; n < HISTO_BINS ; n++ ) bincount[n] = 0;
-  
   /* main loop sweeping over all triangles */ 
   FOR_ALL_FACETS(f_id)
     {
         REAL sside[FACET_EDGES]; /* squares of side lengths */ 
         REAL angle;      /* area of triangle */
-        facetedge_id fe[FACET_EDGES]; /* edges of triangle */
+        facetedge_id fe[FACET_EDGES+1]; /* edges of triangle */
         int i;              /* side number */
         int smallside,mid,big;
 
@@ -987,9 +1004,14 @@ void skinny_histogram()
         else
           n = HISTO_BINS + (int)floor(log(angle/M_PI)*HISTO_BINSIZE); 
           
-        if ( n < 0 ) n = 0; if ( n >= HISTO_BINS ) n = HISTO_BINS - 1;
+        if ( n < 0 ) 
+          n = 0; 
+        else if ( n >= HISTO_BINS ) 
+          n = HISTO_BINS - 1;
+
         bincount[n]++;
      }
+
   outstring("         angle                 number\n");
   if ( bincount[0] )
      {
@@ -1214,13 +1236,17 @@ edge_id short_edge;
       if ( valid_id(get_fe_facet(fe)) )
       { int abcomp = compare_edge_attr(get_fe_edge(fe_a),get_fe_edge(fe_b));
         if ( abcomp == INCOMPARABLE)
-        {
-          if ( verbose_flag )
-          { sprintf(msg,
-           "Can't delete edge %s due to constraints or methods on edges %s and %s.\n",ELNAME(short_edge),ELNAME1(get_fe_edge(fe_a)),ELNAME2(get_fe_edge(fe_b)));
-             outstring(msg);
+        { facet_id f_id = get_fe_facet(fe);
+          int acomp = compare_edge_facet_attr(get_fe_edge(fe_a),f_id);
+          int bcomp = compare_edge_facet_attr(get_fe_edge(fe_b),f_id);
+          if ( !(acomp==A_SUB_B || acomp==A_EQ_B || bcomp==A_SUB_B || bcomp==A_EQ_B) )
+          { if ( verbose_flag )
+            { sprintf(msg,
+             "Can't delete edge %s due to constraints or methods on edges %s and %s.\n",ELNAME(short_edge),ELNAME1(get_fe_edge(fe_a)),ELNAME2(get_fe_edge(fe_b)));
+               outstring(msg);
+            }
+            return 0; 
           }
-          return 0;
         }
       }
       fe = get_next_facet(fe);
@@ -1257,7 +1283,6 @@ edge_id short_edge;
   }
 
 
-  ENTER_GRAPH_MUTEX;
 
   /* Go through facets around edge,  checking for stars (adjacent
      triangulated triangle) and unstarring if found.              */
@@ -1280,7 +1305,7 @@ edge_id short_edge;
       /* see if we have an adjacent starred triangle which will give
           trouble if we don't unstar it */
       bbase_fe = base_fe;
-      for(;;) /* may take multiple passes for unstarring */
+    /*  for(;;) */ /* may take multiple passes for unstarring */
       { int flag;
         int unstar_count = 0;
         /* first, set up to favor deletion of valence 2 edge */
@@ -1301,7 +1326,8 @@ edge_id short_edge;
           bb = get_prev_edge(b_next);
           if ( get_vattr(get_edge_headv(a_edge)) & AXIAL_POINT ) break;
           if ( equal_id(get_next_facet(aa),inverse_id(bb)) )
-          { int ret = unstar(aa); 
+          { int ret;
+            ret  = unstar(aa); 
             if ( ret < 0 )
             { sprintf(errmsg,
             "Edge %s not deleted due to adjacent configuration involving facet %s.\n",
@@ -1318,7 +1344,7 @@ edge_id short_edge;
         if ( unstar_count ) continue;  /* retry */
         if ( flag >= 2 ) 
         {
-          sprintf(errmsg,"Edge amenity failed second time through on edge %s\n",
+          sprintf(errmsg,"Edge unstarring failed second time through on edge %s\n",
               ELNAME(short_edge));
           kb_error(2889,errmsg, WARNING);
           retval = 0;
@@ -1336,6 +1362,8 @@ edge_id short_edge;
     outstring(msg);
   }
   
+ ENTER_GRAPH_MUTEX;
+
  /* unwrap edge */
  if ( web.torus_flag )
    torus_unwrap_edge(short_edge);
@@ -1412,9 +1440,11 @@ edge_id short_edge;
     facetedge_id aa,bb;
     facetedge_id bbase_fe;
     facetedge_id next_fe,prev_fe;
-     
+    vertex_id third_v;
+
     next_fe = get_next_edge(base_fe);
     prev_fe = get_prev_edge(base_fe);
+    third_v = get_fe_headv(next_fe);
     if ( (web.representation == STRING) && 
        ((!valid_id(get_fe_facet(base_fe)) || !valid_id(prev_fe) || 
        (!equal_id(get_next_edge(next_fe), prev_fe)) && !equal_id(prev_fe,next_fe))
@@ -1468,9 +1498,10 @@ edge_id short_edge;
       /* see if we have an adjacent starred triangle which will give
           trouble if we don't unstar it */
       bbase_fe = base_fe;
-      for(;;) /* may take multiple passes for unstarring */
+      /* for(;;) */ /* may take multiple passes for unstarring */
       { int flag;
         int unstar_count = 0;
+        int ktcomp;
         /* first, set up to favor deletion of valence 2 edge */
         if ( get_edge_valence(get_fe_edge(get_prev_edge(bbase_fe))) == 2 )
               bbase_fe = inverse_id(bbase_fe);
@@ -1487,8 +1518,17 @@ edge_id short_edge;
           b_next = get_next_facet(b);
           b_prev = get_prev_facet(b);
           keep_edge = b_edge; throw_edge = a_edge; 
-          if ( compare_edge_attr(keep_edge,throw_edge) == A_SUB_B )
-          { bbase_fe = inverse_id(bbase_fe); continue; }  
+          
+          ktcomp = compare_edge_attr(keep_edge,throw_edge);
+          if ( ktcomp == A_SUB_B )
+          { bbase_fe = inverse_id(bbase_fe); continue; } 
+          if ( ktcomp == INCOMPARABLE )
+          { facet_id f_id = get_fe_facet(bbase_fe);
+            int tcomp = compare_edge_facet_attr(throw_edge,f_id);
+            if ( !(tcomp==A_SUB_B || tcomp==A_EQ_B) )
+            { bbase_fe = inverse_id(bbase_fe); continue; }
+          } 
+          
           if ( equal_element(a_edge,b_edge) ) break;
           aa = get_next_edge(a_next);
           bb = get_prev_edge(b_next);
@@ -1506,7 +1546,7 @@ edge_id short_edge;
             }
             unstar_count = 1;
           }
-          break;
+          break;  
         }
         if ( unstar_count ) continue;  /* retry */
         if ( flag >= 2 ) 
@@ -1515,7 +1555,7 @@ edge_id short_edge;
               ELNAME(short_edge));
           kb_error(2198,errmsg, RECOVERABLE);
         }
-        break;
+        /* break;  since not doing for loop any more */
       }
 
       /* put throw_edge constraints on keep_edge */
@@ -1601,44 +1641,44 @@ edge_id short_edge;
       if ( (web.representation == STRING) && phase_flag )
           set_e_phase_density(keep_edge);
 
-	  /* fix method instances. Oppositely oriented signed methods cancel;
-	     others combined.                                                   */
-	  { struct edge *ea_ptr = eptr(keep_edge);
+      /* fix method instances. Oppositely oriented signed methods cancel;
+         others combined.                                                   */
+      { struct edge *ea_ptr = eptr(keep_edge);
         struct edge *eb_ptr = eptr(throw_edge);
         int meth_offset = get_meth_offset(EDGE);
-		int *ameths = (int*)((char*)ea_ptr+meth_offset);
+        int *ameths = (int*)((char*)ea_ptr+meth_offset);
         int *bmeths = (int*)((char*)eb_ptr+meth_offset);
         int i,j;
         for ( i = 0 ; i < (int)eb_ptr->method_count ; i++ )
         { int m;
-		  int handled = 0;
-		  
-		  int orientable;
+          int handled = 0;
+  
+          int orientable;
           m = abs(bmeths[i]);
-		  orientable = basic_gen_methods[METH_INSTANCE(m)->gen_method].flags &
-					       ORIENTABLE_METHOD;
+          orientable = basic_gen_methods[METH_INSTANCE(m)->gen_method].flags &
+                           ORIENTABLE_METHOD;
           for ( j = 0 ; j < (int)ea_ptr->method_count ; j++ )
           { int n;
             n = abs(ameths[j]);
             if ( n == m ) 
-			{ /* see if orientable and cancellable */
-			  if ( orientable )
-			  { /* drop from keep_edge */
-				ameths[j] = ameths[--ea_ptr->method_count];
-		      }
-			  /* else leave alone */
-			  handled = 1;
-			  break;
-			}
+            { /* see if orientable and cancellable */
+              if ( orientable )
+              { /* drop from keep_edge */
+                ameths[j] = ameths[--ea_ptr->method_count];
+              }
+              /* else leave alone */
+              handled = 1;
+              break;
+            }
           }
-		  if ( handled ) continue;
+          if ( handled ) continue;
           /* Not found on keep_edge, so paste onto keep_edge */
-		  if ( orientable && !same_sign(keep_edge,throw_edge) )
-		    ameths[ea_ptr->method_count++] = -bmeths[i];
-		  else
-			ameths[ea_ptr->method_count++] = bmeths[i];
+          if ( orientable && same_sign(keep_edge,throw_edge) )
+            ameths[ea_ptr->method_count++] = -bmeths[i];
+          else
+            ameths[ea_ptr->method_count++] = bmeths[i];
        } /* end b meth loop */
-	  } /* end method adjust */
+      } /* end method adjust */
 
         
 wasloop:
@@ -1646,9 +1686,12 @@ wasloop:
       /* free structures  */
       if ( !equal_element(a_edge,b_edge) ) 
         free_element(throw_edge);
-      if ( !valid_id(get_edge_fe(keep_edge)) ) 
+      if ( !valid_id(get_edge_fe(keep_edge)) )  // get rid of bare edge
         free_element(keep_edge);
-      if ( !equal_element(a,b) ) free_element(a);
+      if ( !valid_id(get_vertex_edge(third_v)) ) // get rid of bare vertex
+        free_element(third_v);
+      if ( !equal_element(a,b) ) 
+        free_element(a);
       else
       { /* get rid of loop edge */
         free_element(get_fe_edge(a));
@@ -1675,6 +1718,12 @@ wasloop:
 
   if ( !equal_id(elim_v,keep_v) )    /* see note above */
      free_element(elim_v);
+  if ( !valid_id(get_vertex_edge(keep_v)) ) // get rid of bare vertex
+  { free_element(keep_v);
+    retval = 1; 
+    goto eliminate_edge_exit;
+  }
+
 
       /* note: short_edge cannot be eliminated here since caller
           has to use it to continue edge generation.  Caller 
