@@ -25,8 +25,8 @@
 *              Does not add newline.
 */
 
-void outstring(outmsg)
-char *outmsg;
+void outstring ARGS1((outmsg),
+CONST char *outmsg)
 {
   if ( quiet_flag || !outmsg ) return;
   if ( broken_pipe_flag )
@@ -56,9 +56,11 @@ char *outmsg;
 *              Does not add newline.
 */
 
-void erroutstring(outmsg)
-char *outmsg;
+void erroutstring ARGS1((outmsg),
+char *outmsg)
 {
+  if ( !outmsg ) 
+	  return;
 #if defined(MAC_APP) || defined(WIN32S) || defined(MAC_CW)
   write_to_console(outmsg);
 #else
@@ -79,10 +81,10 @@ char *outmsg;
 * ends with null, included in max size.
 */
 
-char *my_fgets(s,n,stream)
-char *s;
-int n;  /* max size of string */
-FILE *stream; 
+char *my_fgets ARGS3((s,n,stream),
+char *s,
+int n,  /* max size of string */
+FILE *stream) 
 { char *p;
   int k;
   for ( k = 0, p = s ; k < n-1 ; k++,p++ )
@@ -110,9 +112,9 @@ FILE *stream;
 *  Purpose: Gets string from user. 
 */
 
-void getstring(inmsg,max)
-char *inmsg;
-int max;
+void getstring ARGS2((inmsg,max),
+char *inmsg,
+int max)
 { char *c;
   max -= 2; /* ensure room */
   while ( commandfd && (commandfd != stdin) )
@@ -123,7 +125,7 @@ int max;
     }
     else 
     {
-      if ( !topflag) outstring(inmsg); /* echo */
+      if ( !topflag && !quiet_load_flag ) outstring(inmsg); /* echo */
       if ( (int)strlen(inmsg) == max-1 ) 
       {inmsg[max-1] = MOREIN; inmsg[max] = 0; }
       return;
@@ -331,10 +333,10 @@ int check_pick()
 *  Purpose: Displays prompt and gets response from user.
 */
 
-int prompt(promptmsg,inmsg,max)
-char *promptmsg; /* to user */
-char *inmsg;    /* from user, space already allocated */
-int max; /* max char, including terminal null */
+int prompt ARGS3((promptmsg,inmsg,max),
+char *promptmsg, /* to user */
+char *inmsg,    /* from user, space already allocated */
+int max) /* max char, including terminal null */
 { char *c,*ptr;
   int oldquiet;
   
@@ -384,8 +386,10 @@ if ( threadflag )
     { return EOF; }
     else 
     { oldquiet = quiet_flag; /* quiet_flag = 0; */
-      outstring(promptmsg);  /* so appears on screen also */
-      outstring(inmsg); /* echo */
+      if ( !quiet_load_flag )
+      { outstring(promptmsg);  /* so appears on screen also */
+        outstring(inmsg); /* echo */
+      }
       if ( (c = strchr(inmsg,'\n')) != NULL ) 
       { if ( strcmp(promptmsg,"Enter command: ") != 0 )
           *c = 0;
@@ -464,9 +468,9 @@ if ( threadflag )
 *  purpose:  push new datafile include file on stack.
 */
 
-void push_datafd(cfd,name)
-FILE *cfd; /* NULL if not opened yet */
-char *name;  /* name of file */
+void push_datafd ARGS2((cfd,name),
+FILE *cfd, /* NULL if not opened yet */
+char *name)  /* name of file */
 {
   if ( include_depth >= NESTDEPTH-1 )
      kb_error(1352,"INCLUDEs nested too deeply.\n",DATAFILE_ERROR);
@@ -514,9 +518,9 @@ void pop_datafd()
 *  purpose:  push new command file on stack.
 */
 
-void push_commandfd(cfd,name)
-FILE *cfd; /* NULL if not opened yet */
-char *name;  /* name of file */
+void push_commandfd ARGS2((cfd,name),
+FILE *cfd, /* NULL if not opened yet */
+char *name)  /* name of file */
 {
   if ( (cfd==NULL) && (name==NULL) ) return;
   if ( read_depth >= NESTDEPTH-1 )
@@ -564,7 +568,19 @@ void pop_commandfd()
   if ( in_comment )
           { kb_error(2205,"End of file in comment\n",WARNING );  }
 #ifdef WIN32
-  if ( (commandfd == stdin) && !echo_flag ) return; /* Ctrl-C gives spurious EOF */
+  if ( commandfd == stdin )
+  { int tty = _isatty(_fileno(stdin));
+    if ( !tty ) 
+       my_exit(10); /* exit in case of redirected input */
+    else return; /* Ctrl-C gives spurious EOF */
+    /* Note:   SIGINT is not supported for any Win32 application, 
+       including Windows 98/Me and Windows NT/2000/XP. When a CTRL+C 
+       interrupt occurs, Win32 operating systems generate a new thread 
+       to specifically handle that interrupt. This can cause a 
+       single-thread application such as UNIX, to become multithreaded, 
+       resulting in unexpected behavior. (MSDN Library on SIGINT )
+       */
+ }
 #endif
   if ( datafile_input_flag & !cmdfile_stack[read_depth-1].datafile_flag )
   { datafile_input_flag = 0;
@@ -594,17 +610,43 @@ void pop_commandfd()
 * purpose: appends message to warning message list.
 */
 
-void add_warning_message(message)
-char *message;
-{ size_t needed = (warning_messages ? strlen(warning_messages):0) + strlen(message);
+#define WARNING_MESSAGES_MAX 100000
+
+void add_warning_message ARGS1((message),
+char *message)
+{ size_t needed = (warning_messages ? strlen(warning_messages):0) + strlen(message) + 10;
   if ( warning_messages_max < needed )
-  { warning_messages = (char*)kb_realloc(warning_messages,needed+1000);
-    warning_messages_max = needed+1000;
+  { if ( warning_messages_max < WARNING_MESSAGES_MAX )
+    { warning_messages = (char*)kb_realloc(warning_messages,needed+1000);
+      warning_messages_max = needed+1000;
+    }
+    else
+    { /* delete early messages */
+      char *c;
+      for ( c = warning_messages + 1000 ; *c != '\n' ; c++ ) ;
+      memcpy(warning_messages,c,warning_messages_max-(c-warning_messages));
+    }
   }
   strcat(warning_messages,message);
   warning_messages_new++;
 }
 
+#ifdef __cplusplus
+void do_throw ( int errnum )
+{
+  throw errnum;
+}
+void do_cmd_throw ( )
+{ struct cmd_excep c;
+  throw c;
+}
+void do_graph_throw ( )
+{ struct graph_excep c;
+  throw c;
+}
+
+#endif
+
 /***********************************************************************
 *
 *  Purpose: Error handling.  Prints error message.  If error is
@@ -613,10 +655,10 @@ char *message;
 *  May be system dependent for message display.
 */
 
-void kb_error(errnum,emsg,mode)
-int errnum; /* error identifier */
-char *emsg; /* might be msg or errmsg */
-int mode;
+void kb_error ARGS3((errnum,emsg,mode),
+int errnum, /* error identifier */
+char *emsg, /* might be msg or errmsg */
+int mode)
 {
   extern int line_no;
   char *fullmsg;
@@ -626,7 +668,8 @@ int mode;
 #ifdef MPI_EVOLVER
 fprintf(stderr,"Task %d: ",this_machine);
 #endif
- 
+
+
   if ( emsg == NULL ) emsg = "";  /* just in case */
   if ( emsg == errmsg ) { fullmsg = msg; size = msgmax; }
   else { fullmsg = errmsg; size = sizeof(errmsg); }
@@ -796,7 +839,11 @@ fprintf(stderr,"Task %d: ",this_machine);
         if ( exit_after_error ) my_exit(1);
         FPRESET;
         quiet_flag = 0;
+#ifdef __cplusplus
+        do_cmd_throw();
+#else
         longjmp(cmdbuf,1);
+#endif
         break;
 
      case PARSE_ERROR:
@@ -869,6 +916,7 @@ bailout:
   }
   clear_symtable();     /* clear symbol table */
   vgrad_end();          /* clear up volume gradients */
+  memset((char*)(&Met),0,sizeof(Met));
   #ifdef PTHREADS
   if ( pthread_equal(locking_thread,pthread_self()) )
   {  ABORT_GRAPH_MUTEX
@@ -890,12 +938,21 @@ bailout:
   ABORT_GRAPH_MUTEX;
 #ifdef WIN32
   if ( draw_thread_id == GetCurrentThreadId() )
-      longjmp(graphjumpbuf,1);
 #elif defined(PTHREADS)
   if ( draw_thread_id == pthread_self() ) 
+#endif
+#ifdef __cplusplus
+      do_graph_throw();
+#else
       longjmp(graphjumpbuf,1);
 #endif
+
+#ifdef __cplusplus
+  do_throw(errnum);
+#else
   longjmp(jumpbuf,1);
+#endif
+  
 }
  
 /**************************************************************************
@@ -909,8 +966,8 @@ bailout:
 *
 */
 
-void start_logfile(filename)
-char *filename;
+void start_logfile ARGS1((filename),
+char *filename)
 {
   char *name = filename ? filename : logfilename;
 
@@ -952,8 +1009,8 @@ void stop_logfile()
 *
 */
 
-void start_keylogfile(filename)
-char *filename;
+void start_keylogfile ARGS1((filename),
+char *filename)
 {
   char *name = filename ? filename : keylogfilename;
 
@@ -992,8 +1049,8 @@ void stop_keylogfile()
 * purpose: set output console scroll buffer size in Windows
 */
 
-void set_scroll_size(rows)
-int rows;
+void set_scroll_size ARGS1((rows),
+int rows)
 {
 #ifdef MSC
   COORD size;
