@@ -35,8 +35,10 @@ int mode;    /* VOLKEEP to keep volumes on both sides same */
   vertex_id v_id;
   int i;
  
-  if ( web.modeltype == LAGRANGE )
-     kb_error(1220,"No vertex averaging in Lagrange model yet.\n",RECOVERABLE );
+  #ifdef MPI_EVOLVER
+  if ( this_machine == 0 )
+    mpi_vertex_average(mode);
+  #endif
 
   if ( web.skel[VERTEX].count == 0 ) return;
 
@@ -44,19 +46,19 @@ int mode;    /* VOLKEEP to keep volumes on both sides same */
                                                   sizeof(struct averages));
   FOR_ALL_VERTICES(v_id)
    if ( get_vattr(v_id) & (FIXED|BOUNDARY) ) 
-     average[ordinal(v_id)].status = -10;
+     average[loc_ordinal(v_id)].status = -10;
 
   if ( web.representation == SIMPLEX ) 
     { old_vertex_average(mode); return; }
 
   FOR_ALL_VERTICES(v_id)
-     average[ordinal(v_id)].status = 
-              find_vertex_average(v_id,average[ordinal(v_id)].x,mode);
+     average[loc_ordinal(v_id)].status = 
+              find_vertex_average(v_id,average[loc_ordinal(v_id)].x,mode);
 
   FOR_ALL_VERTICES(v_id)
   {
      int attr = get_vattr(v_id);
-     REAL *newx = average[ordinal(v_id)].x;
+     REAL *newx = average[loc_ordinal(v_id)].x;
      x = get_coord(v_id);
      if (attr & BOUNDARY )
      { /* update boundary parameter to agree with coordinates */
@@ -103,7 +105,7 @@ int mode;    /* VOLKEEP to keep volumes on both sides same */
   average = (struct averages *)temp_calloc(web.skel[VERTEX].max_ord+1,
                                                   sizeof(struct averages));
   FOR_ALL_VERTICES(v_id)
-      if ( get_vattr(v_id) & (FIXED|BOUNDARY) ) average[ordinal(v_id)].status = -10;
+      if ( get_vattr(v_id) & (FIXED|BOUNDARY) ) average[loc_ordinal(v_id)].status = -10;
 
   if ( vedge_timestamp < top_timestamp ) make_vedge_lists();
 
@@ -130,7 +132,7 @@ int mode;    /* VOLKEEP to keep volumes on both sides same */
   { FOR_ALL_VERTICES(v_id)
     {
       if ( get_vattr(v_id) & (FIXED|BOUNDARY) ) continue;
-      ave = average + ordinal(v_id);
+      ave = average + loc_ordinal(v_id);
       if (ave->status <= 0 ) continue;
       if (ave->area == 0.0 ) continue;
       if (ave->triples == 1 ) continue;
@@ -178,7 +180,7 @@ int mode;    /* VOLKEEP to keep volumes on both sides same */
   else /* string */
   { FOR_ALL_VERTICES(v_id)
     {
-      ave = average + ordinal(v_id);
+      ave = average + loc_ordinal(v_id);
       if (ave->status != 2 ) continue; /* too many edges or something */
       x = get_coord(v_id);
 
@@ -253,8 +255,8 @@ int mode; /* averaging mode */
       struct averages *head,*tail;
 
       k++; /* instead of maybe being skipped by continue at end of loop */
-      head = average + ordinal(headv);
-      tail = average + ordinal(tailv);
+      head = average + loc_ordinal(headv);
+      tail = average + loc_ordinal(tailv);
 
       if ( mode != RAWEST )
       { /* check constraint compatibility */
@@ -401,7 +403,7 @@ int mode; /* averaging mode */
   get_edge_side(e_id,side);  /* in case of wraps */
 
   /* head */
-  head = average + ordinal(headv);
+  head = average + loc_ordinal(headv);
   attr = get_vattr(headv);
   if ( attr & FIXED ) goto dotail;
   if (  mode != RAWEST )
@@ -437,7 +439,7 @@ int mode; /* averaging mode */
   head->status++;  /* mark one more edge done */
 
 dotail:
-  tail = average + ordinal(tailv);
+  tail = average + loc_ordinal(tailv);
 
   attr = get_vattr(tailv);
   if ( attr & FIXED ) return;
@@ -499,7 +501,7 @@ int mode;
   { REAL *x = get_coord(v[i]);
 
     for ( j = 0 ; j <= web.dimension ; j++ )
-    { head = average + ordinal(v[j]);
+    { head = average + loc_ordinal(v[j]);
       if ( mode != RAWEST )
       { conmap_t *conmapj = get_v_constraint_map(v[j]);
         for ( k = 1 ; k <= conmapj[0] ; k++ )
@@ -584,6 +586,9 @@ int mode;    /* VOLKEEP to keep volumes on both sides same */
   oldx = get_coord(v_id);
   for ( i = 0 ; i < SDIM ; i++ ) vx[i] = oldx[i];
 
+  if ( web.modeltype == LAGRANGE )
+     return lagrange_vertex_average(v_id,vx);
+
   if ( attr & (FIXED|AXIAL_POINT) ) return 0;
 
   if ( vedge_timestamp < top_timestamp ) make_vedge_lists();
@@ -618,11 +623,11 @@ int mode;    /* VOLKEEP to keep volumes on both sides same */
   }
   else if ( attr & Q_MIDFACET )
   { /* Lagrange model middle of facet */
-     return 0;
+     return lagrange_vertex_average(v_id,vx);
   }
   else if ( attr & Q_MIDEDGE )
   { /* Lagrange model middle of edge */
-     return 0;
+     return lagrange_vertex_average(v_id,vx);
   }
   else
   { /* corner vertex, average with neighbors */
@@ -691,7 +696,13 @@ int mode;    /* VOLKEEP to keep volumes on both sides same */
       { fe = start_fe = get_edge_fe(e_id);
         weight = 0.0;
         if ( valid_id(fe) )
-          do { weight += get_facet_area(get_fe_facet(fe));
+          do { facet_id f_id = get_fe_facet(fe);
+		       REAL a = get_facet_area(f_id);
+		       if ( a == 0.0 ) 
+			   { (*calc_facet_energy)(f_id,AREA_ONLY);
+			      a = get_facet_area(f_id);
+			   }
+		       weight += a;
                fe = get_next_facet(fe);
              } while ( !equal_id(fe,start_fe) );
       }
@@ -753,3 +764,146 @@ new_avg_exit:
   return 1;
 }
 
+/******************************************************************************
+ * 
+ * function: lagrange_vertex_average()
+ *
+ * purpose: Do vertex averaging for a vertex in the Lagrange model.
+ */
+
+int lagrange_vertex_average(v_id,newx)
+vertex_id v_id;
+REAL *newx;
+{ int attr = get_vattr(v_id);
+  int i,j,k;
+  
+  if ( attr & Q_MIDEDGE )
+  { /* average with neighbors on edge */
+    edge_id e_id = get_vertex_edge(v_id);
+    vertex_id *v = get_edge_vertices(e_id);
+    int knx; /* index of vertex */
+    vertex_id vv[2];  /* the two neighbors to average with */
+    REAL perturb[MAXCOORD];
+    REAL dxdu[MAXCOORD],u,du,rhs,*x;
+    struct gauss_lag *g = &gauss_lagrange[1][web.gauss1D_order];
+    int ctrl = web.lagrange_order+1;
+    REAL aa;
+    REAL *xx[MAXLAGRANGE+1];
+
+    /* find which vertex it is in the edge */
+    for ( knx = 0 ; knx <= web.lagrange_order ; knx++ )
+      if ( equal_id(v_id,v[knx]) )
+        goto edge_found_knx;
+    kb_error(3789,"Internal error:lagrange_vertex_average() couldn't find vertex index in edge.\n",
+      RECOVERABLE);
+         
+ edge_found_knx:
+    /* the two neighbors */
+    vv[0] = v[knx-1] ;
+    vv[1] = v[knx+1];
+
+    /* find perturbation in u,v so can calculate new position exactly on 
+       the surface */
+    /* first, the xyz perturbation */
+    memset((char*)newx,0,SDIM*sizeof(REAL));
+    for ( i = 0 ; i < 2 ; i++ )
+    { REAL *xx = get_coord(vv[i]);
+      for ( j = 0 ; j < SDIM ; j++ )
+        newx[j] += xx[j];
+    }
+    x = get_coord(v_id);
+    for ( j = 0 ; j < SDIM ; j++ )
+      perturb[j] = newx[j]/2 - x[j];
+
+    /* get partials matrix */
+    for(j=0;j<SDIM;j++) dxdu[j] = 0.0;
+    for ( k = 0 ; k < ctrl ; k++ )
+    { xx[k] = get_coord(v[k]);
+      for ( j = 0 ; j < SDIM ; j++ )
+      { dxdu[j] += g->lpolypart[knx][0][k]*xx[k][j];
+      }
+    }
+    rhs = dot(perturb,dxdu,SDIM);
+    aa =  dot(dxdu,dxdu,SDIM);
+    du = rhs/aa;
+    u = (knx + du)/web.lagrange_order;
+    lagrange_eval_1d(web.lagrange_order,SDIM,u,xx,newx);
+    /* newx holds return value */
+     
+    return 1;
+  }
+  else if ( attr & Q_MIDFACET )
+  { facet_id f_id = get_vertex_facet(v_id);
+    vertex_id *v = get_facet_vertices(f_id);
+    int inx=0,jnx,knx; /* col, row, index of vertex */
+    vertex_id vv[6];  /* the six neighbors to average with */
+    MAT2D(dxdu,MAXCOORD,2);
+    REAL *xx[(MAXLAGRANGE+1)*(MAXLAGRANGE+2)/2];
+    MAT2D(aa,2,2);
+    REAL perturb[MAXCOORD], *p=perturb;
+    REAL u[2],du[2],rhs[2],*r=rhs,*x;
+    struct gauss_lag *g = &gauss_lagrange[2][web.gauss2D_order];
+    int ctrl = (web.lagrange_order+1)*(web.lagrange_order+2)/2;
+
+    /* find which vertex it is in the facet */
+    for ( jnx = 0, knx = 0 ; jnx <= web.lagrange_order ; jnx++ )
+      for ( inx = 0 ; inx+jnx <= web.lagrange_order ; inx++,knx++ )
+        if ( equal_id(v_id,v[knx]) )
+          goto found_knx;
+    kb_error(3788,"Internal error:lagrange_vertex_average() couldn't find vertex index in facet.\n",
+      RECOVERABLE);
+         
+ found_knx:
+    /* the six neighbors */
+    vv[0] = v[knx - web.lagrange_order + jnx - 2];
+    vv[1] = v[knx - web.lagrange_order + jnx - 1];
+    vv[2] = v[knx - 1];
+    vv[3] = v[knx + 1];
+    vv[4] = v[knx + web.lagrange_order - jnx];
+    vv[5] = v[knx + web.lagrange_order - jnx + 1];
+
+    /* find perturbation in u,v so can calculate new position exactly on 
+       the surface */
+    /* first, the xyz perturbation */
+    memset((char*)newx,0,SDIM*sizeof(REAL));
+    for ( i = 0 ; i < 6 ; i++ )
+    { REAL *xx = get_coord(vv[i]);
+      for ( j = 0 ; j < SDIM ; j++ )
+        newx[j] += xx[j];
+    }
+    x = get_coord(v_id);
+    for ( j = 0 ; j < SDIM ; j++ )
+      perturb[j] = newx[j]/6 - x[j];
+
+    /* get partials matrix */
+    for(j=0;j<SDIM;j++) dxdu[j][0] = dxdu[j][1] = 0.0;
+    for ( k = 0 ; k < ctrl ; k++ )
+    { xx[k] = get_coord(v[k]);
+      for ( j = 0 ; j < SDIM ; j++ )
+      { dxdu[j][0] += g->lpolypart[knx][0][k]*xx[k][j];
+        dxdu[j][1] += g->lpolypart[knx][1][k]*xx[k][j];
+      }
+    }
+    /* kludging due to way lpolypart defined in terms of barycentric */
+    for ( j = 0  ; j < SDIM  ; j++ )
+    { dxdu[j][0] -= dxdu[j][1];
+      dxdu[j][1] += dxdu[j][0];
+    }
+    /* Least squares solve for du */
+    mat_mult(&p,dxdu,&r,1,SDIM,2);
+    tr_mat_mul(dxdu,dxdu,aa,SDIM,2,2);
+    mat_inv(aa,2);
+    matvec_mul(aa,rhs,du,2,2);
+    u[0] = (inx + du[0])/web.lagrange_order;
+    u[1] = (jnx + du[1])/web.lagrange_order;
+    lagrange_eval_2d(web.lagrange_order,SDIM,u,xx,newx);
+    /* newx holds return value */
+     
+    return 1;
+  }
+  else /* corner */
+  { /* not doing anything yet */
+    return 0;
+  }
+
+}
