@@ -829,6 +829,37 @@ newpop_exit:
   return 1;
 }
 
+/************************************************************************************
+    Autopop and autochop functions.
+********************************************************************************/
+edge_id *autopop_list;  /* list of short edges found */
+edge_id *autochop_list;  /* list of long edges found */
+
+void autopop_init()
+{
+  if ( autopop_flag )
+  {
+    autopop_list = (edge_id *)temp_calloc(web.skel[EDGE].count,sizeof(edge_id));
+  
+  }
+  if ( autochop_flag )
+  {
+    autochop_list = (edge_id *)temp_calloc(web.skel[EDGE].count,sizeof(edge_id));
+  }
+}
+
+void autopop_cleanup()
+{ if ( autochop_list )
+  { temp_free((char*)autochop_list);
+    autochop_list = NULL;
+  }
+  if ( autopop_list )
+  { temp_free((char*)autopop_list);
+    autopop_list = NULL;
+  }
+}
+
+
 /********************************************************************
 *
 *  function: autopop_detect()
@@ -841,25 +872,13 @@ newpop_exit:
 *
 */
 
-edge_id *autopop_list;  /* list of edges found */
-edge_id *autochop_list;  /* list of edges found */
-
 void autopop_detect(scale)
 REAL scale;  /* scale factor for motion */
 {
   edge_id e_id;
   REAL minlength = sqrt(2*scale); /* stability critical length */
-
-  if ( autopop_flag )
-  {
-    autopop_list = (edge_id *)temp_calloc(web.skel[EDGE].count,sizeof(edge_id));
-    autopop_count = 0;
-  }
-  if ( autochop_flag )
-  {
-    autochop_list = (edge_id *)temp_calloc(web.skel[EDGE].count,sizeof(edge_id));
-    autochop_count = 0;
-  }
+  autopop_count = 0;
+  autochop_count = 0;  
   FOR_ALL_EDGES(e_id)
     {
         vertex_id headv = get_edge_headv(e_id);
@@ -890,7 +909,8 @@ REAL scale;  /* scale factor for motion */
              autochop_list[autochop_count++] = e_id;
           }
     }
-}
+
+} 
 
 /***********************************************************************
 *
