@@ -10,6 +10,10 @@
 * variables needed to export for distributed computing.
 */
 
+#ifdef  __cplusplus
+extern "C" {
+#endif
+
 /* structure type name different from structure variable name since
    Visual C is incompetent at distinguishing in debugger. */
 
@@ -17,6 +21,10 @@ struct webstruct {
      struct skeleton skel[NUMELEMENTS];
      int sizes[NUMELEMENTS];  /* allocated space for element structure */
      int usedsizes[NUMELEMENTS];  /* used space for element structure */
+     struct element **elhashtable; /* id hash list of element pointers */
+     int elhashcount;  /* actual number of live entries */
+     int elhashmask;  /* for picking off index bits of id hash */
+     int elhashsize;  /* size of hash table; power of 2 */
      int sdim;  /* dimension of ambient space */
      int dimension;    /* where tension resides */
      int representation; /* STRING, SOAPFILM, or SIMPLEX */
@@ -107,6 +115,8 @@ struct webstruct {
      int edge_dissolve_count; 
      int facet_dissolve_count; 
      int body_dissolve_count; 
+     int edge_reverse_count;
+     int facet_reverse_count;
      int vertex_pop_count; 
      int edge_pop_count; 
      int pop_tri_to_edge_count;
@@ -144,6 +154,8 @@ struct webstruct {
      #define fix_count_bit 0x00080000 
      #define unfix_count_bit 0x00100000 
      #define t1_edgeswap_count_bit 0x00200000 
+     #define edge_reverse_count_bit 0x00400000
+     #define facet_reverse_count_bit 0x00800000
      
 
      /* here follows stuff moved from independent globals to inside web
@@ -184,3 +196,7 @@ struct webstruct {
 
 extern struct webstruct web;
 
+
+#ifdef  __cplusplus
+}
+#endif
