@@ -263,23 +263,29 @@
 #define	GO_COUNT_	517
 #define	ELEMENT_IDENT_	518
 #define	BODY_METIS_	519
-#define	ASSIGNOP_	520
-#define	PIPE_	521
-#define	THEN_	522
-#define	ELSE_	523
-#define	OR_	524
-#define	AND_	525
-#define	NOT_	526
-#define	EQ_	527
-#define	LE_	528
-#define	GE_	529
-#define	NE_	530
-#define	ON_CONSTRAINT_	531
-#define	HIT_CONSTRAINT_	532
-#define	ON_BOUNDARY_	533
-#define	ON_QUANTITY_	534
-#define	ON_METHOD_INSTANCE_	535
-#define	IMOD_	536
-#define	IDIV_	537
-#define	EPRINT_	538
+#define	REVERSE_ORIENTATION_	520
+#define	MATRIX_MULTIPLY_	521
+#define	MATRIX_INVERSE_	522
+#define	BINARY_PRINTF_	523
+#define	DUMP_MEMLIST_	524
+#define	FREE_DISCARDS_	525
+#define	ASSIGNOP_	526
+#define	PIPE_	527
+#define	THEN_	528
+#define	ELSE_	529
+#define	OR_	530
+#define	AND_	531
+#define	NOT_	532
+#define	EQ_	533
+#define	LE_	534
+#define	GE_	535
+#define	NE_	536
+#define	ON_CONSTRAINT_	537
+#define	HIT_CONSTRAINT_	538
+#define	ON_BOUNDARY_	539
+#define	ON_QUANTITY_	540
+#define	ON_METHOD_INSTANCE_	541
+#define	IMOD_	542
+#define	IDIV_	543
+#define	EPRINT_	544
 
